@@ -4,6 +4,7 @@ Contains library functions
 import json
 import logging
 import os.path
+from typing import Optional
 
 from markupsafe import escape
 
@@ -11,8 +12,11 @@ from galaxy import util
 from galaxy.exceptions import (
     AdminRequiredException,
     ConfigDoesNotAllowException,
+    ItemAccessibilityException,
+    ObjectNotFound,
     RequestParameterInvalidException,
 )
+from galaxy.model import LibraryDataset
 from galaxy.tools.actions import upload_common
 from galaxy.tools.parameters import populate_state
 from galaxy.util.path import (
@@ -70,7 +74,7 @@ class LibraryActions:
     Mixin for controllers that provide library functionality.
     """
 
-    def _upload_dataset(self, trans, library_id, folder_id, replace_dataset=None, **kwd):
+    def _upload_dataset(self, trans, folder_id: str, replace_dataset: Optional[LibraryDataset] = None, **kwd):
         # Set up the traditional tool state/params
         cntrller = 'api'
         tool_id = 'upload1'
@@ -85,7 +89,7 @@ class LibraryActions:
         populate_state(trans, tool.inputs, kwd, state.inputs)
         tool_params = state.inputs
         dataset_upload_inputs = []
-        for input_name, input in tool.inputs.items():
+        for input in tool.inputs.values():
             if input.type == "upload_dataset":
                 dataset_upload_inputs.append(input)
         # Library-specific params
@@ -126,8 +130,7 @@ class LibraryActions:
         job_params['link_data_only'] = json.dumps(kwd.get('link_data_only', 'copy_files'))
         job_params['uuid'] = json.dumps(kwd.get('uuid', None))
         job, output = upload_common.create_job(trans, tool_params, tool, json_file_path, data_list, folder=library_bunch.folder, job_params=job_params)
-        trans.sa_session.add(job)
-        trans.sa_session.flush()
+        trans.app.job_manager.enqueue(job, tool=tool)
         return output
 
     def _get_server_dir_uploaded_datasets(self, trans, params, full_dir, import_dir_desc, library_bunch, response_code, message):
@@ -169,11 +172,11 @@ class LibraryActions:
                 if os.path.isfile(path):
                     files.append(path)
         except Exception as e:
-            message = "Unable to get file list for configured {}, error: {}".format(import_dir_desc, util.unicodify(e))
+            message = f"Unable to get file list for configured {import_dir_desc}, error: {util.unicodify(e)}"
             response_code = 500
             return None, response_code, message
         if not files:
-            message = "The directory '%s' contains no valid files" % full_dir
+            message = f"The directory '{full_dir}' contains no valid files"
             response_code = 400
             return None, response_code, message
         return files, None, None
@@ -203,7 +206,7 @@ class LibraryActions:
         if os.path.isfile(path):
             name = os.path.basename(path)
             files_and_folders.append((path, name, None))
-        for basedir, dirs, files in os.walk(line):
+        for basedir, _dirs, files in os.walk(line):
             for file in files:
                 file_path = os.path.abspath(os.path.join(basedir, file))
                 if preserve_dirs:
@@ -214,7 +217,7 @@ class LibraryActions:
         return files_and_folders
 
     def _paths_list(self, params):
-        return [(l.strip(), os.path.abspath(l.strip())) for l in params.get('filesystem_paths', '').splitlines() if l.strip()]
+        return [(line.strip(), os.path.abspath(line.strip())) for line in params.get('filesystem_paths', '').splitlines() if line.strip()]
 
     def _check_path_paste_params(self, params):
         if params.get('filesystem_paths', '') == '':
@@ -292,20 +295,19 @@ class LibraryActions:
         return 200, dict(created=new_folder)
 
     def _check_access(self, trans, is_admin, item, current_user_roles):
-        can_access = True
         if isinstance(item, trans.model.HistoryDatasetAssociation):
             # Make sure the user has the DATASET_ACCESS permission on the history_dataset_association.
             if not item:
-                message = "Invalid history dataset (%s) specified." % escape(str(item))
-                can_access = False
+                message = f"Invalid history dataset ({escape(str(item))}) specified."
+                raise ObjectNotFound(message)
             elif not trans.app.security_agent.can_access_dataset(current_user_roles, item.dataset) and item.history.user == trans.user:
-                message = "You do not have permission to access the history dataset with id (%s)." % str(item.id)
-                can_access = False
+                message = f"You do not have permission to access the history dataset with id ({str(item.id)})."
+                raise ItemAccessibilityException(message)
         else:
             # Make sure the user has the LIBRARY_ACCESS permission on the library item.
             if not item:
-                message = "Invalid library item (%s) specified." % escape(str(item))
-                can_access = False
+                message = f"Invalid library item ({escape(str(item))}) specified."
+                raise ObjectNotFound(message)
             elif not (is_admin or trans.app.security_agent.can_access_library_item(current_user_roles, item, trans.user)):
                 if isinstance(item, trans.model.Library):
                     item_type = 'data library'
@@ -313,13 +315,11 @@ class LibraryActions:
                     item_type = 'folder'
                 else:
                     item_type = '(unknown item type)'
-                message = "You do not have permission to access the {} with id ({}).".format(escape(item_type), str(item.id))
-                can_access = False
-        if not can_access:
-            return 400, message
+                message = f"You do not have permission to access the {escape(item_type)} with id ({str(item.id)})."
+                raise ItemAccessibilityException(message)
 
     def _check_add(self, trans, is_admin, item, current_user_roles):
         # Deny access if the user is not an admin and does not have the LIBRARY_ADD permission.
         if not (is_admin or trans.app.security_agent.can_add_library_item(current_user_roles, item)):
-            message = "You are not authorized to add an item to (%s)." % escape(item.name)
-            return 403, message
+            message = f"You are not authorized to add an item to ({escape(item.name)})."
+            raise ItemAccessibilityException(message)
