@@ -6,11 +6,12 @@ from os.path import (
 )
 
 from galaxy import util
+from galaxy.job_execution.output_collect import default_exit_code_file
 from galaxy.jobs.runners.util.job_script import (
-    check_script_integrity,
     INTEGRITY_INJECTION,
     write_script,
 )
+from galaxy.tool_util.deps.container_classes import TRAP_KILL_CONTAINER
 
 log = getLogger(__name__)
 
@@ -36,10 +37,9 @@ def build_command(
     include_metadata=False,
     include_work_dir_outputs=True,
     create_tool_working_directory=True,
-    remote_command_params={},
+    remote_command_params=None,
     remote_job_directory=None,
-    stdout_file=None,
-    stderr_file=None,
+    stream_stdout_stderr=False,
 ):
     """
     Compose the sequence of commands necessary to execute a job. This will
@@ -50,27 +50,20 @@ def build_command(
         - command line taken from job wrapper
         - commands to set metadata (if include_metadata is True)
     """
+    remote_command_params = remote_command_params or {}
     shell = job_wrapper.shell
     base_command_line = job_wrapper.get_command_line()
     # job_id = job_wrapper.job_id
     # log.debug( 'Tool evaluation for job (%s) produced command-line: %s' % ( job_id, base_command_line ) )
-    if not base_command_line:
-        raise Exception("Attempting to run a tool with empty command definition.")
 
     commands_builder = CommandsBuilder(base_command_line)
 
-    # All job runners currently handle this case which should never occur
-    if not commands_builder.commands:
-        return None
-
-    # Version, dependency resolution, and task splitting are prepended to the
+    # Dependency resolution and task splitting are prepended to the
     # command - so they need to appear in the following order to ensure that
     # the underlying application used by version command is available in the
     # environment after dependency resolution, but the task splitting command
     # is still executed in Galaxy's Python environment.
 
-    __handle_version_command(commands_builder, job_wrapper)
-
     # One could imagine also allowing dependencies inside of the container but
     # that is too sophisticated for a first crack at this - build your
     # containers ready to go!
@@ -100,6 +93,10 @@ def build_command(
         else:
             commands_builder = CommandsBuilder(externalized_commands)
 
+    for_pulsar = 'script_directory' in remote_command_params
+    if not for_pulsar:
+        commands_builder.capture_stdout_stderr('../outputs/tool_stdout', '../outputs/tool_stderr', stream_stdout_stderr=stream_stdout_stderr)
+
     # Don't need to create a separate tool working directory for Pulsar
     # jobs - that is handled by Pulsar.
     if create_tool_working_directory:
@@ -110,20 +107,20 @@ def build_command(
         # xref https://github.com/galaxyproject/galaxy/issues/3289
         commands_builder.prepend_command(PREPARE_DIRS)
 
+    __handle_remote_command_line_building(commands_builder, job_wrapper, for_pulsar=for_pulsar)
+
     container_monitor_command = job_wrapper.container_monitor_command(container)
     if container_monitor_command:
         commands_builder.prepend_command(container_monitor_command)
 
+    working_directory = remote_job_directory or job_wrapper.working_directory
+    commands_builder.capture_return_code(default_exit_code_file(working_directory, job_wrapper.job_id))
+
     if include_work_dir_outputs:
         __handle_work_dir_outputs(commands_builder, job_wrapper, runner, remote_command_params)
 
-    if stdout_file and stderr_file:
-        commands_builder.capture_stdout_stderr(stdout_file, stderr_file)
-    commands_builder.capture_return_code()
-
     if include_metadata and job_wrapper.requires_setting_metadata:
-        working_directory = remote_job_directory or job_wrapper.working_directory
-        commands_builder.append_command("cd '%s'" % working_directory)
+        commands_builder.append_command(f"cd '{working_directory}'")
         __handle_metadata(commands_builder, job_wrapper, runner, remote_command_params)
 
     return commands_builder.build()
@@ -132,14 +129,13 @@ def build_command(
 def __externalize_commands(job_wrapper, shell, commands_builder, remote_command_params, script_name="tool_script.sh", container=None):
     local_container_script = join(job_wrapper.working_directory, script_name)
     tool_commands = commands_builder.build()
-    config = job_wrapper.app.config
     integrity_injection = ""
     # Setting shell to none in job_conf.xml disables creating a tool command script,
     # set -e doesn't work for composite commands but this is necessary for Windows jobs
     # for instance.
     if shell and shell.lower() == 'none':
         return tool_commands
-    if check_script_integrity(config):
+    if job_wrapper.job_io.check_job_script_integrity:
         integrity_injection = INTEGRITY_INJECTION
     set_e = ""
     if job_wrapper.strict_shell:
@@ -154,8 +150,12 @@ def __externalize_commands(job_wrapper, shell, commands_builder, remote_command_
         source_command,
         tool_commands,
     )
-    write_script(local_container_script, script_contents, config)
-    commands = "{} {}".format(shell, local_container_script)
+    write_script(
+        local_container_script,
+        script_contents,
+        job_io=job_wrapper.job_io,
+    )
+    commands = f"{shell} {local_container_script}"
     # TODO: Cleanup for_pulsar hack.
     # - Integrate Pulsar sending tool_stdout/tool_stderr back
     #   https://github.com/galaxyproject/pulsar/pull/202
@@ -164,21 +164,22 @@ def __externalize_commands(job_wrapper, shell, commands_builder, remote_command_
     #   doesn't need to mount the job directory (rw) and then eliminate this hack
     #   (or restrict to older Pulsar versions).
     #   https://github.com/galaxyproject/galaxy/pull/8449
-    for_pulsar = False
-    if 'script_directory' in remote_command_params:
-        commands = "{} {}".format(shell, join(remote_command_params['script_directory'], script_name))
-        for_pulsar = True
-    if not for_pulsar:
-        commands += " > ../outputs/tool_stdout 2> ../outputs/tool_stderr"
-    log.info("Built script [{}] for tool command [{}]".format(local_container_script, tool_commands))
+    for_pulsar = 'script_directory' in remote_command_params
+    if for_pulsar:
+        commands = f"{shell} {join(remote_command_params['script_directory'], script_name)}"
+    log.info(f"Built script [{local_container_script}] for tool command [{tool_commands}]")
     return commands
 
 
-def __handle_version_command(commands_builder, job_wrapper):
-    # Prepend version string
-    write_version_cmd = job_wrapper.write_version_cmd
-    if write_version_cmd:
-        commands_builder.prepend_command(write_version_cmd)
+def __handle_remote_command_line_building(commands_builder, job_wrapper, for_pulsar=False):
+    if getattr(job_wrapper, 'remote_command_line', False):
+        sep = "" if for_pulsar else "&&"
+        command = 'PYTHONPATH="$GALAXY_LIB:$PYTHONPATH" python "$GALAXY_LIB"/galaxy/tools/remote_tool_eval.py'
+        if for_pulsar:
+            # TODO: that's not how to do this, pulsar doesn't execute an externalized script by default.
+            # This also breaks rewriting paths etc, so it doesn't really work if there are no shared paths
+            command = f"{command} && bash ../tool_script.sh"
+        commands_builder.prepend_command(command, sep=sep)
 
 
 def __handle_task_splitting(commands_builder, job_wrapper):
@@ -201,7 +202,6 @@ def __handle_work_dir_outputs(commands_builder, job_wrapper, runner, remote_comm
         work_dir_outputs_kwds['job_working_directory'] = remote_command_params['working_directory']
     work_dir_outputs = runner.get_work_dir_outputs(job_wrapper, **work_dir_outputs_kwds)
     if work_dir_outputs:
-        commands_builder.capture_return_code()
         copy_commands = map(__copy_if_exists_command, work_dir_outputs)
         commands_builder.append_commands(copy_commands)
 
@@ -213,7 +213,7 @@ def __handle_metadata(commands_builder, job_wrapper, runner, remote_command_para
     exec_dir = metadata_kwds.get('exec_dir', abspath(getcwd()))
     tmp_dir = metadata_kwds.get('tmp_dir', job_wrapper.working_directory)
     dataset_files_path = metadata_kwds.get('dataset_files_path', runner.app.model.Dataset.file_path)
-    output_fnames = metadata_kwds.get('output_fnames', job_wrapper.get_output_fnames())
+    output_fnames = metadata_kwds.get('output_fnames', job_wrapper.job_io.get_output_fnames())
     config_root = metadata_kwds.get('config_root', None)
     config_file = metadata_kwds.get('config_file', None)
     datatypes_config = metadata_kwds.get('datatypes_config', None)
@@ -236,14 +236,15 @@ def __handle_metadata(commands_builder, job_wrapper, runner, remote_command_para
     metadata_command = metadata_command.strip()
     if metadata_command:
         # Place Galaxy and its dependencies in environment for metadata regardless of tool.
-        metadata_command = "{}{}".format(SETUP_GALAXY_FOR_METADATA, metadata_command)
-        commands_builder.capture_return_code()
+        metadata_command = f"{SETUP_GALAXY_FOR_METADATA}{metadata_command}"
         commands_builder.append_command(metadata_command)
 
 
 def __copy_if_exists_command(work_dir_output):
     source_file, destination = work_dir_output
-    return "if [ -f {} ] ; then cp {} {} ; fi".format(source_file, source_file, destination)
+    if '?' in source_file or '*' in source_file:
+        source_file = source_file.replace('*', '"*"').replace('?', '"?"')
+    return f'\nif [ -f "{source_file}" ] ; then cp "{source_file}" "{destination}" ; fi'
 
 
 class CommandsBuilder:
@@ -251,7 +252,8 @@ class CommandsBuilder:
     def __init__(self, initial_command=''):
         # Remove trailing semi-colon so we can start hacking up this command.
         # TODO: Refactor to compose a list and join with ';', would be more clean.
-        initial_command = util.unicodify(initial_command)
+        self.raw_command = initial_command
+        initial_command = util.unicodify(initial_command or '')
         commands = initial_command.rstrip("; ")
         self.commands = commands
 
@@ -262,9 +264,7 @@ class CommandsBuilder:
 
     def prepend_command(self, command, sep=";"):
         if command:
-            self.commands = "{}{} {}".format(command,
-                                         sep,
-                                         self.commands)
+            self.commands = f"{command}{sep} {self.commands}"
         return self
 
     def prepend_commands(self, commands):
@@ -272,29 +272,34 @@ class CommandsBuilder:
 
     def append_command(self, command, sep=';'):
         if command:
-            self.commands = "{}{} {}".format(self.commands,
-                                          sep,
-                                          command)
+            self.commands = f"{self.commands}{sep} {command}"
         return self
 
     def append_commands(self, commands):
         self.append_command("; ".join(c for c in commands if c))
 
-    def capture_stdout_stderr(self, stdout_file, stderr_file):
-        self.prepend_command("""out="${TMPDIR:-/tmp}/out.$$" err="${TMPDIR:-/tmp}/err.$$"
-mkfifo "$out" "$err"
-trap 'rm "$out" "$err"' EXIT
-tee -a stdout.log < "$out" &
-tee -a stderr.log < "$err" >&2 &""",
-                             sep="")
-        self.append_command("> '{stdout_file}' 2> '{stderr_file}'".format(stdout_file=stdout_file,
-                                                                          stderr_file=stderr_file),
-                            sep="")
+    def capture_stdout_stderr(self, stdout_file, stderr_file, stream_stdout_stderr=False):
+        if not stream_stdout_stderr:
+            self.append_command(f"> '{stdout_file}' 2> '{stderr_file}'", sep="")
+            return
+        trap_command = """trap 'rm -f "$__out" "$__err"' EXIT"""
+        if TRAP_KILL_CONTAINER in self.commands:
+            # We need to replace the container kill trap with one that removes the named pipes and kills the container
+            self.commands = self.commands.replace(TRAP_KILL_CONTAINER, "")
+            trap_command = """trap 'rm -f "$__out" "$__err"; _on_exit' EXIT"""
+        self.prepend_command(
+            f"""__out="${{TMPDIR:-.}}/out.$$" __err="${{TMPDIR:-.}}/err.$$"
+mkfifo "$__out" "$__err"
+{trap_command}
+tee -a '{stdout_file}' < "$__out" &
+tee -a '{stderr_file}' < "$__err" >&2 &""",
+            sep="")
+        self.append_command('> "$__out" 2> "$__err"', sep="")
 
-    def capture_return_code(self):
-        if not self.return_code_captured:
-            self.return_code_captured = True
-            self.append_command(CAPTURE_RETURN_CODE)
+    def capture_return_code(self, exit_code_path):
+        self.append_command(CAPTURE_RETURN_CODE)
+        self.append_command(f"echo $return_code > {exit_code_path}")
+        self.return_code_captured = True
 
     def build(self):
         if self.return_code_captured:
