@@ -1,10 +1,11 @@
 import importlib
 import logging
+from inspect import getfullargspec
+from types import ModuleType
 
 import galaxy.jobs.rules
 from galaxy.jobs import stock_rules
 from galaxy.jobs.dynamic_tool_destination import map_tool_to_destination
-from galaxy.util.getargspec import getfullargspec
 from galaxy.util.submodules import import_submodules
 from .rule_helper import RuleHelper
 
@@ -48,6 +49,7 @@ class JobRunnerMapper:
     This class is responsible to managing the mapping of jobs
     (in the form of job_wrappers) to job runner url strings.
     """
+    rules_module: ModuleType
 
     def __init__(self, job_wrapper, url_to_destination, job_config):
         self.job_wrapper = job_wrapper
@@ -197,7 +199,7 @@ class JobRunnerMapper:
         elif expand_type in STOCK_RULES:
             expand_function = STOCK_RULES[expand_type]
         else:
-            raise JobMappingConfigurationException("Unhandled dynamic job runner type specified - %s" % expand_type)
+            raise JobMappingConfigurationException(f"Unhandled dynamic job runner type specified - {expand_type}")
 
         return self.__handle_rule(expand_function, destination)
 
@@ -214,7 +216,7 @@ class JobRunnerMapper:
     def __determine_job_destination(self, params, raw_job_destination=None):
         if self.job_wrapper.tool is None:
             raise JobMappingException(
-                "Can't map job to destination, tool '%s' is unavailable" % self.job_wrapper.get_job().tool_id
+                f"Can't map job to destination, tool '{self.job_wrapper.get_job().tool_id}' is unavailable"
             )
         if raw_job_destination is None:
             raw_job_destination = self.job_wrapper.tool.get_job_destination(params)
