@@ -33,8 +33,6 @@ class ShellJobRunner(AsynchronousJobRunner):
         super().__init__(app, nworkers)
 
         self.cli_interface = CliInterface()
-        self._init_monitor_thread()
-        self._init_worker_threads()
 
     def get_cli_plugins(self, shell_params, job_params):
         return self.cli_interface.get_plugins(shell_params, job_params)
@@ -43,11 +41,11 @@ class ShellJobRunner(AsynchronousJobRunner):
         params = {}
         shell_params, job_params = url.split('/')[2:4]
         # split 'foo=bar&baz=quux' into { 'foo' : 'bar', 'baz' : 'quux' }
-        shell_params = {'shell_' + k: v for k, v in [kv.split('=', 1) for kv in shell_params.split('&')]}
-        job_params = {'job_' + k: v for k, v in [kv.split('=', 1) for kv in job_params.split('&')]}
+        shell_params = {f"shell_{k}": v for k, v in [kv.split('=', 1) for kv in shell_params.split('&')]}
+        job_params = {f"job_{k}": v for k, v in [kv.split('=', 1) for kv in job_params.split('&')]}
         params.update(shell_params)
         params.update(job_params)
-        log.debug("Converted URL '{}' to destination runner=cli, params={}".format(url, params))
+        log.debug(f"Converted URL '{url}' to destination runner=cli, params={params}")
         # Create a dynamic JobDestination
         return JobDestination(runner='cli', params=params)
 
@@ -81,20 +79,20 @@ class ShellJobRunner(AsynchronousJobRunner):
         )
 
         try:
-            self.write_executable_script(ajs.job_file, script)
+            self.write_executable_script(ajs.job_file, script, job_io=job_wrapper.job_io)
         except Exception:
-            log.exception("(%s) failure writing job script" % galaxy_id_tag)
+            log.exception(f"({galaxy_id_tag}) failure writing job script")
             job_wrapper.fail("failure preparing job script", exception=True)
             return
 
         # job was deleted while we were preparing it
-        if job_wrapper.get_state() == model.Job.states.DELETED:
-            log.info("(%s) Job deleted by user before it entered the queue" % galaxy_id_tag)
+        if job_wrapper.get_state() in (model.Job.states.DELETED, model.Job.states.STOPPED):
+            log.debug("(%s) Job deleted/stopped by user before it entered the queue", galaxy_id_tag)
             if job_wrapper.cleanup_job in ("always", "onsuccess"):
                 job_wrapper.cleanup()
             return
 
-        log.debug("({}) submitting file: {}".format(galaxy_id_tag, ajs.job_file))
+        log.debug(f"({galaxy_id_tag}) submitting file: {ajs.job_file}")
 
         returncode, stdout = self.submit(shell, job_interface, ajs.job_file, galaxy_id_tag, retry=MAX_SUBMIT_RETRY)
         if returncode != 0:
@@ -104,11 +102,11 @@ class ShellJobRunner(AsynchronousJobRunner):
         # Strip and split to get job ID.
         external_job_id = stdout.strip().split()[-1]
         if not external_job_id:
-            log.error('(%s) submission did not return a job identifier, failing job' % galaxy_id_tag)
+            log.error(f'({galaxy_id_tag}) submission did not return a job identifier, failing job')
             job_wrapper.fail("failure submitting job")
             return
 
-        log.info("({}) queued with identifier: {}".format(galaxy_id_tag, external_job_id))
+        log.info(f"({galaxy_id_tag}) queued with identifier: {external_job_id}")
 
         # store runner information for tracking if Galaxy restarts
         job_wrapper.set_external_id(external_job_id)
@@ -131,8 +129,8 @@ class ShellJobRunner(AsynchronousJobRunner):
         cmd_out = shell.execute(job_interface.submit(job_file))
         if cmd_out.returncode == 0:
             return cmd_out.returncode, cmd_out.stdout
-        stdout = '({}) submission failed (stdout): {}'.format(galaxy_id_tag, cmd_out.stdout)
-        stderr = '({}) submission failed (stderr): {}'.format(galaxy_id_tag, cmd_out.stderr)
+        stdout = f'({galaxy_id_tag}) submission failed (stdout): {cmd_out.stdout}'
+        stderr = f'({galaxy_id_tag}) submission failed (stderr): {cmd_out.stderr}'
         if retry > 0:
             log.debug("%s, retrying in %s seconds", stdout, timeout)
             log.debug("%s, retrying in %s seconds", stderr, timeout)
@@ -161,32 +159,34 @@ class ShellJobRunner(AsynchronousJobRunner):
                 if ajs.job_wrapper.get_state() == model.Job.states.DELETED:
                     continue
 
-                log.debug("({}/{}) job not found in batch state check".format(id_tag, external_job_id))
+                log.debug(f"({id_tag}/{external_job_id}) job not found in batch state check")
                 shell_params, job_params = self.parse_destination_params(ajs.job_destination.params)
                 shell, job_interface = self.get_cli_plugins(shell_params, job_params)
                 cmd_out = shell.execute(job_interface.get_single_status(external_job_id))
                 state = job_interface.parse_single_status(cmd_out.stdout, external_job_id)
                 if not state == model.Job.states.OK:
-                    log.warning('({}/{}) job not found in batch state check, but found in individual state check'.format(id_tag, external_job_id))
+                    log.warning(f'({id_tag}/{external_job_id}) job not found in batch state check, but found in individual state check')
+            job_state = ajs.job_wrapper.get_state()
             if state != old_state:
-                log.debug("({}/{}) state change: from {} to {}".format(id_tag, external_job_id, old_state, state))
-                if not state == model.Job.states.OK:
-                    # No need to change_state when the state is OK, this will be handled by `self.finish_job`
-                    ajs.job_wrapper.change_state(state)
-                if state == model.Job.states.ERROR:
-                    # Try to find out the reason for exiting
+                log.debug(f"({id_tag}/{external_job_id}) state change: from {old_state} to {state}")
+                if state == model.Job.states.ERROR and job_state != model.Job.states.STOPPED:
+                    # Try to find out the reason for exiting - this needs to happen before change_state
+                    # otherwise jobs depending on resubmission outputs see that job as failed and pause.
                     self.__handle_out_of_memory(ajs, external_job_id)
                     self.work_queue.put((self.mark_as_failed, ajs))
                     # Don't add the job to the watched items once it fails, deals with https://github.com/galaxyproject/galaxy/issues/7820
                     continue
+                if not state == model.Job.states.OK:
+                    # No need to change_state when the state is OK, this will be handled by `self.finish_job`
+                    ajs.job_wrapper.change_state(state)
             if state == model.Job.states.RUNNING and not ajs.running:
                 ajs.running = True
             ajs.old_state = state
-            if state == model.Job.states.OK:
+            if state == model.Job.states.OK or job_state == model.Job.states.STOPPED:
                 external_metadata = not asbool(ajs.job_wrapper.job_destination.params.get("embed_metadata_in_job", DEFAULT_EMBED_METADATA_IN_JOB))
                 if external_metadata:
                     self.work_queue.put((self.handle_metadata_externally, ajs))
-                log.debug('({}/{}) job execution finished, running job wrapper finish method'.format(id_tag, external_job_id))
+                log.debug(f'({id_tag}/{external_job_id}) job execution finished, running job wrapper finish method')
                 self.work_queue.put((self.finish_job, ajs))
             else:
                 new_watched.append(ajs)
@@ -234,9 +234,9 @@ class ShellJobRunner(AsynchronousJobRunner):
             shell, job_interface = self.get_cli_plugins(shell_params, job_params)
             cmd_out = shell.execute(job_interface.delete(job.job_runner_external_id))
             assert cmd_out.returncode == 0, cmd_out.stderr
-            log.debug("({}/{}) Terminated at user's request".format(job.id, job.job_runner_external_id))
+            log.debug(f"({job.id}/{job.job_runner_external_id}) Terminated at user's request")
         except Exception as e:
-            log.debug("({}/{}) User killed running job, but error encountered during termination: {}".format(job.id, job.job_runner_external_id, e))
+            log.debug(f"({job.id}/{job.job_runner_external_id}) User killed running job, but error encountered during termination: {e}")
 
     def recover(self, job, job_wrapper):
         """Recovers jobs stuck in the queued/running state when Galaxy started"""
@@ -249,13 +249,13 @@ class ShellJobRunner(AsynchronousJobRunner):
         ajs.command_line = job.command_line
         ajs.job_wrapper = job_wrapper
         ajs.job_destination = job_wrapper.job_destination
-        if job.state == model.Job.states.RUNNING:
-            log.debug("({}/{}) is still in running state, adding to the runner monitor queue".format(job.id, job.job_runner_external_id))
+        if job.state in (model.Job.states.RUNNING, model.Job.states.STOPPED):
+            log.debug(f"({job.id}/{job.job_runner_external_id}) is still in {job.state} state, adding to the runner monitor queue")
             ajs.old_state = model.Job.states.RUNNING
             ajs.running = True
             self.monitor_queue.put(ajs)
         elif job.state == model.Job.states.QUEUED:
-            log.debug("({}/{}) is still in queued state, adding to the runner monitor queue".format(job.id, job.job_runner_external_id))
+            log.debug(f"({job.id}/{job.job_runner_external_id}) is still in queued state, adding to the runner monitor queue")
             ajs.old_state = model.Job.states.QUEUED
             ajs.running = False
             self.monitor_queue.put(ajs)
