@@ -51,27 +51,6 @@ class CondorJobRunner(AsynchronousJobRunner):
     """
     runner_name = "CondorRunner"
 
-    def __init__(self, app, nworkers):
-        """Initialize this job runner and start the monitor thread"""
-        super().__init__(app, nworkers)
-        self._init_monitor_thread()
-        self._init_worker_threads()
-
-    def __old_state_paths(self, cjs):
-        """For recovery of jobs started prior to standardizing the naming of
-        files in the AsychronousJobState object
-        Remove this function in 21.01
-        """
-        if cjs.job_wrapper is not None:
-            job_file = "{}/galaxy_{}.sh".format(self.app.config.cluster_files_directory, cjs.job_wrapper.job_id)
-            if not os.path.exists(cjs.job_file) and os.path.exists(job_file):
-                cluster_files_dir_and_id = (self.app.config.cluster_files_directory, cjs.job_wrapper.get_id_tag())
-                cjs.output_file = "%s/galaxy_%s.o" % cluster_files_dir_and_id
-                cjs.error_file = "%s/galaxy_%s.e" % cluster_files_dir_and_id
-                cjs.exit_code_file = "%s/galaxy_%s.ec" % cluster_files_dir_and_id
-                cjs.user_log = "%s/galaxy_%s.condor.log" % cluster_files_dir_and_id
-                cjs.job_file = job_file
-
     def queue_job(self, job_wrapper):
         """Create job script and submit it to the DRM"""
 
@@ -98,9 +77,9 @@ class CondorJobRunner(AsynchronousJobRunner):
 
         galaxy_slots = query_params.get('request_cpus', None)
         if galaxy_slots:
-            galaxy_slots_statement = 'GALAXY_SLOTS="%s"; export GALAXY_SLOTS_CONFIGURED="1"' % galaxy_slots
+            galaxy_slots_statement = f'GALAXY_SLOTS="{galaxy_slots}"; export GALAXY_SLOTS; GALAXY_SLOTS_CONFIGURED="1"; export GALAXY_SLOTS_CONFIGURED;'
         else:
-            galaxy_slots_statement = 'GALAXY_SLOTS="1"'
+            galaxy_slots_statement = 'GALAXY_SLOTS="1"; export GALAXY_SLOTS;'
 
         # define job attributes
         cjs = CondorJobState(
@@ -108,9 +87,9 @@ class CondorJobRunner(AsynchronousJobRunner):
             job_wrapper=job_wrapper
         )
 
-        cjs.user_log = os.path.join(job_wrapper.working_directory, 'galaxy_%s.condor.log' % galaxy_id_tag)
+        cjs.user_log = os.path.join(job_wrapper.working_directory, f'galaxy_{galaxy_id_tag}.condor.log')
         cjs.register_cleanup_file_attribute('user_log')
-        submit_file = os.path.join(job_wrapper.working_directory, 'galaxy_%s.condor.desc' % galaxy_id_tag)
+        submit_file = os.path.join(job_wrapper.working_directory, f'galaxy_{galaxy_id_tag}.condor.desc')
         executable = cjs.job_file
 
         build_submit_params = dict(
@@ -129,10 +108,10 @@ class CondorJobRunner(AsynchronousJobRunner):
             shell=job_wrapper.shell,
         )
         try:
-            self.write_executable_script(executable, script)
+            self.write_executable_script(executable, script, job_io=job_wrapper.job_io)
         except Exception:
             job_wrapper.fail("failure preparing job script", exception=True)
-            log.exception("(%s) failure preparing job script" % galaxy_id_tag)
+            log.exception(f"({galaxy_id_tag}) failure preparing job script")
             return
 
         cleanup_job = job_wrapper.cleanup_job
@@ -143,23 +122,23 @@ class CondorJobRunner(AsynchronousJobRunner):
                 cjs.cleanup()
                 # job_wrapper.fail() calls job_wrapper.cleanup()
             job_wrapper.fail("failure preparing submit file", exception=True)
-            log.exception("(%s) failure preparing submit file" % galaxy_id_tag)
+            log.exception(f"({galaxy_id_tag}) failure preparing submit file")
             return
 
         # job was deleted while we were preparing it
-        if job_wrapper.get_state() == model.Job.states.DELETED:
-            log.debug("Job %s deleted by user before it entered the queue" % galaxy_id_tag)
+        if job_wrapper.get_state() in (model.Job.states.DELETED, model.Job.states.STOPPED):
+            log.debug("(%s) Job deleted/stopped by user before it entered the queue", galaxy_id_tag)
             if cleanup_job in ("always", "onsuccess"):
                 os.unlink(submit_file)
                 cjs.cleanup()
                 job_wrapper.cleanup()
             return
 
-        log.debug("({}) submitting file {}".format(galaxy_id_tag, executable))
+        log.debug(f"({galaxy_id_tag}) submitting file {executable}")
 
         external_job_id, message = condor_submit(submit_file)
         if external_job_id is None:
-            log.debug("condor_submit failed for job {}: {}".format(job_wrapper.get_id_tag(), message))
+            log.debug(f"condor_submit failed for job {job_wrapper.get_id_tag()}: {message}")
             if self.app.config.cleanup_job == "always":
                 os.unlink(submit_file)
                 cjs.cleanup()
@@ -168,7 +147,7 @@ class CondorJobRunner(AsynchronousJobRunner):
 
         os.unlink(submit_file)
 
-        log.info("({}) queued as {}".format(galaxy_id_tag, external_job_id))
+        log.info(f"({galaxy_id_tag}) queued as {external_job_id}")
 
         # store runner information for tracking if Galaxy restarts
         job_wrapper.set_external_id(external_job_id)
@@ -189,7 +168,6 @@ class CondorJobRunner(AsynchronousJobRunner):
         for cjs in self.watched:
             job_id = cjs.job_id
             galaxy_id_tag = cjs.job_wrapper.get_id_tag()
-            self.__old_state_paths(cjs)  # remove in 21.01
             try:
                 if cjs.job_wrapper.tool.tool_type != 'interactive' and os.stat(cjs.user_log).st_size == cjs.user_log_size:
                     new_watched.append(cjs)
@@ -201,8 +179,8 @@ class CondorJobRunner(AsynchronousJobRunner):
                 cjs.user_log_size = log_size
             except Exception:
                 # so we don't kill the monitor thread
-                log.exception("({}/{}) Unable to check job status".format(galaxy_id_tag, job_id))
-                log.warning("({}/{}) job will now be errored".format(galaxy_id_tag, job_id))
+                log.exception(f"({galaxy_id_tag}/{job_id}) Unable to check job status")
+                log.warning(f"({galaxy_id_tag}/{job_id}) job will now be errored")
                 cjs.fail_message = "Cluster could not complete job"
                 self.work_queue.put((self.fail_job, cjs))
                 continue
@@ -212,22 +190,23 @@ class CondorJobRunner(AsynchronousJobRunner):
                 cjs.job_wrapper.check_for_entry_points()
 
             if job_running and not cjs.running:
-                log.debug("({}/{}) job is now running".format(galaxy_id_tag, job_id))
+                log.debug(f"({galaxy_id_tag}/{job_id}) job is now running")
                 cjs.job_wrapper.change_state(model.Job.states.RUNNING)
             if not job_running and cjs.running:
-                log.debug("({}/{}) job has stopped running".format(galaxy_id_tag, job_id))
+                log.debug(f"({galaxy_id_tag}/{job_id}) job has stopped running")
                 # Will switching from RUNNING to QUEUED confuse Galaxy?
                 # cjs.job_wrapper.change_state( model.Job.states.QUEUED )
-            if job_complete:
-                if cjs.job_wrapper.get_state() != model.Job.states.DELETED:
+            job_state = cjs.job_wrapper.get_state()
+            if job_complete or job_state == model.Job.states.STOPPED:
+                if job_state != model.Job.states.DELETED:
                     external_metadata = not asbool(cjs.job_wrapper.job_destination.params.get("embed_metadata_in_job", True))
                     if external_metadata:
                         self._handle_metadata_externally(cjs.job_wrapper, resolve_requirements=True)
-                    log.debug("({}/{}) job has completed".format(galaxy_id_tag, job_id))
+                    log.debug(f"({galaxy_id_tag}/{job_id}) job has completed")
                     self.work_queue.put((self.finish_job, cjs))
                 continue
             if job_failed:
-                log.debug("({}/{}) job failed".format(galaxy_id_tag, job_id))
+                log.debug(f"({galaxy_id_tag}/{job_id}) job failed")
                 cjs.failed = True
                 self.work_queue.put((self.finish_job, cjs))
                 continue
@@ -243,7 +222,7 @@ class CondorJobRunner(AsynchronousJobRunner):
         galaxy_id_tag = job_wrapper.get_id_tag()
         if job.container:
             try:
-                log.info("stop_job(): {}: trying to stop container .... ({})".format(job.id, external_id))
+                log.info(f"stop_job(): {job.id}: trying to stop container .... ({external_id})")
                 # self.watched = [cjs for cjs in self.watched if cjs.job_id != external_id]
                 new_watch_list = list()
                 cjs = None
@@ -260,21 +239,21 @@ class CondorJobRunner(AsynchronousJobRunner):
                     external_metadata = not asbool(cjs.job_wrapper.job_destination.params.get("embed_metadata_in_job", True))
                     if external_metadata:
                         self._handle_metadata_externally(cjs.job_wrapper, resolve_requirements=True)
-                    log.debug("({}/{}) job has completed".format(galaxy_id_tag, external_id))
+                    log.debug(f"({galaxy_id_tag}/{external_id}) job has completed")
                     self.work_queue.put((self.finish_job, cjs))
             except Exception as e:
-                log.warning("stop_job(): {}: trying to stop container failed. ({})".format(job.id, e))
+                log.warning(f"stop_job(): {job.id}: trying to stop container failed. ({e})")
                 try:
                     self._kill_container(job_wrapper)
                 except Exception as e:
-                    log.warning("stop_job(): {}: trying to kill container failed. ({})".format(job.id, e))
+                    log.warning(f"stop_job(): {job.id}: trying to kill container failed. ({e})")
                     failure_message = condor_stop(external_id)
                     if failure_message:
-                        log.debug("({}). Failed to stop condor {}".format(external_id, failure_message))
+                        log.debug(f"({external_id}). Failed to stop condor {failure_message}")
         else:
             failure_message = condor_stop(external_id)
             if failure_message:
-                log.debug("({}). Failed to stop condor {}".format(external_id, failure_message))
+                log.debug(f"({external_id}). Failed to stop condor {failure_message}")
 
     def recover(self, job, job_wrapper):
         """Recovers jobs stuck in the queued/running state when Galaxy started"""
@@ -289,15 +268,14 @@ class CondorJobRunner(AsynchronousJobRunner):
         cjs.command_line = job.get_command_line()
         cjs.job_wrapper = job_wrapper
         cjs.job_destination = job_wrapper.job_destination
-        cjs.user_log = os.path.join(job_wrapper.working_directory, 'galaxy_%s.condor.log' % galaxy_id_tag)
+        cjs.user_log = os.path.join(job_wrapper.working_directory, f'galaxy_{galaxy_id_tag}.condor.log')
         cjs.register_cleanup_file_attribute('user_log')
-        self.__old_state_paths(cjs)  # remove in 21.01
-        if job.state == model.Job.states.RUNNING:
-            log.debug("({}/{}) is still in running state, adding to the DRM queue".format(job.id, job.job_runner_external_id))
+        if job.state in (model.Job.states.RUNNING, model.Job.states.STOPPED):
+            log.debug(f"({job.id}/{job.get_job_runner_external_id()}) is still in {job.state} state, adding to the DRM queue")
             cjs.running = True
             self.monitor_queue.put(cjs)
         elif job.state == model.Job.states.QUEUED:
-            log.debug("({}/{}) is still in DRM queued state, adding to the DRM queue".format(job.id, job.job_runner_external_id))
+            log.debug(f"({job.id}/{job.job_runner_external_id}) is still in DRM queued state, adding to the DRM queue")
             cjs.running = False
             self.monitor_queue.put(cjs)
 
@@ -317,7 +295,7 @@ class CondorJobRunner(AsynchronousJobRunner):
                     return self._run_command(cont.container_info['commands'][command], external_id)[0]
 
     def _run_command(self, command, external_job_id):
-        command = 'condor_ssh_to_job {} {}'.format(external_job_id, command)
+        command = f'condor_ssh_to_job {external_job_id} {command}'
 
         p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, close_fds=True, preexec_fn=os.setpgrp)
         stdout, stderr = p.communicate()
