@@ -45,8 +45,8 @@ class DRMAAJobRunner(AsynchronousJobRunner):
         runner_param_specs = {
             'drmaa_library_path': dict(map=str, default=os.environ.get('DRMAA_LIBRARY_PATH', None))}
         for retry_exception in RETRY_EXCEPTIONS_LOWER:
-            runner_param_specs[retry_exception + '_state'] = dict(map=str, valid=lambda x: x in (model.Job.states.OK, model.Job.states.ERROR), default=model.Job.states.OK)
-            runner_param_specs[retry_exception + '_retries'] = dict(map=int, valid=lambda x: int(x) >= 0, default=0)
+            runner_param_specs[f"{retry_exception}_state"] = dict(map=str, valid=lambda x: x in (model.Job.states.OK, model.Job.states.ERROR), default=model.Job.states.OK)
+            runner_param_specs[f"{retry_exception}_retries"] = dict(map=int, valid=lambda x: int(x) >= 0, default=0)
 
         if 'runner_param_specs' not in kwargs:
             kwargs['runner_param_specs'] = dict()
@@ -95,8 +95,6 @@ class DRMAAJobRunner(AsynchronousJobRunner):
 
         self.userid = None
 
-        self._init_monitor_thread()
-        self._init_worker_threads()
         self.redact_email_in_job_name = self.app.config.redact_email_in_job_name
 
     def url_to_destination(self, url):
@@ -106,10 +104,10 @@ class DRMAAJobRunner(AsynchronousJobRunner):
         native_spec = url.split('/')[2]
         if native_spec:
             params = dict(nativeSpecification=native_spec)
-            log.debug("Converted URL '{}' to destination runner=drmaa, params={}".format(url, params))
+            log.debug(f"Converted URL '{url}' to destination runner=drmaa, params={params}")
             return JobDestination(runner='drmaa', params=params)
         else:
-            log.debug("Converted URL '%s' to destination runner=drmaa" % url)
+            log.debug(f"Converted URL '{url}' to destination runner=drmaa")
             return JobDestination(runner='drmaa')
 
     def get_native_spec(self, url):
@@ -144,8 +142,8 @@ class DRMAAJobRunner(AsynchronousJobRunner):
             remoteCommand=ajs.job_file,
             jobName=ajs.job_name,
             workingDirectory=job_wrapper.working_directory,
-            outputPath=":%s" % ajs.output_file,
-            errorPath=":%s" % ajs.error_file
+            outputPath=f":{ajs.output_file}",
+            errorPath=f":{ajs.error_file}"
         )
 
         # Avoid a jt.exitCodePath for now - it's only used when finishing.
@@ -158,15 +156,15 @@ class DRMAAJobRunner(AsynchronousJobRunner):
         # fill in the DRM's job run template
         script = self.get_job_file(job_wrapper, exit_code_path=ajs.exit_code_file, shell=job_wrapper.shell)
         try:
-            self.write_executable_script(ajs.job_file, script)
+            self.write_executable_script(ajs.job_file, script, job_io=job_wrapper.job_io)
         except Exception:
             job_wrapper.fail("failure preparing job script", exception=True)
-            log.exception("(%s) failure writing job script" % galaxy_id_tag)
+            log.exception(f"({galaxy_id_tag}) failure writing job script")
             return
 
         # job was deleted while we were preparing it
-        if job_wrapper.get_state() == model.Job.states.DELETED:
-            log.debug("(%s) Job deleted by user before it entered the queue" % galaxy_id_tag)
+        if job_wrapper.get_state() in (model.Job.states.DELETED, model.Job.states.STOPPED):
+            log.debug("(%s) Job deleted/stopped by user before it entered the queue", galaxy_id_tag)
             if job_wrapper.cleanup_job in ("always", "onsuccess"):
                 job_wrapper.cleanup()
             return
@@ -195,7 +193,7 @@ class DRMAAJobRunner(AsynchronousJobRunner):
                     log.exception('(%s) drmaa.Session.runJob() failed unconditionally', galaxy_id_tag)
                     trynum = 5
             else:
-                log.error("(%s) All attempts to submit job failed" % galaxy_id_tag)
+                log.error(f"({galaxy_id_tag}) All attempts to submit job failed")
                 if not fail_msg:
                     fail_msg = DEFAULT_JOB_PUT_FAILURE_MESSAGE
                 job_wrapper.fail(fail_msg)
@@ -207,18 +205,18 @@ class DRMAAJobRunner(AsynchronousJobRunner):
             pwent = job_wrapper.user_system_pwent
             if pwent is None:
                 if not allow_guests:
-                    fail_msg = "User %s is not mapped to any real user, and not permitted to start jobs." % job_wrapper.user
+                    fail_msg = f"User {job_wrapper.user} is not mapped to any real user, and not permitted to start jobs."
                     job_wrapper.fail(fail_msg)
                     return
                 pwent = job_wrapper.galaxy_system_pwent
-            log.debug('({}) submitting with credentials: {} [uid: {}]'.format(galaxy_id_tag, pwent[0], pwent[2]))
+            log.debug(f'({galaxy_id_tag}) submitting with credentials: {pwent[0]} [uid: {pwent[2]}]')
             filename = self.store_jobtemplate(job_wrapper, jt)
             self.userid = pwent[2]
             external_job_id = self.external_runjob(external_runjob_script, filename, pwent[2])
             if external_job_id is None:
-                job_wrapper.fail("(%s) could not queue job" % galaxy_id_tag)
+                job_wrapper.fail(f"({galaxy_id_tag}) could not queue job")
                 return
-        log.info("({}) queued as {}".format(galaxy_id_tag, external_job_id))
+        log.info(f"({galaxy_id_tag}) queued as {external_job_id}")
 
         # store runner information for tracking if Galaxy restarts
         job_wrapper.set_external_id(external_job_id)
@@ -241,17 +239,18 @@ class DRMAAJobRunner(AsynchronousJobRunner):
         does not determine if a job was terminal, but the implementation
         in the subclasses is supposed to do this.)
         """
-        if drmaa_state == drmaa.JobState.FAILED:
-            if ajs.job_wrapper.get_state() != model.Job.states.DELETED:
+        job_state = ajs.job_wrapper.get_state()
+        if drmaa_state == drmaa.JobState.FAILED and job_state != model.Job.states.STOPPED:
+            if job_state != model.Job.states.DELETED:
                 ajs.stop_job = False
                 ajs.fail_message = "The cluster DRM system terminated this job"
                 self.work_queue.put((self.fail_job, ajs))
-        elif drmaa_state == drmaa.JobState.DONE:
+        elif drmaa_state == drmaa.JobState.DONE or job_state == model.Job.states.STOPPED:
             # External metadata processing for external runjobs
             external_metadata = not asbool(ajs.job_wrapper.job_destination.params.get("embed_metadata_in_job", True))
             if external_metadata:
                 self._handle_metadata_externally(ajs.job_wrapper, resolve_requirements=True)
-            if ajs.job_wrapper.get_state() != model.Job.states.DELETED:
+            if job_state != model.Job.states.DELETED:
                 self.work_queue.put((self.finish_job, ajs))
 
     def check_watched_item(self, ajs, new_watched):
@@ -260,13 +259,17 @@ class DRMAAJobRunner(AsynchronousJobRunner):
         that could happen in this process. to be called from check_watched_items()
         returns the state or None if exceptions occurred
         in the latter case the job is appended to new_watched if a
+
         1 drmaa.InternalException,
         2 drmaa.InvalidJobExceptionnot, or
         3 drmaa.DrmCommunicationException occurred
+
         (which causes the job to be tested again in the next iteration of check_watched_items)
+
         - the job is finished as errored if any other exception occurs
         - the job is finished OK or errored after the maximum number of retries
           depending on the exception
+
         Note that None is returned in all cases where the loop in check_watched_items
         is to be continued
         """
@@ -274,15 +277,15 @@ class DRMAAJobRunner(AsynchronousJobRunner):
         galaxy_id_tag = ajs.job_wrapper.get_id_tag()
         state = None
         try:
-            assert external_job_id not in (None, 'None'), '({}/{}) Invalid job id'.format(galaxy_id_tag, external_job_id)
+            assert external_job_id not in (None, 'None'), f'({galaxy_id_tag}/{external_job_id}) Invalid job id'
             state = self.ds.job_status(external_job_id)
             # Reset exception retries
             for retry_exception in RETRY_EXCEPTIONS_LOWER:
-                setattr(ajs, retry_exception + '_retries', 0)
+                setattr(ajs, f"{retry_exception}_retries", 0)
         except (drmaa.InternalException, drmaa.InvalidJobException) as e:
             ecn = type(e).__name__
-            retry_param = ecn.lower() + '_retries'
-            state_param = ecn.lower() + '_state'
+            retry_param = f"{ecn.lower()}_retries"
+            state_param = f"{ecn.lower()}_state"
             retries = getattr(ajs, retry_param, 0)
             log.warning("(%s/%s) unable to check job status because of %s exception for %d consecutive tries: %s", galaxy_id_tag, external_job_id, ecn, retries + 1, e)
             if self.runner_params[retry_param] > 0:
@@ -306,8 +309,8 @@ class DRMAAJobRunner(AsynchronousJobRunner):
             return None
         except Exception:
             # so we don't kill the monitor thread
-            log.exception("({}/{}) unable to check job status".format(galaxy_id_tag, external_job_id))
-            log.warning("({}/{}) job will now be errored".format(galaxy_id_tag, external_job_id))
+            log.exception(f"({galaxy_id_tag}/{external_job_id}) unable to check job status")
+            log.warning(f"({galaxy_id_tag}/{external_job_id}) job will now be errored")
             ajs.fail_message = "Cluster could not complete job"
             self.work_queue.put((self.fail_job, ajs))
             return None
@@ -327,7 +330,7 @@ class DRMAAJobRunner(AsynchronousJobRunner):
             if state is None:
                 continue
             if state != old_state:
-                log.debug("({}/{}) state change: {}".format(galaxy_id_tag, external_job_id, self.drmaa_job_state_strings[state]))
+                log.debug(f"({galaxy_id_tag}/{external_job_id}) state change: {self.drmaa_job_state_strings[state]}")
             if state == drmaa.JobState.RUNNING and not ajs.running:
                 ajs.running = True
                 ajs.job_wrapper.change_state(model.Job.states.RUNNING)
@@ -361,13 +364,13 @@ class DRMAAJobRunner(AsynchronousJobRunner):
                 cmd = shlex.split(kill_script)
                 cmd.extend([str(ext_id), str(self.userid)])
                 commands.execute(cmd)
-            log.info("({}/{}) Removed from DRM queue at user's request".format(job.id, ext_id))
+            log.info(f"({job.id}/{ext_id}) Removed from DRM queue at user's request")
         except drmaa.InvalidJobException:
-            log.exception("({}/{}) User killed running job, but it was already dead".format(job.id, ext_id))
+            log.exception(f"({job.id}/{ext_id}) User killed running job, but it was already dead")
         except commands.CommandLineException as e:
-            log.error("({}/{}) User killed running job, but command execution failed: {}".format(job.id, ext_id, unicodify(e)))
+            log.error(f"({job.id}/{ext_id}) User killed running job, but command execution failed: {unicodify(e)}")
         except Exception:
-            log.exception("({}/{}) User killed running job, but error encountered removing from DRM queue".format(job.id, ext_id))
+            log.exception(f"({job.id}/{ext_id}) User killed running job, but error encountered removing from DRM queue")
 
     def recover(self, job, job_wrapper):
         """Recovers jobs stuck in the queued/running state when Galaxy started"""
@@ -380,13 +383,13 @@ class DRMAAJobRunner(AsynchronousJobRunner):
         ajs.command_line = job.get_command_line()
         ajs.job_wrapper = job_wrapper
         ajs.job_destination = job_wrapper.job_destination
-        if job.state == model.Job.states.RUNNING:
-            log.debug("({}/{}) is still in running state, adding to the DRM queue".format(job.id, job.get_job_runner_external_id()))
+        if job.state in (model.Job.states.RUNNING, model.Job.states.STOPPED):
+            log.debug(f"({job.id}/{job.get_job_runner_external_id()}) is still in {job.state} state, adding to the DRM queue")
             ajs.old_state = drmaa.JobState.RUNNING
             ajs.running = True
             self.monitor_queue.put(ajs)
         elif job.get_state() == model.Job.states.QUEUED:
-            log.debug("({}/{}) is still in DRM queued state, adding to the DRM queue".format(job.id, job.get_job_runner_external_id()))
+            log.debug(f"({job.id}/{job.get_job_runner_external_id()}) is still in DRM queued state, adding to the DRM queue")
             ajs.old_state = drmaa.JobState.QUEUED_ACTIVE
             ajs.running = False
             self.monitor_queue.put(ajs)
@@ -395,10 +398,10 @@ class DRMAAJobRunner(AsynchronousJobRunner):
         """ Stores the content of a DRMAA JobTemplate object in a file as a JSON string.
         Path is hard-coded, but it's no worse than other path in this module.
         Uses Galaxy's JobID, so file is expected to be unique."""
-        filename = "{}/{}.jt_json".format(self.app.config.cluster_files_directory, job_wrapper.get_id_tag())
+        filename = f"{self.app.config.cluster_files_directory}/{job_wrapper.get_id_tag()}.jt_json"
         with open(filename, 'w+') as fp:
             json.dump(jt, fp)
-        log.debug('({}) Job script for external submission is: {}'.format(job_wrapper.job_id, filename))
+        log.debug(f'({job_wrapper.job_id}) Job script for external submission is: {filename}')
         return filename
 
     def external_runjob(self, external_runjob_script, jobtemplate_filename, username):
@@ -408,7 +411,7 @@ class DRMAAJobRunner(AsynchronousJobRunner):
         """
         cmd = shlex.split(external_runjob_script)
         cmd.extend([str(username), jobtemplate_filename])
-        log.info("Running command %s" % cmd)
+        log.info(f"Running command: {' '.join(cmd)}")
         try:
             stdoutdata = commands.execute(cmd).strip()
         except commands.CommandLineException:
@@ -426,12 +429,12 @@ class DRMAAJobRunner(AsynchronousJobRunner):
         galaxy_id_tag = job_wrapper.get_id_tag()
 
         # define job attributes
-        job_name = 'g%s' % galaxy_id_tag
+        job_name = f'g{galaxy_id_tag}'
         if job_wrapper.tool.old_id:
-            job_name += '_%s' % job_wrapper.tool.old_id
+            job_name += f'_{job_wrapper.tool.old_id}'
         if not self.redact_email_in_job_name and external_runjob_script is None:
-            job_name += '_%s' % job_wrapper.user
-        job_name = ''.join(x if x in (string.ascii_letters + string.digits + '_') else '_' for x in job_name)
+            job_name += f'_{job_wrapper.user}'
+        job_name = ''.join(x if x in (f"{string.ascii_letters + string.digits}_") else '_' for x in job_name)
         if self.restrict_job_name_length:
             job_name = job_name[:self.restrict_job_name_length]
         return job_name
