@@ -14,6 +14,7 @@ from galaxy.job_execution.output_collect import default_exit_code_file
 from galaxy.util import (
     asbool,
 )
+from galaxy.util.commands import new_clean_env
 from . import (
     BaseJobRunner,
     JobState
@@ -42,17 +43,12 @@ class LocalJobRunner(BaseJobRunner):
     def __init__(self, app, nworkers):
         """Start the job runner """
 
-        # create a local copy of os.environ to use as env for subprocess.Popen
-        self._environ = os.environ.copy()
         self._proc_lock = threading.Lock()
         self._procs = []
 
-        # Set TEMP if a valid temp value is not already set
-        if not ('TMPDIR' in self._environ or 'TEMP' in self._environ or 'TMP' in self._environ):
-            self._environ['TEMP'] = os.path.abspath(tempfile.gettempdir())
+        self._environ = new_clean_env()
 
         super().__init__(app, nworkers)
-        self._init_worker_threads()
 
     def __command_line(self, job_wrapper):
         """
@@ -61,9 +57,9 @@ class LocalJobRunner(BaseJobRunner):
 
         # slots would be cleaner name, but don't want deployers to see examples and think it
         # is going to work with other job runners.
-        slots = job_wrapper.job_destination.params.get("local_slots", None) or os.environ.get("GALAXY_SLOTS", None)
+        slots = job_wrapper.job_destination.params.get("local_slots") or os.environ.get("GALAXY_SLOTS")
         if slots:
-            slots_statement = 'GALAXY_SLOTS="%d"; export GALAXY_SLOTS; GALAXY_SLOTS_CONFIGURED="1"; export GALAXY_SLOTS_CONFIGURED;' % (int(slots))
+            slots_statement = f'GALAXY_SLOTS="{int(slots)}"; export GALAXY_SLOTS; GALAXY_SLOTS_CONFIGURED="1"; export GALAXY_SLOTS_CONFIGURED;'
         else:
             slots_statement = 'GALAXY_SLOTS="1"; export GALAXY_SLOTS;'
 
@@ -78,7 +74,7 @@ class LocalJobRunner(BaseJobRunner):
             'shell': job_wrapper.shell,
         }
         job_file_contents = self.get_job_file(job_wrapper, **job_script_props)
-        self.write_executable_script(job_file, job_file_contents)
+        self.write_executable_script(job_file, job_file_contents, job_io=job_wrapper.job_io)
         return job_file, exit_code_path
 
     def queue_job(self, job_wrapper):
@@ -94,7 +90,7 @@ class LocalJobRunner(BaseJobRunner):
         try:
             stdout_file = tempfile.NamedTemporaryFile(mode='wb+', suffix='_stdout', dir=job_wrapper.working_directory)
             stderr_file = tempfile.NamedTemporaryFile(mode='wb+', suffix='_stderr', dir=job_wrapper.working_directory)
-            log.debug('({}) executing job script: {}'.format(job_id, job_file))
+            log.debug(f'({job_id}) executing job script: {job_file}')
             # The preexec_fn argument of Popen() is used to call os.setpgrp() in
             # the child process just before the child is executed. This will set
             # the PGID of the child process to its PID (i.e. ensures that it is
@@ -137,7 +133,7 @@ class LocalJobRunner(BaseJobRunner):
             stderr = self._job_io_for_db(stderr_file)
             stdout_file.close()
             stderr_file.close()
-            log.debug('execution finished: %s' % job_file)
+            log.debug(f'execution finished: {job_file}')
         except Exception:
             log.exception("failure running job %d", job_wrapper.job_id)
             self._fail_job_local(job_wrapper, "failure running job")
@@ -162,7 +158,7 @@ class LocalJobRunner(BaseJobRunner):
             # metadata internal or job not complete yet
             pid = job.get_job_runner_external_id()
         if pid in [None, '']:
-            log.warning("stop_job(): %s: no PID in database for job, unable to stop" % job.id)
+            log.warning(f"stop_job(): {job.id}: no PID in database for job, unable to stop")
             return
         pid = int(pid)
         if not check_pg(pid):
