@@ -2,9 +2,6 @@
 SLURM job control via the DRMAA API.
 """
 import os
-import re
-import shutil
-import tempfile
 import time
 
 from galaxy import model
@@ -27,11 +24,6 @@ SLURM_MEMORY_LIMIT_EXCEEDED_MSG = 'slurmstepd: error: Exceeded job memory limit'
 # https://github.com/SchedMD/slurm/
 SLURM_MEMORY_LIMIT_EXCEEDED_PARTIAL_WARNINGS = [': Exceeded job memory limit at some point.',
                                                 ': Exceeded step memory limit at some point.']
-SLURM_MEMORY_LIMIT_SCAN_SIZE = 16 * 1024 * 1024  # 16MB
-SLURM_CGROUP_RE = re.compile(r"""slurmstepd: .*cgroup.*$""")
-SLURM_TOP_WARNING_RES = (
-    SLURM_CGROUP_RE,
-)
 
 # These messages are returned to the user
 OUT_OF_MEMORY_MSG = 'This job was terminated because it used more memory than it was allocated.'
@@ -93,7 +85,7 @@ class SlurmJobRunner(DRMAAJobRunner):
                     job_info_values.append(v)
                 except ValueError:
                     # Some value may contain spaces (e.g. `Comment=** time_limit (60m) min_nodes (1) **`)
-                    job_info_values[-1] += ' ' + job_info
+                    job_info_values[-1] += f" {job_info}"
             job_info_dict = dict(zip(job_info_keys, job_info_values))
             return job_info_dict['JobState']
 
@@ -139,6 +131,9 @@ class SlurmJobRunner(DRMAAJobRunner):
                         ajs.fail_message = check_memory_limit_msg
                         ajs.runner_state = ajs.runner_states.MEMORY_LIMIT_REACHED
                     else:
+                        if ajs.job_wrapper.get_state() == model.Job.states.STOPPED:
+                            # User requested to stop job, this isn't an error, just finish as normal
+                            return super()._complete_terminal_job(ajs, drmaa_state=drmaa_state)
                         log.info('(%s/%s) Job was cancelled via SLURM (e.g. with scancel(1))', ajs.job_wrapper.get_id_tag(), ajs.job_id)
                         ajs.fail_message = "This job failed because it was cancelled by an administrator."
                 elif slurm_state in ('PENDING', 'RUNNING'):
@@ -152,23 +147,6 @@ class SlurmJobRunner(DRMAAJobRunner):
                     ajs.stop_job = False
                     self.work_queue.put((self.fail_job, ajs))
                     return
-            if drmaa_state == self.drmaa_job_states.DONE:
-                with open(ajs.error_file) as rfh:
-                    _remove_spurious_top_lines(rfh, ajs)
-                with open(ajs.error_file, 'r+') as f:
-                    if os.path.getsize(ajs.error_file) > SLURM_MEMORY_LIMIT_SCAN_SIZE:
-                        f.seek(-SLURM_MEMORY_LIMIT_SCAN_SIZE, os.SEEK_END)
-                        f.readline()
-                    pos = f.tell()
-                    lines = f.readlines()
-                    f.seek(pos)
-                    for line in lines:
-                        stripped_line = line.strip()
-                        if any(_ in stripped_line for _ in SLURM_MEMORY_LIMIT_EXCEEDED_PARTIAL_WARNINGS):
-                            log.debug('(%s/%s) Job completed, removing SLURM exceeded memory warning: "%s"', ajs.job_wrapper.get_id_tag(), ajs.job_id, stripped_line)
-                        else:
-                            f.write(line)
-                    f.truncate()
         except Exception:
             log.exception('(%s/%s) Failure in SLURM _complete_terminal_job(), job final state will be: %s', ajs.job_wrapper.get_id_tag(), ajs.job_id, drmaa_state)
         # by default, finish the job with the state from drmaa
@@ -195,32 +173,3 @@ class SlurmJobRunner(DRMAAJobRunner):
             log.exception('Error reading end of %s:', efile_path)
 
         return False
-
-
-def _remove_spurious_top_lines(rfh, ajs, maxlines=3):
-    bad = []
-    putback = None
-    for _ in range(maxlines):
-        line = rfh.readline()
-        log.trace('checking line: %s', line)
-        for pattern in SLURM_TOP_WARNING_RES:
-            if pattern.match(line):
-                bad.append(line)
-                # found a match, stop checking REs and check next line
-                break
-        else:
-            if bad:
-                # no match found on this line so line is now a good line, but previous bad lines are found, so it needs to be put back
-                putback = line
-            # no match on this line, stop looking
-            break
-        # check next line
-    if bad:
-        with tempfile.NamedTemporaryFile('w', delete=False) as wfh:
-            if putback is not None:
-                wfh.write(putback)
-            shutil.copyfileobj(rfh, wfh)
-            wf_name = wfh.name
-        shutil.move(wf_name, ajs.error_file)
-        for line in bad:
-            log.debug('(%s/%s) Job completed, removing SLURM spurious warning: "%s"', ajs.job_wrapper.get_id_tag(), ajs.job_id, line)
