@@ -8,7 +8,7 @@ AST_NODE_TYPE_ALLOWLIST = [
     'Expr', 'Load', 'Str', 'Num', 'BoolOp', 'Compare', 'And', 'Eq', 'NotEq',
     'Or', 'GtE', 'LtE', 'Lt', 'Gt', 'BinOp', 'Add', 'Div', 'Sub', 'Mult', 'Mod',
     'Pow', 'LShift', 'GShift', 'BitAnd', 'BitOr', 'BitXor', 'UnaryOp', 'Invert',
-    'Not', 'NotIn', 'In', 'Is', 'IsNot', 'List', 'Index', 'Subscript',
+    'Not', 'NotIn', 'In', 'Is', 'IsNot', 'List', 'Index', 'Subscript', 'Constant',
     # Further checks
     'Name', 'Call', 'Attribute',
 ]
@@ -137,5 +137,5 @@ def safe_eval(expression, variables):
     True
     """
     if not _check_expression(expression, allowed_variables=list(variables.keys())):
-        raise Exception("Invalid expression [%s], only a very simple subset of Python is allowed." % expression)
+        raise Exception(f"Invalid expression [{expression}], only a very simple subset of Python is allowed.")
     return eval(expression, globals(), variables)
