@@ -1,16 +1,10 @@
 # A simple CLI runner for slurm that can be used when running Galaxy from a
 # non-submit host and using a Slurm cluster.
 from logging import getLogger
+from os import path
 
-from galaxy.jobs import JobState
-try:
-    from galaxy.model import Job
-    job_states = Job.states
-except ImportError:
-    # Not in Galaxy, map Galaxy job states to Pulsar ones.
-    from pulsar.util import enum
-    job_states = enum(RUNNING='running', OK='complete', QUEUED='queued', ERROR="failed")
-from ..job import BaseJobExec
+from ..job import BaseJobExec, job_states
+from ... import runner_states
 
 log = getLogger(__name__)
 
@@ -25,11 +19,6 @@ argmap = {
 
 class LSF(BaseJobExec):
 
-    def __init__(self, **params):
-        self.params = {}
-        for k, v in params.items():
-            self.params[k] = v
-
     def job_script_kwargs(self, ofile, efile, job_name):
         scriptargs = {'-o': ofile,
                       '-e': efile,
@@ -37,22 +26,25 @@ class LSF(BaseJobExec):
 
         # Map arguments using argmap.
         for k, v in self.params.items():
-            if k == 'plugin':
+            if k == 'plugin' or k == 'excluded_hosts':
                 continue
             try:
                 if k == 'memory':
                     # Memory requires both -m and -R rusage[mem=v] request
-                    scriptargs['-R'] = "\"rusage[mem=%s]\"" % v
+                    scriptargs['-R'] = f"\"rusage[mem={v}]\""
                 if not k.startswith('-'):
                     k = argmap[k]
                 scriptargs[k] = v
             except Exception:
-                log.warning('Unrecognized long argument passed to LSF CLI plugin: %s' % k)
+                log.warning(f'Unrecognized long argument passed to LSF CLI plugin: {k}')
 
         # Generated template.
         template_scriptargs = ''
         for k, v in scriptargs.items():
-            template_scriptargs += '#BSUB {} {}\n'.format(k, v)
+            template_scriptargs += f'#BSUB {k} {v}\n'
+        # Excluded hosts use the same -R option already in use for mem, so easier adding here.
+        for host in self._get_excluded_hosts():
+            template_scriptargs += f'#BSUB -R \"select[hname!=\'{host}\']\"\n'
         return dict(headers=template_scriptargs)
 
     def submit(self, script_file):
@@ -63,13 +55,13 @@ class LSF(BaseJobExec):
         return "bsub <%s | awk '{ print $2}' | sed 's/[<>]//g'" % script_file
 
     def delete(self, job_id):
-        return 'bkill %s' % job_id
+        return f'bkill {job_id}'
 
     def get_status(self, job_ids=None):
         return "bjobs -a -o \"id stat\" -noheader"  # check this
 
     def get_single_status(self, job_id):
-        return "bjobs -o stat -noheader " + job_id
+        return f"bjobs -o stat -noheader {job_id}"
 
     def parse_status(self, status, job_ids):
         # Get status for each job, skipping header.
@@ -89,12 +81,12 @@ class LSF(BaseJobExec):
             # which would be badly handled here. So this only works well when Galaxy
             # is constantly monitoring the jobs. The logic here is that DONE jobs get forgotten
             # faster than failed jobs.
-            log.warning("Job id '%s' not found LSF status check" % job_id)
+            log.warning(f"Job id '{job_id}' not found LSF status check")
             return job_states.OK
         return self._get_job_state(status)
 
     def get_failure_reason(self, job_id):
-        return "bjobs -l " + job_id
+        return f"bjobs -l {job_id}"
 
     def parse_failure_reason(self, reason, job_id):
         # LSF will produce the following in the job output file:
@@ -102,7 +94,7 @@ class LSF(BaseJobExec):
         # Exited with exit code 143.
         for line in reason.splitlines():
             if "TERM_MEMLIMIT" in line:
-                return JobState.runner_states.MEMORY_LIMIT_REACHED
+                return runner_states.MEMORY_LIMIT_REACHED
         return None
 
     def _get_job_state(self, state):
@@ -123,7 +115,32 @@ class LSF(BaseJobExec):
                 'ZOMBI': job_states.ERROR
             }.get(state)
         except KeyError:
-            raise KeyError("Failed to map LSF status code [%s] to job state." % state)
+            raise KeyError(f"Failed to map LSF status code [{state}] to job state.")
+
+    def _get_excluded_hosts(self):
+        """
+        Reads a file in the set path with one node name per line. All these nodes will be added
+        to the exclusion list for execution.
+
+        The path can be added to destinations like this:
+
+        <destination id="lsf_8cpu_16GbRam" runner="cli">
+            <param id="shell_plugin">LocalShell</param>
+            <param id="job_plugin">LSF</param>
+            <param id="job_memory">16000</param>
+            <param id="job_cores">7</param>
+            <param id="job_excluded_hosts">/path/to/file/with/hosts/to/exclude/one/per/line.txt</param>
+        </destination>
+
+        :param pathExcludedNodes:
+        :return: list with node names
+        """
+        if "excluded_hosts" in self.params:
+            path_excluded = self.params["excluded_hosts"]
+            if path.isfile(path_excluded):
+                with open(path_excluded) as f:
+                    return f.read().splitlines()
+        return []
 
 
 __all__ = ('LSF',)
