@@ -26,11 +26,12 @@ SUBMIT_PARAM_PREFIX = "submit_"
 
 def submission_params(prefix=SUBMIT_PARAM_PREFIX, **kwds):
     submission_params = {}
+    prefix_len = len(prefix)
     for key in kwds:
         value = kwds[key]
         key = key.lower()
         if key.startswith(prefix):
-            condor_key = key[len(prefix):]
+            condor_key = key[prefix_len:]
             submission_params[condor_key] = value
     return submission_params
 
@@ -58,11 +59,11 @@ def build_submit_description(executable, output, error, user_log, query_params):
 
     submit_description = []
     for key, value in all_query_params.items():
-        submit_description.append('{} = {}'.format(key, value))
-    submit_description.append('executable = ' + executable)
-    submit_description.append('output = ' + output)
-    submit_description.append('error = ' + error)
-    submit_description.append('log = ' + user_log)
+        submit_description.append(f'{key} = {value}')
+    submit_description.append(f"executable = {executable}")
+    submit_description.append(f"output = {output}")
+    submit_description.append(f"error = {error}")
+    submit_description.append(f"log = {user_log}")
     submit_description.append('queue')
     return '\n'.join(submit_description)
 
@@ -73,16 +74,17 @@ def condor_submit(submit_file):
     the submission or return None and a reason for the failure.
     """
     external_id = None
+    failure_message = None
     try:
-        message = commands.execute(('condor_submit', submit_file))
+        condor_message = commands.execute(('condor_submit', submit_file))
     except commands.CommandLineException as e:
-        message = unicodify(e)
+        failure_message = unicodify(e)
     else:
         try:
-            external_id = parse_external_id(message, type='condor')
+            external_id = parse_external_id(condor_message, type='condor')
         except Exception:
-            message = PROBLEM_PARSING_EXTERNAL_ID
-    return external_id, message
+            failure_message = f"{PROBLEM_PARSING_EXTERNAL_ID}: {condor_message}"
+    return external_id, failure_message
 
 
 def condor_stop(external_id):
@@ -96,7 +98,7 @@ def condor_stop(external_id):
     except CalledProcessError:
         failure_message = "condor_rm failed"
     except Exception as e:
-        failure_message = "error encountered calling condor_rm: %s" % unicodify(e)
+        failure_message = f"error encountered calling condor_rm: {unicodify(e)}"
     return failure_message
 
 
@@ -107,15 +109,15 @@ def summarize_condor_log(log_file, external_id):
     s1 = s4 = s7 = s5 = s9 = False
     with open(log_file) as log_handle:
         for line in log_handle:
-            if '001 (' + log_job_id + '.' in line:
+            if f"001 ({log_job_id}." in line:
                 s1 = True
-            if '004 (' + log_job_id + '.' in line:
+            if f"004 ({log_job_id}." in line:
                 s4 = True
-            if '007 (' + log_job_id + '.' in line:
+            if f"007 ({log_job_id}." in line:
                 s7 = True
-            if '005 (' + log_job_id + '.' in line:
+            if f"005 ({log_job_id}." in line:
                 s5 = True
-            if '009 (' + log_job_id + '.' in line:
+            if f"009 ({log_job_id}." in line:
                 s9 = True
         file_size = log_handle.tell()
     return s1, s4, s7, s5, s9, file_size
