@@ -1,9 +1,12 @@
+from galaxy.structured_app import BasicSharedApp
+
+
 class ApiKeyManager:
 
-    def __init__(self, app):
+    def __init__(self, app: BasicSharedApp):
         self.app = app
 
-    def create_api_key(self, user):
+    def create_api_key(self, user) -> str:
         guid = self.app.security.get_new_guid()
         new_key = self.app.model.APIKeys()
         new_key.user_id = user.id
@@ -13,7 +16,7 @@ class ApiKeyManager:
         sa_session.flush()
         return guid
 
-    def get_or_create_api_key(self, user):
+    def get_or_create_api_key(self, user) -> str:
         # Logic Galaxy has always used - but it would appear to have a race
         # condition. Worth fixing? Would kind of need a message queue to fix
         # in multiple process mode.
