@@ -28,18 +28,47 @@ attribute change to a model object.
 import datetime
 import logging
 import re
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generic,
+    List,
+    NamedTuple,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
-import routes
 import sqlalchemy
+from sqlalchemy.orm import Query
+from sqlalchemy.orm.scoping import scoped_session
+from typing_extensions import Protocol
 
 from galaxy import exceptions
 from galaxy import model
 from galaxy.model import tool_shed_install
-from galaxy.util import namedtuple
+from galaxy.schema import FilterQueryParams
+from galaxy.security.idencoding import IdEncodingHelper
+from galaxy.structured_app import BasicSharedApp, MinimalManagerApp
+from galaxy.web import url_for as gx_url_for
 
 log = logging.getLogger(__name__)
 
-parsed_filter = namedtuple("ParsedFilter", "filter_type filter")
+
+class ParsedFilter(NamedTuple):
+    filter_type: str  # orm_function, function, or orm
+    filter: Any
+
+
+parsed_filter = ParsedFilter
+OrmFilterParserType = Union[None, Dict[str, Any], Callable]
+OrmFilterParsersType = Dict[str, OrmFilterParserType]
+FunctionFilterParserType = Dict[str, Any]
+FunctionFilterParsersType = Dict[str, Any]
 
 
 # ==== accessors from base/controller.py
@@ -61,7 +90,7 @@ def security_check(trans, item, check_ownership=False, check_accessible=False):
         if not trans.user:
             raise exceptions.ItemOwnershipException("Must be logged in to manage Galaxy items", type='error')
         if item.user != trans.user:
-            raise exceptions.ItemOwnershipException("%s is not owned by the current user" % item.__class__.__name__, type='error')
+            raise exceptions.ItemOwnershipException(f"{item.__class__.__name__} is not owned by the current user", type='error')
 
     # Verify accessible:
     #   if it's part of a lib - can they access via security
@@ -69,10 +98,10 @@ def security_check(trans, item, check_ownership=False, check_accessible=False):
     if check_accessible:
         if type(item) in (trans.app.model.LibraryFolder, trans.app.model.LibraryDatasetDatasetAssociation, trans.app.model.LibraryDataset):
             if not trans.app.security_agent.can_access_library_item(trans.get_current_user_roles(), item, trans.user):
-                raise exceptions.ItemAccessibilityException("%s is not accessible to the current user" % item.__class__.__name__, type='error')
+                raise exceptions.ItemAccessibilityException(f"{item.__class__.__name__} is not accessible to the current user", type='error')
         else:
             if (item.user != trans.user) and (not item.importable) and (trans.user not in item.users_shared_with_dot_users):
-                raise exceptions.ItemAccessibilityException("%s is not accessible to the current user" % item.__class__.__name__, type='error')
+                raise exceptions.ItemAccessibilityException(f"{item.__class__.__name__} is not accessible to the current user", type='error')
     return item
 
 
@@ -85,19 +114,23 @@ def get_class(class_name):
         item_class = tool_shed_install.ToolShedRepository
     else:
         if not hasattr(model, class_name):
-            raise exceptions.MessageException("Item class '%s' not available." % class_name)
+            raise exceptions.MessageException(f"Item class '{class_name}' not available.")
         item_class = getattr(model, class_name)
     return item_class
 
 
-def decode_id(app, id):
-    try:
-        # note: use str - occasionally a fully numeric id will be placed in post body and parsed as int via JSON
-        #   resulting in error for valid id
-        return app.security.decode_id(str(id))
-    except (ValueError, TypeError):
-        msg = "Malformed id ( %s ) specified, unable to decode" % (str(id))
-        raise exceptions.MalformedId(msg, id=str(id))
+def decode_id(app: BasicSharedApp, id: Any):
+    # note: use str - occasionally a fully numeric id will be placed in post body and parsed as int via JSON
+    #   resulting in error for valid id
+    return decode_with_security(app.security, id)
+
+
+def decode_with_security(security: IdEncodingHelper, id: Any):
+    return security.decode_id(str(id))
+
+
+def encode_with_security(security: IdEncodingHelper, id: Any):
+    return security.encode_id(id)
 
 
 def get_object(trans, id, class_name, check_ownership=False, check_accessible=False, deleted=None):
@@ -114,8 +147,8 @@ def get_object(trans, id, class_name, check_ownership=False, check_accessible=Fa
         item = trans.sa_session.query(item_class).get(decoded_id)
         assert item is not None
     except Exception:
-        log.exception("Invalid {} id ( {} ) specified.".format(class_name, id))
-        raise exceptions.MessageException("Invalid {} id ( {} ) specified".format(class_name, id), type="error")
+        log.exception(f"Invalid {class_name} id ( {id} ) specified.")
+        raise exceptions.MessageException(f"Invalid {class_name} id ( {id} ) specified", type="error")
 
     if check_ownership or check_accessible:
         security_check(trans, item, check_ownership, check_accessible)
@@ -155,20 +188,18 @@ class ModelManager:
     Provides common queries and CRUD operations as a (hopefully) light layer
     over the ORM.
     """
-    model_class = object
-    foreign_key_name = None
+    model_class: Type[model._HasTable]
+    foreign_key_name: str
+    app: BasicSharedApp
 
-    def __init__(self, app):
+    def __init__(self, app: BasicSharedApp):
         self.app = app
 
-    def session(self):
+    def session(self) -> scoped_session:
         return self.app.model.context
 
-    def _session_setattr(self, item, attr, val, fn=None, flush=True):
-        if fn:
-            fn(item, attr, val)
-        else:
-            setattr(item, attr, val)
+    def _session_setattr(self, item: model._HasTable, attr: str, val: Any, flush: bool = True):
+        setattr(item, attr, val)
 
         self.session().add(item)
         if flush:
@@ -176,7 +207,7 @@ class ModelManager:
         return item
 
     # .... query foundation wrapper
-    def query(self, eagerloads=True, **kwargs):
+    def query(self, eagerloads: bool = True, filters=None, order_by=None, limit: Optional[int] = None, offset: Optional[int] = None) -> Query:
         """
         Return a basic query from model_class, filters, order_by, and limit and offset.
 
@@ -186,9 +217,9 @@ class ModelManager:
         # joined table loading
         if eagerloads is False:
             query = query.enable_eagerloads(False)
-        return self._filter_and_order_query(query, **kwargs)
+        return self._filter_and_order_query(query, filters=filters, order_by=order_by, limit=limit, offset=offset)
 
-    def _filter_and_order_query(self, query, filters=None, order_by=None, limit=None, offset=None, **kwargs):
+    def _filter_and_order_query(self, query: Query, filters=None, order_by=None, limit: Optional[int] = None, offset: Optional[int] = None) -> Query:
         # TODO: not a lot of functional cohesion here
         query = self._apply_orm_filters(query, filters)
         query = self._apply_order_by(query, order_by)
@@ -196,7 +227,7 @@ class ModelManager:
         return query
 
     # .... filters
-    def _apply_orm_filters(self, query, filters):
+    def _apply_orm_filters(self, query: Query, filters) -> Query:
         """
         Add any filters to the given query.
         """
@@ -219,7 +250,7 @@ class ModelManager:
         return munge_lists(filtersA, filtersB)
 
     # .... order, limit, and offset
-    def _apply_order_by(self, query, order_by):
+    def _apply_order_by(self, query: Query, order_by) -> Query:
         """
         Return the query after adding the order_by clauses.
 
@@ -236,9 +267,9 @@ class ModelManager:
         """
         Returns a tuple of columns for the default order when getting multiple models.
         """
-        return (self.model_class.create_time, )
+        return (self.model_class.table.c.create_time, )
 
-    def _apply_orm_limit_offset(self, query, limit, offset):
+    def _apply_orm_limit_offset(self, query: Query, limit: Optional[int], offset: Optional[int]) -> Query:
         """
         Return the query after applying the given limit and offset (if not None).
         """
@@ -267,9 +298,9 @@ class ModelManager:
         try:
             return query.one()
         except sqlalchemy.orm.exc.NoResultFound:
-            raise exceptions.ObjectNotFound(self.model_class.__name__ + ' not found')
+            raise exceptions.ObjectNotFound(f"{self.model_class.__name__} not found")
         except sqlalchemy.orm.exc.MultipleResultsFound:
-            raise exceptions.InconsistentDatabase('found more than one ' + self.model_class.__name__)
+            raise exceptions.InconsistentDatabase(f"found more than one {self.model_class.__name__}")
 
     def _one_or_none(self, query):
         """
@@ -283,12 +314,12 @@ class ModelManager:
             return None
 
     # NOTE: at this layer, all ids are expected to be decoded and in int form
-    def by_id(self, id, **kwargs):
+    def by_id(self, id: int):
         """
         Gets a model by primary id.
         """
-        id_filter = self.model_class.id == id
-        return self.one(filters=id_filter, **kwargs)
+        id_filter = self.model_class.table.c.id == id
+        return self.one(filters=id_filter)
 
     # .... multirow queries
     def list(self, filters=None, order_by=None, limit=None, offset=None, **kwargs):
@@ -319,7 +350,8 @@ class ModelManager:
             a list of filters to be added to the SQL query
         and a list of functional filters to be applied after the SQL query.
         """
-        orm_filters, fn_filters = ([], [])
+        orm_filters: list = []
+        fn_filters: list = []
         if filters is None:
             return (orm_filters, fn_filters)
         if not isinstance(filters, list):
@@ -380,7 +412,7 @@ class ModelManager:
         """
         if not ids:
             return []
-        ids_filter = parsed_filter("orm", self.model_class.id.in_(ids))
+        ids_filter = parsed_filter("orm", self.model_class.table.c.id.in_(ids))
         found = self.list(filters=self._munge_filters(ids_filter, filters), **kwargs)
         # TODO: this does not order by the original 'ids' array
 
@@ -470,9 +502,12 @@ class ModelManager:
     #    return item
 
 
+T = TypeVar('T')
+
+
 # ---- code for classes that use one *main* model manager
 # TODO: this may become unecessary if we can access managers some other way (class var, app, etc.)
-class HasAModelManager:
+class HasAModelManager(Generic[T]):
     """
     Mixin used where serializers, deserializers, filter parsers, etc.
     need some functionality around the model they're mainly concerned with
@@ -480,20 +515,22 @@ class HasAModelManager:
     """
 
     #: the class used to create this serializer's generically accessible model_manager
-    model_manager_class = None
+    model_manager_class: Type[T]  # ideally this would be Type[ModelManager] but HistoryContentsManager cannot be a ModelManager
     # examples where this doesn't really work are ConfigurationSerializer (no manager)
     # and contents (2 managers)
+    app: MinimalManagerApp
 
-    def __init__(self, app, manager=None, **kwargs):
+    def __init__(self, app: MinimalManagerApp, manager=None, **kwargs):
         self._manager = manager
+        self.app = app
 
     @property
-    def manager(self):
+    def manager(self) -> T:
         """Return an appropriate manager if it exists, instantiate if not."""
         # PRECONDITION: assumes self.app is assigned elsewhere
         if not self._manager:
             # TODO: pass this serializer to it
-            self._manager = self.model_manager_class(self.app)
+            self._manager = self.app[self.model_manager_class]
             # this will error for unset model_manager_class'es
         return self._manager
 
@@ -516,7 +553,13 @@ class SkipAttribute(Exception):
     """
 
 
-class ModelSerializer(HasAModelManager):
+class Serializer(Protocol):
+
+    def __call__(self, item: Any, key: str, **context) -> Any:
+        ...
+
+
+class ModelSerializer(HasAModelManager[T]):
     """
     Turns models into JSONable dicts.
 
@@ -533,23 +576,22 @@ class ModelSerializer(HasAModelManager):
         keys_to_serialize = [ 'id', 'name', 'attr1', 'attr2', ... ]
         item_dict = MySerializer.serialize( my_item, keys_to_serialize )
     """
-    #: 'service' to use for getting urls - use class var to allow overriding when testing
-    url_for = staticmethod(routes.url_for)
+    default_view: Optional[str]
+    views: Dict[str, List[str]]
 
-    def __init__(self, app, **kwargs):
+    def __init__(self, app: MinimalManagerApp, **kwargs):
         """
         Set up serializer map, any additional serializable keys, and views here.
         """
         super().__init__(app, **kwargs)
-        self.app = app
 
         # a list of valid serializable keys that can use the default (string) serializer
         #   this allows us to: 'mention' the key without adding the default serializer
         # TODO: we may want to eventually error if a key is requested
         #   that is in neither serializable_keyset or serializers
-        self.serializable_keyset = set()
+        self.serializable_keyset: Set[str] = set()
         # a map of dictionary keys to the functions (often lambdas) that create the values for those keys
-        self.serializers = {}
+        self.serializers: Dict[str, Serializer] = {}
         # add subclass serializers defined there
         self.add_serializers()
         # update the keyset by the serializers (removing the responsibility from subclasses)
@@ -560,15 +602,21 @@ class ModelSerializer(HasAModelManager):
         self.views = {}
         self.default_view = None
 
+    @staticmethod
+    def url_for(*args, context=None, **kwargs):
+        trans = context and context.get('trans')
+        url_for = trans and trans.url_builder or gx_url_for
+        return url_for(*args, **kwargs)
+
     def add_serializers(self):
         """
         Register a map of attribute keys -> serializing functions that will serialize
         the attribute.
         """
         self.serializers.update({
-            'id'            : self.serialize_id,
-            'create_time'   : self.serialize_date,
-            'update_time'   : self.serialize_date,
+            'id': self.serialize_id,
+            'create_time': self.serialize_date,
+            'update_time': self.serialize_date,
         })
 
     def add_view(self, view_name, key_list, include_keys_from=None):
@@ -620,7 +668,7 @@ class ModelSerializer(HasAModelManager):
             return self.serializers[original_key]
         if original_key in self.serializable_keyset:
             return lambda i, k, **c: self.default_serializer(i, original_key, **c)
-        raise KeyError('serializer not found for remap: ' + original_key)
+        raise KeyError(f"serializer not found for remap: {original_key}")
 
     def default_serializer(self, item, key, **context):
         """
@@ -630,14 +678,14 @@ class ModelSerializer(HasAModelManager):
         return getattr(item, key)
 
     # serializers for common galaxy objects
-    def serialize_date(self, item, key, **context):
+    def serialize_date(self, item: Any, key: str, **context):
         """
         Serialize a date attribute of `item`.
         """
         date = getattr(item, key)
         return date.isoformat() if date is not None else None
 
-    def serialize_id(self, item, key, **context):
+    def serialize_id(self, item: Any, key: str, **context):
         """
         Serialize an id attribute of `item`.
         """
@@ -645,7 +693,7 @@ class ModelSerializer(HasAModelManager):
         # Note: it may not be best to encode the id at this layer
         return self.app.security.encode_id(id) if id is not None else None
 
-    def serialize_type_id(self, item, key, **context):
+    def serialize_type_id(self, item: Any, key: str, **context):
         """
         Serialize an type-id for `item`.
         """
@@ -700,25 +748,114 @@ class ModelSerializer(HasAModelManager):
         return self.views[view][:]
 
 
-class ModelDeserializer(HasAModelManager):
+class ModelValidator:
+    """
+    An object that inspects a dictionary (generally meant to be a set of
+    new/updated values for the model) and raises an error if a value is
+    not acceptable.
+    """
+
+    @staticmethod
+    def matches_type(key: str, val: Any, types: Union[type, Tuple[Union[type, Tuple[Any, ...]], ...]]):
+        """
+        Check `val` against the type (or tuple of types) in `types`.
+
+        :raises exceptions.RequestParameterInvalidException: if not an instance.
+        """
+        if not isinstance(val, types):
+            msg = f'must be a type: {str(types)}'
+            raise exceptions.RequestParameterInvalidException(msg, key=key, val=val)
+        return val
+
+    # validators for primitives and compounds of primitives
+    @staticmethod
+    def basestring(key: str, val: Any) -> str:
+        return ModelValidator.matches_type(key, val, (str,))
+
+    @staticmethod
+    def bool(key: str, val: Any) -> bool:
+        return ModelValidator.matches_type(key, val, bool)
+
+    @staticmethod
+    def nullable_basestring(key: str, val: Any) -> str:
+        """
+        Must be a basestring or None.
+        """
+        return ModelValidator.matches_type(key, val, ((str,), type(None)))
+
+    @staticmethod
+    def int_range(key: str, val: Any, min: Optional[int] = None, max: Optional[int] = None) -> int:
+        """
+        Must be a int between min and max.
+        """
+        val_ = ModelValidator.matches_type(key, val, int)
+        if min is not None and val_ < min:
+            raise exceptions.RequestParameterInvalidException("less than minimum", key=key, val=val_, min=min)
+        if max is not None and val_ > max:
+            raise exceptions.RequestParameterInvalidException("greater than maximum", key=key, val=val_, max=max)
+        return val_
+
+    @staticmethod
+    def basestring_list(key: str, val: Any) -> List[str]:
+        """
+        Must be a list of basestrings.
+        """
+        # TODO: Here's where compound types start becoming a nightmare. Any more or more complex
+        #   and should find a different way.
+        val_ = ModelValidator.matches_type(key, val, list)
+        return [ModelValidator.basestring(key, elem) for elem in val_]
+
+    # validators for Galaxy
+    @staticmethod
+    def genome_build(key: str, val: Any) -> str:
+        """
+        Must be a valid base_string.
+
+        Note: no checking against installation's ref list is done as many
+        data sources consider this an open field.
+        """
+        # TODO: is this correct?
+        if val is None:
+            return '?'
+        # currently, data source sites like UCSC are able to set the genome build to non-local build names
+        # afterwards, attempting to validate the whole model will choke here
+        # for genome_build_shortname, longname in self.app.genome_builds.get_genome_build_names( trans=trans ):
+        #     if val == genome_build_shortname:
+        #         return val
+        # raise exceptions.RequestParameterInvalidException( "invalid reference", key=key, val=val )
+        # IOW: fallback to string validation
+        return ModelValidator.basestring(key, val)
+
+    # def slug( self, item, key, val ):
+    #    """validate slug"""
+    #    pass
+
+
+class Deserializer(Protocol):
+
+    def __call__(self, item: Any, key: Any, val: Any, **kwargs) -> Any:
+        ...
+
+
+class ModelDeserializer(HasAModelManager[T]):
     """
     An object that converts an incoming serialized dict into values that can be
     directly assigned to an item's attributes and assigns them.
     """
+    validate = ModelValidator()
+    app: MinimalManagerApp
+
     # TODO:?? a larger question is: which should be first? Deserialize then validate - or - validate then deserialize?
 
-    def __init__(self, app, validator=None, **kwargs):
+    def __init__(self, app: MinimalManagerApp, **kwargs):
         """
         Set up deserializers and validator.
         """
         super().__init__(app, **kwargs)
-        self.app = app
 
-        self.deserializers = {}
-        self.deserializable_keyset = set()
+        self.deserializers: Dict[str, Deserializer] = {}
+        self.deserializable_keyset: Set[str] = set()
         self.add_deserializers()
-        # a sub object that can validate incoming values
-        self.validate = validator or ModelValidator(self.app)
 
     def add_deserializers(self):
         """
@@ -784,89 +921,6 @@ class ModelDeserializer(HasAModelManager):
         return self.default_deserializer(item, key, val, **context)
 
 
-class ModelValidator(HasAModelManager):
-    """
-    An object that inspects a dictionary (generally meant to be a set of
-    new/updated values for the model) and raises an error if a value is
-    not acceptable.
-    """
-
-    def __init__(self, app, *args, **kwargs):
-        super().__init__(app, **kwargs)
-        self.app = app
-
-    def type(self, key, val, types):
-        """
-        Check `val` against the type (or tuple of types) in `types`.
-
-        :raises exceptions.RequestParameterInvalidException: if not an instance.
-        """
-        if not isinstance(val, types):
-            msg = 'must be a type: %s' % (str(types))
-            raise exceptions.RequestParameterInvalidException(msg, key=key, val=val)
-        return val
-
-    # validators for primitives and compounds of primitives
-    def basestring(self, key, val):
-        return self.type(key, val, (str,))
-
-    def bool(self, key, val):
-        return self.type(key, val, bool)
-
-    def int(self, key, val):
-        return self.type(key, val, int)
-
-    def nullable_basestring(self, key, val):
-        """
-        Must be a basestring or None.
-        """
-        return self.type(key, val, ((str,), type(None)))
-
-    def int_range(self, key, val, min=None, max=None):
-        """
-        Must be a int between min and max.
-        """
-        val = self.type(key, val, int)
-        if min is not None and val < min:
-            raise exceptions.RequestParameterInvalidException("less than minimum", key=key, val=val, min=min)
-        if max is not None and val > max:
-            raise exceptions.RequestParameterInvalidException("greater than maximum", key=key, val=val, max=max)
-        return val
-
-    def basestring_list(self, key, val):
-        """
-        Must be a list of basestrings.
-        """
-        # TODO: Here's where compound types start becoming a nightmare. Any more or more complex
-        #   and should find a different way.
-        val = self.type(key, val, list)
-        return [self.basestring(key, elem) for elem in val]
-
-    # validators for Galaxy
-    def genome_build(self, key, val):
-        """
-        Must be a valid base_string.
-
-        Note: no checking against installation's ref list is done as many
-        data sources consider this an open field.
-        """
-        # TODO: is this correct?
-        if val is None:
-            return '?'
-        # currently, data source sites like UCSC are able to set the genome build to non-local build names
-        # afterwards, attempting to validate the whole model will choke here
-        # for genome_build_shortname, longname in self.app.genome_builds.get_genome_build_names( trans=trans ):
-        #     if val == genome_build_shortname:
-        #         return val
-        # raise exceptions.RequestParameterInvalidException( "invalid reference", key=key, val=val )
-        # IOW: fallback to string validation
-        return self.basestring(key, val)
-
-    # def slug( self, item, key, val ):
-    #    """validate slug"""
-    #    pass
-
-
 # ==== Building query filters based on model data
 class ModelFilterParser(HasAModelManager):
     """
@@ -893,17 +947,16 @@ class ModelFilterParser(HasAModelManager):
     # (as the model informs how the filter params are parsed)
     # I have no great idea where this 'belongs', so it's here for now
 
-    #: model class
-    model_class = None
-    subcontainer_model_class = None
+    model_class: Type[model._HasTable]
     parsed_filter = parsed_filter
+    orm_filter_parsers: OrmFilterParsersType
+    fn_filter_parsers: FunctionFilterParsersType
 
-    def __init__(self, app, **kwargs):
+    def __init__(self, app: MinimalManagerApp, **kwargs):
         """
         Set up serializer map, any additional serializable keys, and views here.
         """
         super().__init__(app, **kwargs)
-        self.app = app
 
         #: regex for testing/dicing iso8601 date strings, with optional time and ms, but allowing only UTC timezone
         self.date_string_re = re.compile(r'^(\d{4}\-\d{2}\-\d{2})[T| ]{0,1}(\d{2}:\d{2}:\d{2}(?:\.\d{1,6}){0,1}){0,1}Z{0,1}$')
@@ -924,15 +977,60 @@ class ModelFilterParser(HasAModelManager):
         Set up, extend, or alter `orm_filter_parsers` and `fn_filter_parsers`.
         """
         # note: these are the default filters for all models
-        self.orm_filter_parsers.update({
-            # (prob.) applicable to all models
-            'id'            : {'op': ('in')},
-            'encoded_id'    : {'column' : 'id', 'op': ('in'), 'val': self.parse_id_list},
-            # dates can be directly passed through the orm into a filter (no need to parse into datetime object)
-            'extension'     : {'op': ('eq', 'like', 'in')},
-            'create_time'   : {'op': ('le', 'ge', 'lt', 'gt'), 'val': self.parse_date},
-            'update_time'   : {'op': ('le', 'ge', 'lt', 'gt'), 'val': self.parse_date},
-        })
+        self.orm_filter_parsers.update(
+            {
+                # (prob.) applicable to all models
+                "id": {"op": ("in")},
+                "encoded_id": {"column": "id", "op": ("in"), "val": self.parse_id_list},
+                # dates can be directly passed through the orm into a filter (no need to parse into datetime object)
+                "extension": {"op": ("eq", "like", "in"), "val": {"in": lambda v: v.split(",")}},
+                "create_time": {"op": ("le", "ge", "lt", "gt"), "val": self.parse_date},
+                "update_time": {"op": ("le", "ge", "lt", "gt"), "val": self.parse_date},
+            }
+        )
+
+    def build_filter_params(
+        self,
+        query_params: FilterQueryParams,
+        filter_attr_key: str = 'q',
+        filter_value_key: str = 'qv',
+        attr_op_split_char: str = '-',
+    ) -> List[Tuple[str, str, str]]:
+        """
+        Builds a list of tuples containing filtering information in the form of (attribute, operator, value).
+        """
+        DEFAULT_OP = 'eq'
+        qdict = query_params.dict(exclude_defaults=True)
+        if filter_attr_key not in qdict:
+            return []
+        # precondition: attrs/value pairs are in-order in the qstring
+        attrs = qdict.get(filter_attr_key)
+        if not isinstance(attrs, list):
+            attrs = [attrs]
+        # ops are strings placed after the attr strings and separated by a split char (e.g. 'create_time-lt')
+        # ops are optional and default to 'eq'
+        reparsed_attrs = []
+        ops = []
+        for attr in attrs:
+            op = DEFAULT_OP
+            if attr_op_split_char in attr:
+                # note: only split the last (e.g. q=community-tags-in&qv=rna yields ( 'community-tags', 'in', 'rna' )
+                attr, op = attr.rsplit(attr_op_split_char, 1)
+            ops.append(op)
+            reparsed_attrs.append(attr)
+        attrs = reparsed_attrs
+
+        values = qdict.get(filter_value_key, [])
+        if not isinstance(values, list):
+            values = [values]
+        # TODO: it may be more helpful to the consumer if we error on incomplete 3-tuples
+        #   (instead of relying on zip to shorten)
+        return list(zip(attrs, ops, values))
+
+    def parse_query_filters(self, query_filters: FilterQueryParams):
+        """Convenience function to parse a FilterQueryParams object into a collection of filtering criteria."""
+        filter_params = self.build_filter_params(query_filters)
+        return self.parse_filters(filter_params)
 
     def parse_filters(self, filter_tuple_list):
         """
@@ -979,13 +1077,13 @@ class ModelFilterParser(HasAModelManager):
         """
         Attempt to parse a non-ORM filter function.
         """
-        # fn_filter_list is a dict: fn_filter_list[ attr ] = { 'opname1' : opfn1, 'opname2' : opfn2, etc. }
+        # fn_filter_parsers is a dict: fn_filter_parsers[attr] = {"opname1": opfn1, "opname2": opfn2, etc. }
 
         # attr, op is a nested dictionary pointing to the filter fn
         attr_map = self.fn_filter_parsers.get(attr, None)
         if not attr_map:
             return None
-        allowed_ops = attr_map.get('op')
+        allowed_ops = attr_map['op']
         # allowed ops is a map here, op => fn
         filter_fn = allowed_ops.get(op, None)
         if not filter_fn:
@@ -999,13 +1097,13 @@ class ModelFilterParser(HasAModelManager):
         return self.parsed_filter(filter_type="function", filter=lambda i: filter_fn(i, val))
 
     # ---- ORM filters
-    def _parse_orm_filter(self, attr, op, val):
+    def _parse_orm_filter(self, attr, op, val) -> Optional[ParsedFilter]:
         """
         Attempt to parse a ORM-based filter.
 
         Using SQLAlchemy, this would yield a sql.elements.BinaryExpression.
         """
-        # orm_filter_list is a dict: orm_filter_list[ attr ] = <list of allowed ops>
+        # orm_filter_parsers is a dict: orm_filter_parsers[attr] = <column map>
         column_map = self.orm_filter_parsers.get(attr, None)
         if not column_map:
             # no column mapping (not allowlisted)
@@ -1025,19 +1123,23 @@ class ModelFilterParser(HasAModelManager):
             return None
 
         # op must be allowlisted: contained in the list orm_filter_list[ attr ][ 'op' ]
-        allowed_ops = column_map.get('op')
+        allowed_ops = column_map['op']
         if op not in allowed_ops:
             return None
-        op = self._convert_op_string_to_fn(column, op)
-        if not op:
+        converted_op = self._convert_op_string_to_fn(column, op)
+        if not converted_op:
             return None
 
         # parse the val from string using the 'val' parser if present (otherwise, leave as string)
-        val_parser = column_map.get('val', None)
+        val_parser = column_map.get("val")
+        # val_parser can be a dictionary indexed by the operations, in case different functions
+        # need to be called depending on the operation
+        if isinstance(val_parser, dict):
+            val_parser = val_parser.get(op)
         if val_parser:
             val = val_parser(val)
 
-        orm_filter = op(val)
+        orm_filter = converted_op(val)
         return self.parsed_filter(filter_type="orm", filter=orm_filter)
 
     #: these are the easier/shorter string equivalents to the python operator fn names that need '__' around them
@@ -1051,7 +1153,7 @@ class ModelFilterParser(HasAModelManager):
         # correct op_string to usable function key
         fn_name = op_string
         if op_string in self.UNDERSCORED_OPS:
-            fn_name = '__' + op_string + '__'
+            fn_name = f"__{op_string}__"
         elif op_string == 'in':
             fn_name = 'in_'
 
@@ -1065,28 +1167,17 @@ class ModelFilterParser(HasAModelManager):
     # ---- preset fn_filters: dictionaries of standard filter ops for standard datatypes
     def string_standard_ops(self, key):
         return {
-            'op' : {
-                'eq'        : lambda i, v: v == getattr(i, key),
-                'contains'  : lambda i, v: v in getattr(i, key),
+            'op': {
+                'eq': lambda i, v: v == getattr(i, key),
+                'contains': lambda i, v: v in getattr(i, key),
             }
         }
 
     # --- more parsers! yay!
     # TODO: These should go somewhere central - we've got ~6 parser modules/sections now
-    def parse_bool(self, bool_string):
-        """
-        Parse a boolean from a string.
-        """
-        # Be strict here to remove complexity of options (but allow already parsed).
-        if bool_string in ('True', True):
-            return True
-        if bool_string in ('False', False):
-            return False
-        raise ValueError('invalid boolean: ' + str(bool_string))
-
     def parse_id_list(self, id_list_string, sep=','):
         """
-        Split `id_list_string` at `sep`.
+        Split `id_list_string` at `sep` and decode as ids.
         """
         # TODO: move id decoding out
         id_list = [self.app.security.decode_id(id_) for id_ in id_list_string.split(sep)]
@@ -1121,8 +1212,21 @@ class ModelFilterParser(HasAModelManager):
             return date_string
         raise ValueError('datetime strings must be in the ISO 8601 format and in the UTC')
 
-    def raise_filter_err(self, attr, op, val, msg):
-        raise exceptions.RequestParameterInvalidException(msg, column=attr, operation=op, val=val)
+
+def parse_bool(bool_string: Union[str, bool]) -> bool:
+    """
+    Parse a boolean from a string.
+    """
+    # Be strict here to remove complexity of options (but allow already parsed).
+    if bool_string in ('True', True):
+        return True
+    if bool_string in ('False', False):
+        return False
+    raise ValueError(f"invalid boolean: {str(bool_string)}")
+
+
+def raise_filter_err(attr, op, val, msg):
+    raise exceptions.RequestParameterInvalidException(msg, column=attr, operation=op, val=val)
 
 
 def is_valid_slug(slug):
@@ -1130,3 +1234,12 @@ def is_valid_slug(slug):
 
     VALID_SLUG_RE = re.compile(r"^[a-z0-9\-]+$")
     return VALID_SLUG_RE.match(slug)
+
+
+class SortableManager:
+    """A manager interface for parsing order_by strings into actual 'order by' queries."""
+
+    def parse_order_by(self, order_by_string, default=None):
+        """Return an ORM compatible order_by clause using the given string (i.e.: 'name-dsc,create_time').
+        This must be implemented by the manager."""
+        raise NotImplementedError
