@@ -50,9 +50,6 @@ class CloudManager(sharable.SharableModelManager):
     # setting this property.
     model_class = model.History
 
-    def __init__(self, app, *args, **kwargs):
-        super().__init__(app, *args, **kwargs)
-
     @staticmethod
     def configure_provider(provider, credentials):
         """
@@ -61,11 +58,11 @@ class CloudManager(sharable.SharableModelManager):
 
         :type  provider: string
         :param provider: the name of cloud-based resource provided. A list of supported providers is given in
-        `SUPPORTED_PROVIDERS` variable.
+                         `SUPPORTED_PROVIDERS` variable.
 
         :type  credentials: dict
         :param credentials: a dictionary containing all the credentials required to authenticated to the
-        specified provider.
+                            specified provider.
 
         :rtype: provider specific, e.g., `cloudbridge.cloud.providers.aws.provider.AWSCloudProvider` for AWS.
         :return: a cloudbridge connection to the specified provider.
@@ -248,16 +245,16 @@ class CloudManager(sharable.SharableModelManager):
         try:
             bucket = connection.storage.buckets.get(bucket_name)
             if bucket is None:
-                raise RequestParameterInvalidException("The bucket `{}` not found.".format(bucket_name))
+                raise RequestParameterInvalidException(f"The bucket `{bucket_name}` not found.")
         except Exception as e:
-            raise ItemAccessibilityException("Could not get the bucket `{}`: {}".format(bucket_name, util.unicodify(e)))
+            raise ItemAccessibilityException(f"Could not get the bucket `{bucket_name}`: {util.unicodify(e)}")
 
         datasets = []
         for obj in objects:
             try:
                 key = bucket.objects.get(obj)
             except Exception as e:
-                raise MessageException("The following error occurred while getting the object {}: {}".format(obj, util.unicodify(e)))
+                raise MessageException(f"The following error occurred while getting the object {obj}: {util.unicodify(e)}")
             if key is None:
                 log.exception(
                     "Could not get object `{}` for user `{}`. Object may not exist, or the provided credentials are "
@@ -270,7 +267,7 @@ class CloudManager(sharable.SharableModelManager):
             incoming = params.__dict__
             history = trans.sa_session.query(trans.app.model.History).get(history_id)
             if not history:
-                raise ObjectNotFound("History with ID `{}` not found.".format(trans.app.security.encode_id(history_id)))
+                raise ObjectNotFound(f"History with ID `{trans.app.security.encode_id(history_id)}` not found.")
             output = trans.app.toolbox.get_tool('upload1').handle_input(trans, incoming, history=history)
 
             job_errors = output.get('job_errors', [])
@@ -332,7 +329,7 @@ class CloudManager(sharable.SharableModelManager):
 
         history = trans.sa_session.query(trans.app.model.History).get(history_id)
         if not history:
-            raise ObjectNotFound("History with ID `{}` not found.".format(trans.app.security.encode_id(history_id)))
+            raise ObjectNotFound(f"History with ID `{trans.app.security.encode_id(history_id)}` not found.")
 
         sent = []
         failed = []
@@ -354,7 +351,7 @@ class CloudManager(sharable.SharableModelManager):
                     incoming = (util.Params(args, sanitize=False)).__dict__
                     d2c = trans.app.toolbox.get_tool(SEND_TOOL, SEND_TOOL_VERSION)
                     if not d2c:
-                        log.debug("Failed to get the `send` tool per user `{}` request.".format(trans.user.id))
+                        log.debug(f"Failed to get the `send` tool per user `{trans.user.id}` request.")
                         failed.append(json.dumps(
                             {
                                 "object": object_label,
@@ -369,7 +366,7 @@ class CloudManager(sharable.SharableModelManager):
                             "job_id": trans.app.security.encode_id(job.id)
                         }))
                 except Exception as e:
-                    err_msg = "maybe invalid or unauthorized credentials. {}".format(util.unicodify(e))
+                    err_msg = f"maybe invalid or unauthorized credentials. {util.unicodify(e)}"
                     log.debug("Failed to send the dataset `{}` per user `{}` request to cloud, {}".format(
                         object_label, trans.user.id, err_msg))
                     failed.append(json.dumps(
