@@ -4,6 +4,7 @@ Manager and Serializer for Datasets.
 import glob
 import logging
 import os
+from typing import Dict, List, Type, TypeVar
 
 from galaxy import (
     exceptions,
@@ -17,10 +18,13 @@ from galaxy.managers import (
     secured,
     users
 )
+from galaxy.structured_app import MinimalManagerApp
 from galaxy.util.checkers import check_binary
 
 log = logging.getLogger(__name__)
 
+T = TypeVar('T')
+
 
 class DatasetManager(base.ModelManager, secured.AccessibleManagerMixin, deletable.PurgableManagerMixin):
     """
@@ -28,10 +32,11 @@ class DatasetManager(base.ModelManager, secured.AccessibleManagerMixin, deletabl
     """
     model_class = model.Dataset
     foreign_key_name = 'dataset'
+    app: MinimalManagerApp
 
     # TODO:?? get + error_if_uploading is common pattern, should upload check be worked into access/owed?
 
-    def __init__(self, app):
+    def __init__(self, app: MinimalManagerApp):
         super().__init__(app)
         self.permissions = DatasetRBACPermissions(app)
         # needed for admin test
@@ -85,7 +90,7 @@ class DatasetManager(base.ModelManager, secured.AccessibleManagerMixin, deletabl
         """
         Is this dataset readable/viewable to user?
         """
-        if self.user_manager.is_admin(user, trans=kwargs.get("trans", None)):
+        if self.user_manager.is_admin(user, trans=kwargs.get("trans")):
             return True
         if self.has_access_permission(dataset, user):
             return True
@@ -138,14 +143,14 @@ class DatasetRBACPermissions:
         return ([manage], access)
 
 
-class DatasetSerializer(base.ModelSerializer, deletable.PurgableSerializerMixin):
+class DatasetSerializer(base.ModelSerializer[DatasetManager], deletable.PurgableSerializerMixin):
     model_manager_class = DatasetManager
 
-    def __init__(self, app):
+    def __init__(self, app: MinimalManagerApp, user_manager: users.UserManager):
         super().__init__(app)
         self.dataset_manager = self.manager
         # needed for admin test
-        self.user_manager = users.UserManager(app)
+        self.user_manager = user_manager
 
         self.default_view = 'summary'
         self.add_view('summary', [
@@ -168,55 +173,58 @@ class DatasetSerializer(base.ModelSerializer, deletable.PurgableSerializerMixin)
     def add_serializers(self):
         super().add_serializers()
         deletable.PurgableSerializerMixin.add_serializers(self)
+        serializers: Dict[str, base.Serializer] = {
+            'create_time': self.serialize_date,
+            'update_time': self.serialize_date,
 
-        self.serializers.update({
-            'create_time'   : self.serialize_date,
-            'update_time'   : self.serialize_date,
+            'uuid': lambda item, key, **context: str(item.uuid) if item.uuid else None,
+            'file_name': self.serialize_file_name,
+            'extra_files_path': self.serialize_extra_files_path,
+            'permissions': self.serialize_permissions,
 
-            'uuid'          : lambda i, k, **c: str(i.uuid) if i.uuid else None,
-            'file_name'     : self.serialize_file_name,
-            'extra_files_path' : self.serialize_extra_files_path,
-            'permissions'   : self.serialize_permissions,
+            'total_size': lambda item, key, **context: int(item.get_total_size()),
+            'file_size': lambda item, key, **context: int(item.get_size())
+        }
+        self.serializers.update(serializers)
 
-            'total_size'    : lambda i, k, **c: int(i.get_total_size()),
-            'file_size'     : lambda i, k, **c: int(i.get_size())
-        })
-
-    def serialize_file_name(self, dataset, key, user=None, **context):
+    def serialize_file_name(self, item, key, user=None, **context):
         """
         If the config allows or the user is admin, return the file name
         of the file that contains this dataset's data.
         """
-        is_admin = self.user_manager.is_admin(user, trans=context.get("trans", None))
+        dataset = item
+        is_admin = self.user_manager.is_admin(user, trans=context.get("trans"))
         # expensive: allow config option due to cost of operation
         if is_admin or self.app.config.expose_dataset_path:
             if not dataset.purged:
                 return dataset.file_name
         self.skip()
 
-    def serialize_extra_files_path(self, dataset, key, user=None, **context):
+    def serialize_extra_files_path(self, item, key, user=None, **context):
         """
         If the config allows or the user is admin, return the file path.
         """
-        is_admin = self.user_manager.is_admin(user, trans=context.get("trans", None))
+        dataset = item
+        is_admin = self.user_manager.is_admin(user, trans=context.get("trans"))
         # expensive: allow config option due to cost of operation
         if is_admin or self.app.config.expose_dataset_path:
             if not dataset.purged:
                 return dataset.extra_files_path
         self.skip()
 
-    def serialize_permissions(self, dataset, key, user=None, **context):
+    def serialize_permissions(self, item, key, user=None, **context):
         """
         """
-        trans = context.get("trans", None)
+        dataset = item
+        trans = context.get("trans")
         if not self.dataset_manager.permissions.manage.is_permitted(dataset, user, trans=trans):
             self.skip()
 
         management_permissions = self.dataset_manager.permissions.manage.by_dataset(dataset)
         access_permissions = self.dataset_manager.permissions.access.by_dataset(dataset)
         permissions = {
-            'manage' : [self.app.security.encode_id(perm.role.id) for perm in management_permissions],
-            'access' : [self.app.security.encode_id(perm.role.id) for perm in access_permissions],
+            'manage': [self.app.security.encode_id(perm.role.id) for perm in management_permissions],
+            'access': [self.app.security.encode_id(perm.role.id) for perm in access_permissions],
         }
         return permissions
 
@@ -232,7 +240,8 @@ class DatasetAssociationManager(base.ModelManager,
     """
     # DA's were meant to be proxies - but were never fully implemented as them
     # Instead, a dataset association HAS a dataset but contains metadata specific to a library (lda) or user (hda)
-    model_class = model.DatasetInstance
+    model_class: Type[model.DatasetInstance]
+    app: MinimalManagerApp
 
     # NOTE: model_manager_class should be set in HDA/LDA subclasses
 
@@ -300,8 +309,10 @@ class DatasetAssociationManager(base.ModelManager,
             if not job.finished:
                 # Are *all* of the job's other output datasets deleted?
                 if job.check_if_output_datasets_deleted():
-                    job.mark_deleted(self.app.config.track_jobs_in_database)
-                    self.app.job_manager.stop(job)
+                    track_jobs_in_database = self.app.config.track_jobs_in_database
+                    job.mark_deleted(track_jobs_in_database)
+                    if not track_jobs_in_database:
+                        self.app.job_manager.stop(job)
                     return True
         return False
 
@@ -328,34 +339,24 @@ class DatasetAssociationManager(base.ModelManager,
             dataset = dataset_assoc.dataset
 
         # Omit duplicated roles by converting to set
-        access_roles = set(dataset.get_access_roles(trans))
-        manage_roles = set(dataset.get_manage_permissions_roles(trans))
+        security_agent = trans.app.security_agent
+        access_roles = set(dataset.get_access_roles(security_agent))
+        manage_roles = set(dataset.get_manage_permissions_roles(security_agent))
 
         access_dataset_role_list = [(access_role.name, trans.security.encode_id(access_role.id)) for access_role in access_roles]
         manage_dataset_role_list = [(manage_role.name, trans.security.encode_id(manage_role.id)) for manage_role in manage_roles]
         rval = dict(access_dataset_roles=access_dataset_role_list, manage_dataset_roles=manage_dataset_role_list)
         if library_dataset is not None:
-            modify_roles = set(trans.app.security_agent.get_roles_for_action(library_dataset, trans.app.security_agent.permitted_actions.LIBRARY_MODIFY))
+            modify_roles = set(security_agent.get_roles_for_action(library_dataset, trans.app.security_agent.permitted_actions.LIBRARY_MODIFY))
             modify_item_role_list = [(modify_role.name, trans.security.encode_id(modify_role.id)) for modify_role in modify_roles]
             rval["modify_item_roles"] = modify_item_role_list
         return rval
 
-    def ok_to_edit_metadata(self, dataset_id):
-        # prevent modifying metadata when dataset is queued or running as input/output
-        # This code could be more efficient, i.e. by using mappers, but to prevent slowing down loading a History panel, we'll leave the code here for now
-        sa_session = self.app.model.context
-        for job_to_dataset_association in sa_session.query(
-                self.app.model.JobToInputDatasetAssociation).filter_by(dataset_id=dataset_id).all() \
-                + sa_session.query(self.app.model.JobToOutputDatasetAssociation).filter_by(dataset_id=dataset_id).all():
-            if job_to_dataset_association.job.state not in [job_to_dataset_association.job.states.OK, job_to_dataset_association.job.states.ERROR, job_to_dataset_association.job.states.DELETED]:
-                return False
-        return True
-
     def detect_datatype(self, trans, dataset_assoc):
         """Sniff and assign the datatype to a given dataset association (ldda or hda)"""
         data = trans.sa_session.query(self.model_class).get(dataset_assoc.id)
-        if data.datatype.allow_datatype_change:
-            if not self.ok_to_edit_metadata(data.id):
+        if data.datatype.is_datatype_change_allowed():
+            if not data.ok_to_edit_metadata():
                 raise exceptions.ItemAccessibilityException('This dataset is currently being used as input or output. You cannot change datatype until the jobs have completed or you have canceled them.')
             else:
                 path = data.dataset.file_name
@@ -365,12 +366,12 @@ class DatasetAssociationManager(base.ModelManager,
                 trans.sa_session.flush()
                 self.set_metadata(trans, dataset_assoc)
         else:
-            raise exceptions.InsufficientPermissionsException('Changing datatype "%s" is not allowed.' % (data.extension))
+            raise exceptions.InsufficientPermissionsException(f'Changing datatype "{data.extension}" is not allowed.')
 
     def set_metadata(self, trans, dataset_assoc, overwrite=False, validate=True):
         """Trigger a job that detects and sets metadata on a given dataset association (ldda or hda)"""
         data = trans.sa_session.query(self.model_class).get(dataset_assoc.id)
-        if not self.ok_to_edit_metadata(data.id):
+        if not data.ok_to_edit_metadata():
             raise exceptions.ItemAccessibilityException('This dataset is currently being used as input or output. You cannot edit metadata until the jobs have completed or you have canceled them.')
         else:
             if overwrite:
@@ -380,9 +381,10 @@ class DatasetAssociationManager(base.ModelManager,
                         if spec.get('default'):
                             setattr(data.metadata, name, spec.unwrap(spec.get('default')))
 
-            self.app.datatypes_registry.set_external_metadata_tool.tool_action.execute(
+            job, *_ = self.app.datatypes_registry.set_external_metadata_tool.tool_action.execute(
                 self.app.datatypes_registry.set_external_metadata_tool, trans, incoming={'input1': data, 'validate': validate},
                 overwrite=overwrite)
+            self.app.job_manager.enqueue(job, tool=self.app.datatypes_registry.set_external_metadata_tool)
 
     def update_permissions(self, trans, dataset_assoc, **kwd):
         action = kwd.get('action', 'set_permissions')
@@ -421,7 +423,7 @@ class DatasetAssociationManager(base.ModelManager,
                 return role_id
 
             def parameters_roles_or_none(role_type):
-                encoded_role_ids = kwd.get(role_type, kwd.get("%s_ids[]" % role_type, None))
+                encoded_role_ids = kwd.get(role_type, kwd.get(f"{role_type}_ids[]", None))
                 if encoded_role_ids is not None:
                     return list(map(to_role_id, encoded_role_ids))
                 else:
@@ -443,96 +445,100 @@ class DatasetAssociationManager(base.ModelManager,
         raise exceptions.NotImplemented()
 
 
-class _UnflattenedMetadataDatasetAssociationSerializer(base.ModelSerializer,
+class _UnflattenedMetadataDatasetAssociationSerializer(base.ModelSerializer[T],
                                                        deletable.PurgableSerializerMixin):
 
     def __init__(self, app):
-        self.dataset_serializer = DatasetSerializer(app)
+        self.dataset_serializer = app[DatasetSerializer]
         super().__init__(app)
 
     def add_serializers(self):
         super().add_serializers()
         deletable.PurgableSerializerMixin.add_serializers(self)
 
-        self.serializers.update({
-            'create_time'   : self.serialize_date,
-            'update_time'   : self.serialize_date,
+        serializers: Dict[str, base.Serializer] = {
+            'create_time': self.serialize_date,
+            'update_time': self.serialize_date,
 
             # underlying dataset
-            'dataset'       : lambda i, k, **c: self.dataset_serializer.serialize_to_view(i.dataset, view='summary', **c),
-            'dataset_id'    : self._proxy_to_dataset(key='id'),
+            'dataset': lambda item, key, **context: self.dataset_serializer.serialize_to_view(item.dataset, view='summary', **context),
+            'dataset_id': self._proxy_to_dataset(proxy_key='id'),
             # TODO: why is this named uuid!? The da doesn't have a uuid - it's the underlying dataset's uuid!
-            'uuid'          : self._proxy_to_dataset(key='uuid'),
-            # 'dataset_uuid'  : self._proxy_to_dataset( key='uuid' ),
-            'file_name'     : self._proxy_to_dataset(serializer=self.dataset_serializer.serialize_file_name),
-            'extra_files_path' : self._proxy_to_dataset(serializer=self.dataset_serializer.serialize_extra_files_path),
-            'permissions'   : self._proxy_to_dataset(serializer=self.dataset_serializer.serialize_permissions),
+            'uuid': self._proxy_to_dataset(proxy_key='uuid'),
+            # 'dataset_uuid': self._proxy_to_dataset( key='uuid' ),
+            'file_name': self._proxy_to_dataset(serializer=self.dataset_serializer.serialize_file_name),
+            'extra_files_path': self._proxy_to_dataset(serializer=self.dataset_serializer.serialize_extra_files_path),
+            'permissions': self._proxy_to_dataset(serializer=self.dataset_serializer.serialize_permissions),
             # TODO: do the sizes proxy accurately/in the same way?
-            'size'          : lambda i, k, **c: int(i.get_size()),
-            'file_size'     : lambda i, k, **c: self.serializers['size'](i, k, **c),
-            'nice_size'     : lambda i, k, **c: i.get_size(nice_size=True),
+            'size': lambda item, key, **context: int(item.get_size()),
+            'file_size': lambda item, key, **context: self.serializers['size'](item, key, **context),
+            'nice_size': lambda item, key, **context: item.get_size(nice_size=True),
 
             # common to lddas and hdas - from mapping.py
-            'copied_from_history_dataset_association_id'        : self.serialize_id,
+            'copied_from_history_dataset_association_id': self.serialize_id,
             'copied_from_library_dataset_dataset_association_id': self.serialize_id,
-            'info'          : lambda i, k, **c: i.info.strip() if isinstance(i.info, str) else i.info,
-            'blurb'         : lambda i, k, **c: i.blurb,
-            'peek'          : lambda i, k, **c: i.display_peek() if i.peek and i.peek != 'no peek' else None,
+            'info': lambda item, key, **context: item.info.strip() if isinstance(item.info, str) else item.info,
+            'blurb': lambda item, key, **context: item.blurb,
+            'peek': lambda item, key, **context: item.display_peek() if item.peek and item.peek != 'no peek' else None,
 
-            'meta_files'    : self.serialize_meta_files,
-            'metadata'      : self.serialize_metadata,
+            'meta_files': self.serialize_meta_files,
+            'metadata': self.serialize_metadata,
 
-            'creating_job'  : self.serialize_creating_job,
-            'rerunnable'    : self.serialize_rerunnable,
+            'creating_job': self.serialize_creating_job,
+            'rerunnable': self.serialize_rerunnable,
 
-            'parent_id'     : self.serialize_id,
-            'designation'   : lambda i, k, **c: i.designation,
+            'parent_id': self.serialize_id,
+            'designation': lambda item, key, **context: item.designation,
 
-            # 'extended_metadata'     : self.serialize_extended_metadata,
-            # 'extended_metadata_id'  : self.serialize_id,
+            # 'extended_metadata': self.serialize_extended_metadata,
+            # 'extended_metadata_id': self.serialize_id,
 
             # remapped
-            'genome_build'  : lambda i, k, **c: i.dbkey,
+            'genome_build': lambda item, key, **context: item.dbkey,
 
             # derived (not mapped) attributes
-            'data_type'     : lambda i, k, **c: i.datatype.__class__.__module__ + '.' + i.datatype.__class__.__name__,
+            'data_type': lambda item, key, **context: f"{item.datatype.__class__.__module__}.{item.datatype.__class__.__name__}",
 
-            'converted'     : self.serialize_converted_datasets,
+            'converted': self.serialize_converted_datasets,
             # TODO: metadata/extra files
-        })
+        }
+        self.serializers.update(serializers)
         # this an abstract superclass, so no views created
         # because of that: we need to add a few keys that will use the default serializer
         self.serializable_keyset.update(['name', 'state', 'tool_version', 'extension', 'visible', 'dbkey'])
 
-    def _proxy_to_dataset(self, serializer=None, key=None):
+    def _proxy_to_dataset(self, serializer: base.Serializer = None, proxy_key=None):
         # dataset associations are (rough) proxies to datasets - access their serializer using this remapping fn
         # remapping done by either kwarg key: IOW dataset attr key (e.g. uuid)
         # or by kwarg serializer: a function that's passed in (e.g. permissions)
-        if key:
-            serializer = self.dataset_serializer.serializers.get(key)
+        if proxy_key:
+            serializer = self.dataset_serializer.serializers.get(proxy_key)
         if serializer:
-            return lambda i, k, **c: serializer(i.dataset, key or k, **c)
+            return lambda item, key, **context: serializer(item.dataset, proxy_key or key, **context)
         raise TypeError('kwarg serializer or key needed')
 
-    def serialize_meta_files(self, dataset_assoc, key, **context):
+    def serialize_meta_files(self, item, key, **context):
         """
         Cycle through meta files and return them as a list of dictionaries.
         """
+        dataset_assoc = item
         meta_files = []
         for meta_type in dataset_assoc.metadata_file_types:
             if getattr(dataset_assoc.metadata, meta_type, None):
                 meta_files.append(
                     dict(file_type=meta_type,
-                         download_url=self.url_for('history_contents_metadata_file',
+                         download_url=self.url_for('get_metadata_file',
                                                    history_id=self.app.security.encode_id(dataset_assoc.history_id),
                                                    history_content_id=self.app.security.encode_id(dataset_assoc.id),
-                                                   metadata_file=meta_type)))
+                                                   query_params={'metadata_file': meta_type},
+                                                   context=context)))
         return meta_files
 
-    def serialize_metadata(self, dataset_assoc, key, excluded=None, **context):
+    def serialize_metadata(self, item, key, excluded=None, **context):
         """
         Cycle through metadata and return as dictionary.
         """
+        dataset_assoc = item
         # dbkey is a repeat actually (metadata_dbkey == genome_build)
         # excluded = [ 'dbkey' ] if excluded is None else excluded
         excluded = [] if excluded is None else excluded
@@ -558,34 +564,37 @@ class _UnflattenedMetadataDatasetAssociationSerializer(base.ModelSerializer,
 
         return metadata
 
-    def serialize_creating_job(self, dataset, key, **context):
+    def serialize_creating_job(self, item, key, **context):
         """
         Return the id of the Job that created this dataset (or its original)
         or None if no `creating_job` is found.
         """
+        dataset = item
         if dataset.creating_job:
             return self.serialize_id(dataset.creating_job, 'id')
         else:
             return None
 
-    def serialize_rerunnable(self, dataset, key, **context):
+    def serialize_rerunnable(self, item, key, **context):
         """
         Return False if this tool that created this dataset can't be re-run
         (e.g. upload).
         """
+        dataset = item
         if dataset.creating_job:
             tool = self.app.toolbox.get_tool(dataset.creating_job.tool_id, dataset.creating_job.tool_version)
             if tool and tool.is_workflow_compatible:
                 return True
         return False
 
-    def serialize_converted_datasets(self, dataset_assoc, key, **context):
+    def serialize_converted_datasets(self, item, key, **context):
         """
         Return a file extension -> converted dataset encoded id map with all
         the existing converted datasets associated with this instance.
 
         This filters out deleted associations.
         """
+        dataset_assoc = item
         id_map = {}
         for converted in dataset_assoc.implicitly_converted_datasets:
             if not converted.deleted and converted.dataset:
@@ -593,7 +602,7 @@ class _UnflattenedMetadataDatasetAssociationSerializer(base.ModelSerializer,
         return id_map
 
 
-class DatasetAssociationSerializer(_UnflattenedMetadataDatasetAssociationSerializer):
+class DatasetAssociationSerializer(_UnflattenedMetadataDatasetAssociationSerializer[T]):
     # TODO: remove this class - metadata should be a sub-object instead as in the superclass
 
     def add_serializers(self):
@@ -618,15 +627,15 @@ class DatasetAssociationSerializer(_UnflattenedMetadataDatasetAssociationSeriali
         return serialized
 
     # TODO: this is more util/gen. use
-    def _pluck_from_list(self, l, elems):
+    def _pluck_from_list(self, list_, elems):
         """
-        Removes found elems from list l and returns list of found elems if found.
+        Removes found elems from list list_ and returns list of found elems if found.
         """
         found = []
         for elem in elems:
             try:
-                index = l.index(elem)
-                found.append(l.pop(index))
+                index = list_.index(elem)
+                found.append(list_.pop(index))
             except ValueError:
                 pass
         return found
@@ -642,7 +651,7 @@ class DatasetAssociationSerializer(_UnflattenedMetadataDatasetAssociationSeriali
         # prefix each key within and return
         prefixed = {}
         for key, val in metadata.items():
-            prefixed_key = 'metadata_' + key
+            prefixed_key = f"metadata_{key}"
             prefixed[prefixed_key] = val
         return prefixed
 
@@ -654,9 +663,9 @@ class DatasetAssociationDeserializer(base.ModelDeserializer, deletable.PurgableD
         deletable.PurgableDeserializerMixin.add_deserializers(self)
 
         self.deserializers.update({
-            'name' : self.deserialize_basestring,
-            'info' : self.deserialize_basestring,
-            'datatype' : self.deserialize_datatype,
+            'name': self.deserialize_basestring,
+            'info': self.deserialize_basestring,
+            'datatype': self.deserialize_datatype,
         })
         self.deserializable_keyset.update(self.deserializers.keys())
 
@@ -664,7 +673,7 @@ class DatasetAssociationDeserializer(base.ModelDeserializer, deletable.PurgableD
     def deserialize_metadata(self, dataset_assoc, metadata_key, metadata_dict, **context):
         """
         """
-        self.validate.type(metadata_key, metadata_dict, dict)
+        self.validate.matches_type(metadata_key, metadata_dict, dict)
         returned = {}
         for key, val in metadata_dict.items():
             returned[key] = self.deserialize_metadatum(dataset_assoc, key, val, **context)
@@ -684,17 +693,22 @@ class DatasetAssociationDeserializer(base.ModelDeserializer, deletable.PurgableD
         return unwrapped_val
 
     def deserialize_datatype(self, item, key, val, **context):
-        if not item.datatype.allow_datatype_change:
+        if not item.datatype.is_datatype_change_allowed():
             raise exceptions.RequestParameterInvalidException("The current datatype does not allow datatype changes.")
-        if not self.app.datatypes_registry.get_datatype_by_extension(val):
+        target_datatype = self.app.datatypes_registry.get_datatype_by_extension(val)
+        if not target_datatype:
             raise exceptions.RequestParameterInvalidException("The target datatype does not exist.")
-        if not self.app.datatypes_registry.get_datatype_by_extension(val).allow_datatype_change:
+        if not target_datatype.is_datatype_change_allowed():
             raise exceptions.RequestParameterInvalidException("The target datatype does not allow datatype changes.")
-        if not DatasetAssociationManager(self.app).ok_to_edit_metadata(item.dataset_id):
+        if not item.ok_to_edit_metadata():
             raise exceptions.RequestParameterInvalidException("Dataset metadata could not be updated because it is used as input or output of a running job.")
         item.change_datatype(val)
-        context['trans'].sa_session.flush()
-        self.app.datatypes_registry.set_external_metadata_tool.tool_action.execute(self.app.datatypes_registry.set_external_metadata_tool, context['trans'], incoming={'input1': item}, overwrite=False)  # overwrite is False as per existing behavior
+        sa_session = self.app.model.context
+        sa_session.flush()
+        trans = context.get("trans")
+        assert trans, "Logic error in Galaxy, deserialize_datatype not send a transation object"  # TODO: restructure this for stronger typing
+        job, *_ = self.app.datatypes_registry.set_external_metadata_tool.tool_action.execute(self.app.datatypes_registry.set_external_metadata_tool, trans, incoming={'input1': item}, overwrite=False)  # overwrite is False as per existing behavior
+        trans.app.job_manager.enqueue(job, tool=trans.app.datatypes_registry.set_external_metadata_tool)
         return item.datatype
 
 
@@ -705,16 +719,16 @@ class DatasetAssociationFilterParser(base.ModelFilterParser, deletable.PurgableF
         deletable.PurgableFiltersMixin._add_parsers(self)
 
         self.orm_filter_parsers.update({
-            'name'      : {'op': ('eq', 'contains', 'like')},
-            'state'     : {'column' : '_state', 'op': ('eq', 'in')},
-            'visible'   : {'op': ('eq'), 'val': self.parse_bool},
+            'name': {'op': ('eq', 'contains', 'like')},
+            'state': {'column': '_state', 'op': ('eq', 'in')},
+            'visible': {'op': ('eq'), 'val': base.parse_bool},
         })
         self.fn_filter_parsers.update({
-            'genome_build' : self.string_standard_ops('dbkey'),
-            'data_type' : {
+            'genome_build': self.string_standard_ops('dbkey'),
+            'data_type': {
                 'op': {
-                    'eq' : self.eq_datatype,
-                    'isinstance' : self.isinstance_datatype
+                    'eq': self.eq_datatype,
+                    'isinstance': self.isinstance_datatype
                 }
             }
         })
@@ -724,8 +738,7 @@ class DatasetAssociationFilterParser(base.ModelFilterParser, deletable.PurgableF
         Is the `dataset_assoc` datatype equal to the registered datatype `class_str`?
         """
         comparison_class = self.app.datatypes_registry.get_datatype_class_by_name(class_str)
-        return (comparison_class and
-            dataset_assoc.datatype.__class__ == comparison_class)
+        return comparison_class and dataset_assoc.datatype.__class__ == comparison_class
 
     def isinstance_datatype(self, dataset_assoc, class_strs):
         """
@@ -733,10 +746,9 @@ class DatasetAssociationFilterParser(base.ModelFilterParser, deletable.PurgableF
         datatypes in the comma separated string `class_strs`?
         """
         parse_datatype_fn = self.app.datatypes_registry.get_datatype_class_by_name
-        comparison_classes = []
+        comparison_classes: List[Type] = []
         for class_str in class_strs.split(','):
             datatype_class = parse_datatype_fn(class_str)
             if datatype_class:
                 comparison_classes.append(datatype_class)
-        return (comparison_classes and
-            isinstance(dataset_assoc.datatype, comparison_classes))
+        return comparison_classes and isinstance(dataset_assoc.datatype, tuple(comparison_classes))
