@@ -8,6 +8,15 @@ models have some backing/supporting resources that can be removed as well
 the supporting resources as well. These models also have the boolean
 attribute 'purged'.
 """
+from typing import Any, Dict, Set
+
+from galaxy.model import _HasTable
+from .base import (
+    Deserializer,
+    ModelValidator,
+    OrmFilterParsersType,
+    parse_bool,
+)
 
 
 class DeletableManagerMixin:
@@ -19,6 +28,9 @@ class DeletableManagerMixin:
     removed by an admin/script.
     """
 
+    def _session_setattr(self, item: _HasTable, attr: str, val: Any, flush: bool = True):
+        ...
+
     def delete(self, item, flush=True, **kwargs):
         """
         Mark as deleted and return.
@@ -33,6 +45,7 @@ class DeletableManagerMixin:
 
 
 class DeletableSerializerMixin:
+    serializable_keyset: Set[str]
 
     def add_serializers(self):
         self.serializable_keyset.add('deleted')
@@ -40,6 +53,7 @@ class DeletableSerializerMixin:
 
 # TODO: these are of questionable value if we don't want to enable users to delete/purge via update
 class DeletableDeserializerMixin:
+    deserializers: Dict[str, Deserializer]
 
     def add_deserializers(self):
         self.deserializers['deleted'] = self.deserialize_deleted
@@ -48,7 +62,7 @@ class DeletableDeserializerMixin:
         """
         Delete or undelete `item` based on `val` then return `item.deleted`.
         """
-        new_deleted = self.validate.bool(key, val)
+        new_deleted = ModelValidator.bool(key, val)
         if new_deleted == item.deleted:
             return item.deleted
         # TODO:?? flush=False?
@@ -60,10 +74,11 @@ class DeletableDeserializerMixin:
 
 
 class DeletableFiltersMixin:
+    orm_filter_parsers: OrmFilterParsersType
 
     def _add_parsers(self):
         self.orm_filter_parsers.update({
-            'deleted': {'op': ('eq'), 'val': self.parse_bool}
+            'deleted': {'op': ('eq'), 'val': parse_bool}
         })
 
 
@@ -74,6 +89,9 @@ class PurgableManagerMixin(DeletableManagerMixin):
     file).
     """
 
+    def _session_setattr(self, item: _HasTable, attr: str, val: Any, flush: bool = True):
+        ...
+
     def purge(self, item, flush=True, **kwargs):
         """
         Mark as purged and return.
@@ -85,6 +103,7 @@ class PurgableManagerMixin(DeletableManagerMixin):
 
 
 class PurgableSerializerMixin(DeletableSerializerMixin):
+    serializable_keyset: Set[str]
 
     def add_serializers(self):
         DeletableSerializerMixin.add_serializers(self)
@@ -92,6 +111,7 @@ class PurgableSerializerMixin(DeletableSerializerMixin):
 
 
 class PurgableDeserializerMixin(DeletableDeserializerMixin):
+    deserializers: Dict[str, Deserializer] = {}
 
     def add_deserializers(self):
         DeletableDeserializerMixin.add_deserializers(self)
@@ -101,7 +121,7 @@ class PurgableDeserializerMixin(DeletableDeserializerMixin):
         """
         If `val` is True, purge `item` and return `item.purged`.
         """
-        new_purged = self.validate.bool(key, val)
+        new_purged = ModelValidator.bool(key, val)
         if new_purged == item.purged:
             return item.purged
         # do we want to error if something attempts to 'unpurge'?
@@ -115,5 +135,5 @@ class PurgableFiltersMixin(DeletableFiltersMixin):
     def _add_parsers(self):
         DeletableFiltersMixin._add_parsers(self)
         self.orm_filter_parsers.update({
-            'purged': {'op': ('eq'), 'val': self.parse_bool}
+            'purged': {'op': ('eq'), 'val': parse_bool}
         })
