@@ -5,10 +5,18 @@ Histories are containers for datasets or dataset collections
 created (or copied) by users over the course of an analysis.
 """
 import logging
+from typing import (
+    cast,
+    Dict,
+    List,
+    Optional,
+    Set,
+)
 
 from sqlalchemy import (
+    and_,
     asc,
-    desc
+    desc,
 )
 
 from galaxy import (
@@ -21,11 +29,20 @@ from galaxy.managers import (
     history_contents,
     sharable
 )
+from galaxy.managers.base import (
+    Serializer,
+    SortableManager,
+)
+from galaxy.schema.schema import (
+    HDABasicInfo,
+    ShareHistoryExtra,
+)
+from galaxy.structured_app import MinimalManagerApp
 
 log = logging.getLogger(__name__)
 
 
-class HistoryManager(sharable.SharableModelManager, deletable.PurgableManagerMixin):
+class HistoryManager(sharable.SharableModelManager, deletable.PurgableManagerMixin, SortableManager):
 
     model_class = model.History
     foreign_key_name = 'history'
@@ -37,11 +54,11 @@ class HistoryManager(sharable.SharableModelManager, deletable.PurgableManagerMix
 
     # TODO: incorporate imp/exp (or alias to)
 
-    def __init__(self, app, *args, **kwargs):
-        super().__init__(app, *args, **kwargs)
-        self.hda_manager = hdas.HDAManager(app)
-        self.contents_manager = history_contents.HistoryContentsManager(app)
-        self.contents_filters = history_contents.HistoryContentsFilters(app)
+    def __init__(self, app: MinimalManagerApp, hda_manager: hdas.HDAManager, contents_manager: history_contents.HistoryContentsManager, contents_filters: history_contents.HistoryContentsFilters):
+        super().__init__(app)
+        self.hda_manager = hda_manager
+        self.contents_manager = contents_manager
+        self.contents_filters = contents_filters
 
     def copy(self, history, user, **kwargs):
         """
@@ -82,7 +99,7 @@ class HistoryManager(sharable.SharableModelManager, deletable.PurgableManagerMix
         """
         if self.user_manager.is_anonymous(user):
             return None if (not current_history or current_history.deleted) else current_history
-        desc_update_time = desc(self.model_class.table.c.update_time)
+        desc_update_time = desc(self.model_class.update_time)
         filters = self._munge_filters(filters, self.model_class.user_id == user.id)
         # TODO: normalize this return value
         return self.query(filters=filters, order_by=desc_update_time, limit=1, **kwargs).first()
@@ -168,6 +185,185 @@ class HistoryManager(sharable.SharableModelManager, deletable.PurgableManagerMix
             .filter(model.Job.state.in_(model.Job.non_ready_states)))
         return jobs
 
+    def queue_history_import(self, trans, archive_type, archive_source):
+        # Run job to do import.
+        history_imp_tool = trans.app.toolbox.get_tool('__IMPORT_HISTORY__')
+        incoming = {'__ARCHIVE_SOURCE__': archive_source, '__ARCHIVE_TYPE__': archive_type}
+        job, *_ = history_imp_tool.execute(trans, incoming=incoming)
+        trans.app.job_manager.enqueue(job, tool=history_imp_tool)
+        return job
+
+    # TODO: remove this function when the legacy endpoint using it is removed
+    def legacy_serve_ready_history_export(self, trans, jeha):
+        assert jeha.ready
+        if jeha.compressed:
+            trans.response.set_content_type('application/x-gzip')
+        else:
+            trans.response.set_content_type('application/x-tar')
+        disposition = f'attachment; filename="{jeha.export_name}"'
+        trans.response.headers["Content-Disposition"] = disposition
+        archive = trans.app.object_store.get_filename(jeha.dataset)
+        return open(archive, mode='rb')
+
+    def get_ready_history_export_file_path(self, trans, jeha) -> str:
+        """
+        Serves the history export archive for use as a streaming response so the file
+        doesn't need to be loaded into memory.
+        """
+        assert jeha.ready
+        return trans.app.object_store.get_filename(jeha.dataset)
+
+    def queue_history_export(self, trans, history, gzip=True, include_hidden=False, include_deleted=False, directory_uri=None, file_name=None):
+        # Convert options to booleans.
+        if isinstance(gzip, str):
+            gzip = (gzip in ['True', 'true', 'T', 't'])
+        if isinstance(include_hidden, str):
+            include_hidden = (include_hidden in ['True', 'true', 'T', 't'])
+        if isinstance(include_deleted, str):
+            include_deleted = (include_deleted in ['True', 'true', 'T', 't'])
+
+        params = {
+            'history_to_export': history,
+            'compress': gzip,
+            'include_hidden': include_hidden,
+            'include_deleted': include_deleted
+        }
+
+        if directory_uri is None:
+            export_tool_id = '__EXPORT_HISTORY__'
+        else:
+            params['directory_uri'] = directory_uri
+            params['file_name'] = file_name or None
+            export_tool_id = '__EXPORT_HISTORY_TO_URI__'
+
+        # Run job to do export.
+        history_exp_tool = trans.app.toolbox.get_tool(export_tool_id)
+        job, *_ = history_exp_tool.execute(trans, incoming=params, history=history, set_output_hid=True)
+        trans.app.job_manager.enqueue(job, tool=history_exp_tool)
+        return job
+
+    def get_sharing_extra_information(
+        self, trans, item, users: Set[model.User], errors: Set[str], option: Optional[sharable.SharingOptions] = None
+    ) -> Optional[sharable.ShareWithExtra]:
+        """Returns optional extra information about the datasets of the history that can be accessed by the users."""
+        extra = ShareHistoryExtra()
+        history = cast(model.History, item)
+        if history.empty:
+            errors.add("You cannot share an empty history.")
+            return extra
+
+        owner = trans.user
+        owner_roles = owner.all_roles()
+        can_change_dict = {}
+        cannot_change_dict = {}
+        share_anyway = option is not None and option == sharable.SharingOptions.no_changes
+        datasets = history.activatable_datasets
+        total_dataset_count = len(datasets)
+        for user in users:
+            if self.is_history_shared_with(history, user):
+                continue
+
+            user_roles = user.all_roles()
+            # TODO: Handle this is a more performant way
+            # Only deal with datasets that have not been purged
+            for hda in datasets:
+                if trans.app.security_agent.can_access_dataset(user_roles, hda.dataset):
+                    continue
+                # The user with which we are sharing the history does not have access permission on the current dataset
+                owner_can_manage_dataset = (
+                    trans.app.security_agent.can_manage_dataset(owner_roles, hda.dataset)
+                    and not hda.dataset.library_associations
+                )
+                if option and owner_can_manage_dataset:
+                    if option == sharable.SharingOptions.make_accessible_to_shared:
+                        trans.app.security_agent.privately_share_dataset(hda.dataset, users=[owner, user])
+                    elif option == sharable.SharingOptions.make_public:
+                        trans.app.security_agent.make_dataset_public(hda.dataset)
+                else:
+                    hda_id = trans.security.encode_id(hda.id)
+                    hda_info = HDABasicInfo(id=hda_id, name=hda.name)
+                    if owner_can_manage_dataset:
+                        can_change_dict[hda_id] = hda_info
+                    else:
+                        cannot_change_dict[hda_id] = hda_info
+
+        extra.can_change = list(can_change_dict.values())
+        extra.cannot_change = list(cannot_change_dict.values())
+        extra.accessible_count = total_dataset_count - len(extra.can_change) - len(extra.cannot_change)
+        if not extra.accessible_count and not extra.can_change and not share_anyway:
+            errors.add("The history you are sharing do not contain any datasets that can be accessed by the users with which you are sharing.")
+
+        extra.can_share = not errors and (extra.accessible_count == total_dataset_count or option is not None)
+        return extra
+
+    def is_history_shared_with(self, history, user) -> bool:
+        return bool(self.session().query(self.user_share_model).filter(
+            and_(
+                self.user_share_model.table.c.user_id == user.id,
+                self.user_share_model.table.c.history_id == history.id,
+            )
+        ).first())
+
+    def make_members_public(self, trans, item):
+        """ Make the non-purged datasets in history public.
+        Performs permissions check.
+        """
+        for hda in item.activatable_datasets:
+            dataset = hda.dataset
+            if not trans.app.security_agent.dataset_is_public(dataset):
+                if trans.app.security_agent.can_manage_dataset(trans.user.all_roles(), dataset):
+                    try:
+                        trans.app.security_agent.make_dataset_public(hda.dataset)
+                    except Exception:
+                        log.warning(f"Unable to make dataset with id: {dataset.id} public")
+                else:
+                    log.warning(f"User without permissions tried to make dataset with id: {dataset.id} public")
+
+
+class HistoryExportView:
+
+    def __init__(self, app: MinimalManagerApp):
+        self.app = app
+
+    def get_exports(self, trans, history_id):
+        history = self._history(trans, history_id)
+        matching_exports = history.exports
+        return [self.serialize(trans, history_id, e) for e in matching_exports]
+
+    def serialize(self, trans, history_id, jeha):
+        rval = jeha.to_dict()
+        encoded_jeha_id = trans.security.encode_id(jeha.id)
+        api_url = trans.url_builder("history_archive_download", id=history_id, jeha_id=encoded_jeha_id)
+        external_url = trans.url_builder("history_archive_download", id=history_id, jeha_id="latest", qualified=True)
+        external_permanent_url = trans.url_builder("history_archive_download", id=history_id, jeha_id=encoded_jeha_id, qualified=True)
+        rval["download_url"] = api_url
+        rval["external_download_latest_url"] = external_url
+        rval["external_download_permanent_url"] = external_permanent_url
+        rval = trans.security.encode_all_ids(rval)
+        return rval
+
+    def get_ready_jeha(self, trans, history_id, jeha_id="latest"):
+        history = self._history(trans, history_id)
+        matching_exports = history.exports
+        if jeha_id != "latest":
+            decoded_jeha_id = trans.security.decode_id(jeha_id)
+            matching_exports = [e for e in matching_exports if e.id == decoded_jeha_id]
+        if len(matching_exports) == 0:
+            raise glx_exceptions.ObjectNotFound("Failed to find target history export")
+
+        jeha = matching_exports[0]
+        if not jeha.ready:
+            raise glx_exceptions.MessageException("Export not available or not yet ready.")
+
+        return jeha
+
+    def _history(self, trans, history_id):
+        if history_id is not None:
+            history = self.app.history_manager.get_accessible(trans.security.decode_id(history_id), trans.user, current_history=trans.history)
+        else:
+            history = trans.history
+        return history
+
 
 class HistorySerializer(sharable.SharableModelSerializer, deletable.PurgableSerializerMixin):
     """
@@ -176,13 +372,13 @@ class HistorySerializer(sharable.SharableModelSerializer, deletable.PurgableSeri
     model_manager_class = HistoryManager
     SINGLE_CHAR_ABBR = 'h'
 
-    def __init__(self, app, **kwargs):
-        super().__init__(app, **kwargs)
+    def __init__(self, app: MinimalManagerApp, hda_manager: hdas.HDAManager, hda_serializer: hdas.HDASerializer, history_contents_serializer: history_contents.HistoryContentsSerializer):
+        super().__init__(app)
 
         self.history_manager = self.manager
-        self.hda_manager = hdas.HDAManager(app)
-        self.hda_serializer = hdas.HDASerializer(app)
-        self.history_contents_serializer = history_contents.HistoryContentsSerializer(app)
+        self.hda_manager = hda_manager
+        self.hda_serializer = hda_serializer
+        self.history_contents_serializer = history_contents_serializer
 
         self.default_view = 'summary'
         self.add_view('summary', [
@@ -197,6 +393,7 @@ class HistorySerializer(sharable.SharableModelSerializer, deletable.PurgableSeri
             'published',
             'annotation',
             'tags',
+            'update_time',
         ])
         self.add_view('detailed', [
             'contents_url',
@@ -234,44 +431,73 @@ class HistorySerializer(sharable.SharableModelSerializer, deletable.PurgableSeri
             'hid_counter',
         ], include_keys_from='summary')
 
+        # beta web client fields, no summary/detailed/dev-detailed blah
+        self.add_view('betawebclient', [
+            'annotation',
+            'contents_active',
+            'contents_url',
+            'create_time',
+            'deleted',
+            'empty',
+            'genome_build',
+            'hid_counter',
+            'id',
+            'importable',
+            'name',
+            'nice_size',
+            'published',
+            'purged',
+            # 'shared',
+            'size',
+            'slug',
+            'state',
+            'tags',
+            'update_time',
+            'url',
+            'username_and_slug',
+            'user_id',
+        ])
+
     # assumes: outgoing to json.dumps and sanitized
     def add_serializers(self):
         super().add_serializers()
         deletable.PurgableSerializerMixin.add_serializers(self)
 
-        self.serializers.update({
-            'model_class'   : lambda *a, **c: 'History',
-            'size'          : lambda i, k, **c: int(i.disk_size),
-            'nice_size'     : lambda i, k, **c: i.disk_nice_size,
-            'state'         : self.serialize_history_state,
+        serializers: Dict[str, Serializer] = {
+            'model_class': lambda item, key, **context: 'History',
+            'size': lambda item, key, **context: int(item.disk_size),
+            'nice_size': lambda item, key, **context: item.disk_nice_size,
+            'state': self.serialize_history_state,
 
-            'url'           : lambda i, k, **c: self.url_for('history', id=self.app.security.encode_id(i.id)),
-            'contents_url'  : lambda i, k, **c: self.url_for('history_contents',
-                                                             history_id=self.app.security.encode_id(i.id)),
+            'url': lambda item, key, **context: self.url_for('history', id=self.app.security.encode_id(item.id), context=context),
+            'contents_url': lambda item, key, **context: self.url_for('history_contents',
+                                                           history_id=self.app.security.encode_id(item.id), context=context),
 
-            'empty'         : lambda i, k, **c: (len(i.datasets) + len(i.dataset_collections)) <= 0,
-            'count'         : lambda i, k, **c: len(i.datasets),
-            'hdas'          : lambda i, k, **c: [self.app.security.encode_id(hda.id) for hda in i.datasets],
-            'state_details' : self.serialize_state_counts,
-            'state_ids'     : self.serialize_state_ids,
-            'contents'      : self.serialize_contents,
-            'non_ready_jobs': lambda i, k, **c: [self.app.security.encode_id(job.id) for job
-                                                 in self.manager.non_ready_jobs(i)],
+            'empty': lambda item, key, **context: (len(item.datasets) + len(item.dataset_collections)) <= 0,
+            'count': lambda item, key, **context: len(item.datasets),
+            'hdas': lambda item, key, **context: [self.app.security.encode_id(hda.id) for hda in item.datasets],
+            'state_details': self.serialize_state_counts,
+            'state_ids': self.serialize_state_ids,
+            'contents': self.serialize_contents,
+            'non_ready_jobs': lambda item, key, **context: [self.app.security.encode_id(job.id) for job
+                                                 in self.manager.non_ready_jobs(item)],
 
             'contents_states': self.serialize_contents_states,
             'contents_active': self.serialize_contents_active,
             #  TODO: Use base manager's serialize_id for user_id (and others)
             #  after refactoring hierarchy here?
-            'user_id'       : lambda i, k, **c: self.app.security.encode_id(i.user_id) if i.user_id is not None else None
-        })
+            'user_id': lambda item, key, **context: self.app.security.encode_id(item.user_id) if item.user_id is not None else None
+        }
+        self.serializers.update(serializers)
 
     # remove this
-    def serialize_state_ids(self, history, key, **context):
+    def serialize_state_ids(self, item, key, **context):
         """
         Return a dictionary keyed to possible dataset states and valued with lists
         containing the ids of each HDA in that state.
         """
-        state_ids = {}
+        history = item
+        state_ids: Dict[str, List[str]] = {}
         for state in model.Dataset.states.values():
             state_ids[state] = []
 
@@ -283,11 +509,12 @@ class HistorySerializer(sharable.SharableModelSerializer, deletable.PurgableSeri
         return state_ids
 
     # remove this
-    def serialize_state_counts(self, history, key, exclude_deleted=True, exclude_hidden=False, **context):
+    def serialize_state_counts(self, item, key, exclude_deleted=True, exclude_hidden=False, **context):
         """
         Return a dictionary keyed to possible dataset states and valued with the number
         of datasets in this history that have those states.
         """
+        history = item
         # TODO: the default flags above may not make a lot of sense (T,T?)
         state_counts = {}
         for state in model.Dataset.states.values():
@@ -303,38 +530,39 @@ class HistorySerializer(sharable.SharableModelSerializer, deletable.PurgableSeri
         return state_counts
 
     # TODO: remove this (is state used/useful?)
-    def serialize_history_state(self, history, key, **context):
+    def serialize_history_state(self, item, key, **context):
         """
         Returns the history state based on the states of the HDAs it contains.
         """
+        history = item
         states = model.Dataset.states
         # (default to ERROR)
         state = states.ERROR
         # TODO: history_state and state_counts are classically calc'd at the same time
         #   so this is rel. ineff. - if we keep this...
         hda_state_counts = self.serialize_state_counts(history, 'counts', exclude_deleted=True, **context)
-        num_hdas = sum(hda_state_counts.values())
-        if num_hdas == 0:
+        if history.empty:
             state = states.NEW
-
         else:
-            if (hda_state_counts[states.RUNNING] > 0 or
-                    hda_state_counts[states.SETTING_METADATA] > 0 or
-                    hda_state_counts[states.UPLOAD] > 0):
+            num_hdas = sum(hda_state_counts.values())
+            if (hda_state_counts[states.RUNNING] > 0
+                    or hda_state_counts[states.SETTING_METADATA] > 0
+                    or hda_state_counts[states.UPLOAD] > 0):
                 state = states.RUNNING
             # TODO: this method may be more useful if we *also* polled the histories jobs here too
-            elif (hda_state_counts[states.QUEUED] > 0 or
-                    hda_state_counts[states.NEW] > 0):
+            elif (hda_state_counts[states.QUEUED] > 0
+                    or hda_state_counts[states.NEW] > 0):
                 state = states.QUEUED
-            elif (hda_state_counts[states.ERROR] > 0 or
-                    hda_state_counts[states.FAILED_METADATA] > 0):
+            elif (hda_state_counts[states.ERROR] > 0
+                    or hda_state_counts[states.FAILED_METADATA] > 0):
                 state = states.ERROR
             elif hda_state_counts[states.OK] == num_hdas:
                 state = states.OK
 
         return state
 
-    def serialize_contents(self, history, key, trans=None, user=None, **context):
+    def serialize_contents(self, item, key, trans=None, user=None, **context):
+        history = item
         returned = []
         for content in self.manager.contents_manager._union_of_contents_query(history).all():
             serialized = self.history_contents_serializer.serialize_to_view(content,
@@ -342,16 +570,17 @@ class HistorySerializer(sharable.SharableModelSerializer, deletable.PurgableSeri
             returned.append(serialized)
         return returned
 
-    def serialize_contents_states(self, history, key, trans=None, **context):
+    def serialize_contents_states(self, item, key, trans=None, **context):
         """
         Return a dictionary containing the counts of all contents in each state
         keyed by the distinct states.
 
         Note: does not include deleted/hidden contents.
         """
+        history = item
         return self.manager.contents_manager.state_counts(history)
 
-    def serialize_contents_active(self, history, key, **context):
+    def serialize_contents_active(self, item, key, **context):
         """
         Return a dictionary keyed with 'deleted', 'hidden', and 'active' with values
         for each representing the count of contents in each state.
@@ -359,6 +588,7 @@ class HistorySerializer(sharable.SharableModelSerializer, deletable.PurgableSeri
         Note: counts for deleted and hidden overlap; In other words, a dataset that's
         both deleted and hidden will be added to both totals.
         """
+        history = item
         return self.manager.contents_manager.active_counts(history)
 
 
@@ -368,7 +598,7 @@ class HistoryDeserializer(sharable.SharableModelDeserializer, deletable.Purgable
     """
     model_manager_class = HistoryManager
 
-    def __init__(self, app):
+    def __init__(self, app: MinimalManagerApp):
         super().__init__(app)
         self.history_manager = self.manager
 
@@ -377,8 +607,8 @@ class HistoryDeserializer(sharable.SharableModelDeserializer, deletable.Purgable
         deletable.PurgableDeserializerMixin.add_deserializers(self)
 
         self.deserializers.update({
-            'name'          : self.deserialize_basestring,
-            'genome_build'  : self.deserialize_genome_build,
+            'name': self.deserialize_basestring,
+            'genome_build': self.deserialize_genome_build,
         })
 
 
@@ -391,8 +621,8 @@ class HistoryFilters(sharable.SharableModelFilters, deletable.PurgableFiltersMix
         deletable.PurgableFiltersMixin._add_parsers(self)
         self.orm_filter_parsers.update({
             # history specific
-            'name'          : {'op': ('eq', 'contains', 'like')},
-            'genome_build'  : {'op': ('eq', 'contains', 'like')},
-            'create_time'   : {'op': ('le', 'ge', 'gt', 'lt'), 'val': self.parse_date},
-            'update_time'   : {'op': ('le', 'ge', 'gt', 'lt'), 'val': self.parse_date},
+            'name': {'op': ('eq', 'contains', 'like')},
+            'genome_build': {'op': ('eq', 'contains', 'like')},
+            'create_time': {'op': ('le', 'ge', 'gt', 'lt'), 'val': self.parse_date},
+            'update_time': {'op': ('le', 'ge', 'gt', 'lt'), 'val': self.parse_date},
         })
