@@ -1,7 +1,6 @@
 import logging
 import sqlite3
 
-
 from sqlalchemy import or_
 
 
@@ -28,9 +27,9 @@ class InteractiveToolSqlite:
             conn = sqlite3.connect(self.sqlite_filename)
             try:
                 c = conn.cursor()
-                select = '''SELECT token, host, port, info
-                            FROM %s
-                            WHERE key=? and key_type=?''' % (DATABASE_TABLE_NAME)
+                select = f'''SELECT token, host, port, info
+                            FROM {DATABASE_TABLE_NAME}
+                            WHERE key=? and key_type=?'''
                 c.execute(select, (key, key_type,))
                 try:
                     token, host, port, info = c.fetchone()
@@ -72,7 +71,7 @@ class InteractiveToolSqlite:
                                   )''' % (DATABASE_TABLE_NAME))
                 except Exception:
                     pass
-                delete = '''DELETE FROM %s WHERE key=? and key_type=?''' % (DATABASE_TABLE_NAME)
+                delete = f'''DELETE FROM {DATABASE_TABLE_NAME} WHERE key=? and key_type=?'''
                 c.execute(delete, (key, key_type,))
                 insert = '''INSERT INTO %s
                             (key, key_type, token, host, port, info)
@@ -95,12 +94,12 @@ class InteractiveToolSqlite:
         with external resources. Remove entries that match all provided key=values
         """
         assert kwd, ValueError("You must provide some values to key upon")
-        delete = 'DELETE FROM %s WHERE' % (DATABASE_TABLE_NAME)
+        delete = f'DELETE FROM {DATABASE_TABLE_NAME} WHERE'
         value_list = []
         for i, (key, value) in enumerate(kwd.items()):
             if i != 0:
                 delete += ' and'
-            delete += ' %s=?' % (key)
+            delete += f' {key}=?'
             value_list.append(value)
         with FileLock(self.sqlite_filename):
             conn = sqlite3.connect(self.sqlite_filename)
@@ -143,7 +142,7 @@ class InteractiveToolManager:
     def create_entry_points(self, job, tool, entry_points=None, flush=True):
         entry_points = entry_points or tool.ports
         for entry in entry_points:
-            ep = self.model.InteractiveToolEntryPoint(job=job, tool_port=entry['port'], entry_url=entry['url'], name=entry['name'])
+            ep = self.model.InteractiveToolEntryPoint(job=job, tool_port=entry['port'], entry_url=entry['url'], name=entry['name'], requires_domain=entry['requires_domain'])
             self.sa_session.add(ep)
         if flush:
             self.sa_session.flush()
@@ -228,20 +227,15 @@ class InteractiveToolManager:
         return True
 
     def stop(self, trans, entry_point):
-        try:
-            self.remove_entry_point(entry_point)
-            job = entry_point.job
-            if not job.finished:
-                log.debug('Stopping Job: %s for InteractiveToolEntryPoint: %s', job, entry_point)
-                job.mark_deleted(trans.app.config.track_jobs_in_database)
-                # This self.job_manager.stop(job) does nothing without changing job.state, manually or e.g. with .mark_deleted()
-                self.job_manager.stop(job)
-                trans.sa_session.add(job)
-                trans.sa_session.flush()
-        except Exception as e:
-            log.debug('Unable to stop job for InteractiveToolEntryPoint (%s): %s', entry_point, e)
-            return False
-        return True
+        self.remove_entry_point(entry_point)
+        job = entry_point.job
+        if not job.finished:
+            log.debug('Stopping Job: %s for InteractiveToolEntryPoint: %s', job, entry_point)
+            job.mark_stopped(trans.app.config.track_jobs_in_database)
+            # This self.job_manager.stop(job) does nothing without changing job.state, manually or e.g. with .mark_deleted()
+            self.job_manager.stop(job)
+            trans.sa_session.add(job)
+            trans.sa_session.flush()
 
     def remove_entry_points(self, entry_points):
         if entry_points:
@@ -259,17 +253,44 @@ class InteractiveToolManager:
     def target_if_active(self, trans, entry_point):
         if entry_point.active and not entry_point.deleted:
             request_host = trans.request.host
+            if not self.app.config.interactivetools_upstream_proxy and self.app.config.interactivetools_proxy_host:
+                request_host = self.app.config.interactivetools_proxy_host
             protocol = trans.request.host_url.split('//', 1)[0]
-            entry_point_encoded_id = trans.security.encode_id(entry_point.id)
-            entry_point_class = entry_point.__class__.__name__.lower()
-            entry_point_prefix = self.app.config.interactivetools_prefix
-            interactivetools_proxy_host = self.app.config.interactivetools_proxy_host or request_host
-            rval = '{}//{}-{}.{}.{}.{}/'.format(protocol, entry_point_encoded_id,
-                    entry_point.token, entry_point_class, entry_point_prefix, interactivetools_proxy_host)
-            if entry_point.entry_url:
-                rval = '{}/{}'.format(rval.rstrip('/'), entry_point.entry_url.lstrip('/'))
+            if entry_point.requires_domain:
+                rval = f'{protocol}//{self.get_entry_point_subdomain(trans, entry_point)}.{request_host}/'
+                if entry_point.entry_url:
+                    rval = '{}/{}'.format(rval.rstrip('/'), entry_point.entry_url.lstrip('/'))
+            else:
+                rval = self.get_entry_point_path(trans, entry_point)
+
             return rval
 
+    def get_entry_point_subdomain(self, trans, entry_point):
+        entry_point_encoded_id = trans.security.encode_id(entry_point.id)
+        entry_point_class = entry_point.__class__.__name__.lower()
+        entry_point_prefix = self.app.config.interactivetools_prefix
+        entry_point_token = entry_point.token
+        if self.app.config.interactivetools_shorten_url:
+            return f'{entry_point_encoded_id}-{entry_point_token[:10]}.{entry_point_prefix}'
+        return f'{entry_point_encoded_id}-{entry_point_token}.{entry_point_class}.{entry_point_prefix}'
+
+    def get_entry_point_path(self, trans, entry_point):
+        entry_point_encoded_id = trans.security.encode_id(entry_point.id)
+        entry_point_class = entry_point.__class__.__name__.lower()
+        entry_point_prefix = self.app.config.interactivetools_prefix
+        rval = "/"
+        if not entry_point.requires_domain:
+            rval = str(self.app.config.interactivetools_base_path).rstrip("/").lstrip("/")
+            if self.app.config.interactivetools_shorten_url:
+                rval = f'/{rval}/{entry_point_prefix}/{entry_point_encoded_id}/{entry_point.token[:10]}/'
+            else:
+                rval = f'/{rval}/{entry_point_prefix}/access/{entry_point_class}/{entry_point_encoded_id}/{entry_point.token}/'
+        if entry_point.entry_url:
+            rval = f"{rval.rstrip('/')}/{entry_point.entry_url.lstrip('/')}"
+        if rval[0] != "/":
+            rval = f'/{rval}'
+        return rval
+
     def access_entry_point_target(self, trans, entry_point_id):
         entry_point = trans.sa_session.query(model.InteractiveToolEntryPoint).get(entry_point_id)
         if self.app.interactivetool_manager.can_access_entry_point(trans, entry_point):
