@@ -6,6 +6,8 @@ Galaxy Markdown. Keeping things isolated to allow re-use of these utilities in o
 projects (e.g. gxformat2).
 """
 import re
+from typing import cast, Dict, List, Union
+
 
 BLOCK_FENCE_START = re.compile(r'```.*')
 BLOCK_FENCE_END = re.compile(r'```[\s]*')
@@ -13,7 +15,15 @@ GALAXY_FLAVORED_MARKDOWN_CONTAINER_LINE_PATTERN = re.compile(
     r"```\s*galaxy\s*"
 )
 VALID_CONTAINER_END_PATTERN = re.compile(r"^```\s*$")
-VALID_ARGUMENTS = {
+
+
+class DynamicArguments:
+    pass
+
+
+DYNAMIC_ARGUMENTS = DynamicArguments()
+VALID_ARGUMENTS: Dict[str, Union[List[str], DynamicArguments]] = {
+    "history_link": ["history_id"],
     "history_dataset_display": ["input", "output", "history_dataset_id"],
     "history_dataset_embedded": ["input", "output", "history_dataset_id"],
     "history_dataset_as_image": ["input", "output", "history_dataset_id", "path"],
@@ -31,8 +41,9 @@ VALID_ARGUMENTS = {
     "tool_stdout": ["step", "job_id"],
     "generate_galaxy_version": [],
     "generate_time": [],
-    "invocation_time": ["invocation_id"],
+    "visualization": DYNAMIC_ARGUMENTS,
     # Invocation Flavored Markdown
+    "invocation_time": ["invocation_id"],
     "invocation_outputs": [],
     "invocation_inputs": [],
 }
@@ -40,11 +51,11 @@ GALAXY_FLAVORED_MARKDOWN_CONTAINERS = list(VALID_ARGUMENTS.keys())
 GALAXY_FLAVORED_MARKDOWN_CONTAINER_REGEX = r'(?P<container>%s)' % "|".join(GALAXY_FLAVORED_MARKDOWN_CONTAINERS)
 
 ARG_VAL_REGEX = r'''[\w_\-]+|\"[^\"]+\"|\'[^\']+\''''
-FUNCTION_ARG = r'\s*\w+\s*=\s*(?:%s)\s*' % ARG_VAL_REGEX
+FUNCTION_ARG = r'\s*[\w\|]+\s*=\s*(?:%s)\s*' % ARG_VAL_REGEX
 # embed commas between arguments
-FUNCTION_MULTIPLE_ARGS = r'(?P<firstargcall>{})(?P<restargcalls>(?:,{})*)'.format(FUNCTION_ARG, FUNCTION_ARG)
+FUNCTION_MULTIPLE_ARGS = fr'(?P<firstargcall>{FUNCTION_ARG})(?P<restargcalls>(?:,{FUNCTION_ARG})*)'
 FUNCTION_MULTIPLE_ARGS_PATTERN = re.compile(FUNCTION_MULTIPLE_ARGS)
-FUNCTION_CALL_LINE_TEMPLATE = r'\s*%s\s*\((?:' + FUNCTION_MULTIPLE_ARGS + r')?\)\s*'
+FUNCTION_CALL_LINE_TEMPLATE = f"\\s*%s\\s*\\((?:{FUNCTION_MULTIPLE_ARGS})?\\)\\s*"
 GALAXY_MARKDOWN_FUNCTION_CALL_LINE = re.compile(FUNCTION_CALL_LINE_TEMPLATE % GALAXY_FLAVORED_MARKDOWN_CONTAINER_REGEX)
 WHITE_SPACE_ONLY_PATTERN = re.compile(r"^[\s]+$")
 
@@ -90,7 +101,11 @@ def validate_galaxy_markdown(galaxy_markdown, internal=True):
                 if function_calls > 1:
                     invalid_line("Only one Galaxy directive is allowed per fenced Galaxy block (```galaxy)")
                 container = func_call_match.group("container")
-                valid_args = VALID_ARGUMENTS[container]
+                valid_args_raw = VALID_ARGUMENTS[container]
+                if isinstance(valid_args_raw, DynamicArguments):
+                    continue
+                valid_args = cast(List[str], valid_args_raw)
+
                 first_arg_call = func_call_match.group("firstargcall")
 
                 def _validate_arg(arg_str):
@@ -119,7 +134,7 @@ def validate_galaxy_markdown(galaxy_markdown, internal=True):
 
     if expecting_container_close_for:
         template = "Invalid line %d: %s"
-        msg = template % (last_line_no, "close of block for [{expected_for}] expected".format(expected_for=expecting_container_close_for))
+        msg = template % (last_line_no, f"close of block for [{expecting_container_close_for}] expected")
         raise ValueError(msg)
 
 
@@ -133,7 +148,7 @@ def _split_markdown_lines(markdown):
     indent_fenced = False
     for line_number, line in enumerate(markdown.splitlines(True)):
         open_fence_this_iteration = False
-        indent_fenced = line.startswith("    ") or (indent_fenced and WHITE_SPACE_ONLY_PATTERN.match(line))
+        indent_fenced = bool(line.startswith("    ") or (indent_fenced and WHITE_SPACE_ONLY_PATTERN.match(line)))
         if not block_fenced:
             if BLOCK_FENCE_START.match(line):
                 open_fence_this_iteration = True
