@@ -2,9 +2,11 @@
 Mixins for Ratable model managers and serializers.
 """
 import logging
+from typing import Type
 
 from sqlalchemy.sql.expression import func
 
+from galaxy.model import ItemRatingAssociation
 from . import base
 
 log = logging.getLogger(__name__)
@@ -12,8 +14,7 @@ log = logging.getLogger(__name__)
 
 class RatableManagerMixin:
 
-    #: class of RatingAssociation (e.g. HistoryRatingAssociation)
-    rating_assoc = None
+    rating_assoc: Type[ItemRatingAssociation]
 
     def rating(self, item, user, as_int=True):
         """Returns the integer rating given to this item by the user.
@@ -48,7 +49,7 @@ class RatableManagerMixin:
         # TODO?: update and create to RatingsManager (if not overkill)
         rating = self.rating(item, user, as_int=False)
         if not rating:
-            rating = self.rating_assoc(user=user)
+            rating = self.rating_assoc(user, item)
             self.associate(rating, item)
         rating.rating = value
 
@@ -83,8 +84,8 @@ class RatableSerializerMixin:
         # than getting the rows and calc'ing both here with one query
         manager = self.manager
         return {
-            'average' : manager.ratings_avg(item),
-            'count'   : manager.ratings_count(item),
+            'average': manager.ratings_avg(item),
+            'count': manager.ratings_count(item),
         }
 
 
@@ -114,11 +115,11 @@ class RatableFilterMixin:
         self.fn_filter_parsers.update({
             'community_rating': {
                 'op': {
-                    'eq' : lambda i, v: self._ratings_avg_accessor(i) == v,
+                    'eq': lambda i, v: self._ratings_avg_accessor(i) == v,
                     # TODO: default to greater than (currently 'eq' due to base/controller.py)
-                    'ge' : lambda i, v: self._ratings_avg_accessor(i) >= v,
-                    'le' : lambda i, v: self._ratings_avg_accessor(i) <= v,
+                    'ge': lambda i, v: self._ratings_avg_accessor(i) >= v,
+                    'le': lambda i, v: self._ratings_avg_accessor(i) <= v,
                 },
-                'val' : float
+                'val': float
             }
         })
