@@ -3,8 +3,9 @@ Accessible models can be read and copied but not modified or deleted.
 
 Owned models can be modified and deleted.
 """
+from typing import Type
 
-from galaxy import exceptions
+from galaxy import exceptions, model
 
 
 class AccessibleManagerMixin:
@@ -14,6 +15,12 @@ class AccessibleManagerMixin:
     This can also be thought of as 'read but not modify' privileges.
     """
 
+    # declare what we are using from base ModelManager
+    model_class: Type[model._HasTable]
+
+    def by_id(self, id: int):
+        ...
+
     # don't want to override by_id since consumers will also want to fetch w/o any security checks
     def is_accessible(self, item, user, **kwargs):
         """
@@ -40,7 +47,7 @@ class AccessibleManagerMixin:
         """
         if self.is_accessible(item, user, **kwargs):
             return item
-        raise exceptions.ItemAccessibilityException("%s is not accessible by user" % (self.model_class.__name__))
+        raise exceptions.ItemAccessibilityException(f"{self.model_class.__name__} is not accessible by user")
 
     # TODO:?? are these even useful?
     def list_accessible(self, user, **kwargs):
@@ -74,6 +81,11 @@ class OwnableManagerMixin:
 
     This can also be thought of as write/edit privileges.
     """
+    # declare what we are using from base ModelManager
+    model_class: Type[model._HasTable]
+
+    def by_id(self, id: int):
+        ...
 
     def is_owner(self, item, user, **kwargs):
         """
@@ -100,7 +112,7 @@ class OwnableManagerMixin:
         """
         if self.is_owner(item, user, **kwargs):
             return item
-        raise exceptions.ItemOwnershipException("%s is not owned by user" % (self.model_class.__name__))
+        raise exceptions.ItemOwnershipException(f"{self.model_class.__name__} is not owned by user")
 
     def list_owned(self, user, **kwargs):
         """
