@@ -1,26 +1,28 @@
 import logging
-from uuid import uuid4
+from typing import Optional, TYPE_CHECKING, Union
+from uuid import UUID
 
 from sqlalchemy import sql
 
 from galaxy import exceptions
 from galaxy import model
 from galaxy.exceptions import DuplicatedIdentifierException
-from .base import ModelManager
+from galaxy.tool_util.cwl import tool_proxy
+from .base import ModelManager, raise_filter_err
 from .executables import artifact_class
 
 log = logging.getLogger(__name__)
 
+if TYPE_CHECKING:
+    from galaxy.managers.base import OrmFilterParsersType
+
 
 class DynamicToolManager(ModelManager):
     """ Manages dynamic tools stored in Galaxy's database.
     """
     model_class = model.DynamicTool
 
-    def __init__(self, app):
-        super().__init__(app)
-
-    def get_tool_by_uuid(self, uuid):
+    def get_tool_by_uuid(self, uuid: Optional[Union[UUID, str]]):
         dynamic_tool = self._one_or_none(
             self.query().filter(self.model_class.uuid == uuid)
         )
@@ -39,52 +41,63 @@ class DynamicToolManager(ModelManager):
         return dynamic_tool
 
     def create_tool(self, trans, tool_payload, allow_load=True):
-        src = tool_payload.get("src", "representation")
-        is_path = src == "from_path"
-
-        if is_path:
-            tool_format, representation, object_id = artifact_class(None, tool_payload)
-        else:
-            assert src == "representation"
-            if "representation" not in tool_payload:
-                raise exceptions.ObjectAttributeMissingException(
-                    "A tool 'representation' is required."
-                )
-
-            representation = tool_payload["representation"]
-            if "class" not in representation:
-                raise exceptions.ObjectAttributeMissingException(
-                    "Current tool representations require 'class'."
-                )
-
-        enable_beta_formats = getattr(self.app.config, "enable_beta_tool_formats", False)
-        if not enable_beta_formats:
+        if not getattr(self.app.config, "enable_beta_tool_formats", False):
             raise exceptions.ConfigDoesNotAllowException("Set 'enable_beta_tool_formats' in Galaxy config to create dynamic tools.")
 
-        tool_format = representation["class"]
-        tool_directory = tool_payload.get("tool_directory", None)
-        tool_path = None
-        if tool_format == "GalaxyTool":
-            uuid = tool_payload.get("uuid", None)
-            if uuid is None:
-                uuid = uuid4()
+        dynamic_tool = None
+        uuid_str = tool_payload.get("uuid")
+        # Convert uuid_str to UUID or generate new if None
+        uuid = model.get_uuid(uuid_str)
+        if uuid_str:
+            # TODO: enforce via DB constraint and catch appropriate
+            # exception.
+            dynamic_tool = self.get_tool_by_uuid(uuid_str)
+            if dynamic_tool:
+                if not allow_load:
+                    raise DuplicatedIdentifierException(dynamic_tool.id)
+                assert dynamic_tool.uuid == uuid
+        if not dynamic_tool:
+            src = tool_payload.get("src", "representation")
+            is_path = src == "from_path"
 
-            tool_id = representation.get("id", None)
-            if tool_id is None:
-                tool_id = str(uuid)
+            if is_path:
+                tool_format, representation, _ = artifact_class(None, tool_payload)
+            else:
+                assert src == "representation"
+                representation = tool_payload.get("representation")
+                if not representation:
+                    raise exceptions.ObjectAttributeMissingException(
+                        "A tool 'representation' is required."
+                    )
 
-            tool_version = representation.get("version", None)
-            value = representation
-        else:
-            raise Exception("Unknown tool type encountered.")
-        # TODO: enforce via DB constraint and catch appropriate
-        # exception.
-        existing_tool = self.get_tool_by_uuid(uuid)
-        if existing_tool is not None and not allow_load:
-            raise DuplicatedIdentifierException(existing_tool.id)
-        elif existing_tool:
-            dynamic_tool = existing_tool
-        else:
+                tool_format = representation.get("class")
+                if not tool_format:
+                    raise exceptions.ObjectAttributeMissingException(
+                        "Current tool representations require 'class'."
+                    )
+
+            tool_path = tool_payload.get("path")
+            tool_directory = tool_payload.get("tool_directory")
+            if tool_format == "GalaxyTool":
+                tool_id = representation.get("id")
+                if not tool_id:
+                    tool_id = str(uuid)
+            elif tool_format in ("CommandLineTool", "ExpressionTool"):
+                # CWL tools
+                if is_path:
+                    proxy = tool_proxy(tool_path=tool_path, uuid=uuid)
+                else:
+                    # Build a tool proxy so that we can convert to the persistable
+                    # hash.
+                    proxy = tool_proxy(
+                        tool_object=representation["raw_process_reference"],
+                        tool_directory=tool_directory,
+                        uuid=uuid,
+                    )
+                tool_id = proxy.galaxy_id()
+            else:
+                raise Exception(f"Unknown tool format [{tool_format}] encountered.")
+            tool_version = representation.get("version")
             dynamic_tool = self.create(
                 tool_format=tool_format,
                 tool_id=tool_id,
@@ -92,9 +105,9 @@ class DynamicToolManager(ModelManager):
                 tool_path=tool_path,
                 tool_directory=tool_directory,
                 uuid=uuid,
-                value=value,
+                value=representation,
             )
-            self.app.toolbox.load_dynamic_tool(dynamic_tool)
+        self.app.toolbox.load_dynamic_tool(dynamic_tool)
         return dynamic_tool
 
     def list_tools(self, active=True):
@@ -106,6 +119,7 @@ class DynamicToolManager(ModelManager):
 
 
 class ToolFilterMixin:
+    orm_filter_parsers: "OrmFilterParsersType"
 
     def create_tool_filter(self, attr, op, val):
 
@@ -115,7 +129,7 @@ class ToolFilterMixin:
             elif op == 'contains':
                 cond = model.Job.table.c.tool_id.contains(val, autoescape=True)
             else:
-                self.raise_filter_err(attr, op, val, 'bad op in filter')
+                raise_filter_err(attr, op, val, 'bad op in filter')
             if model_class is model.HistoryDatasetAssociation:
                 return sql.expression.and_(
                     model.Job.table.c.id == model.JobToOutputDatasetAssociation.table.c.job_id,
