@@ -5,16 +5,12 @@ reloading the toolbox, etc., across multiple processes.
 
 import importlib
 import logging
+import math
 import socket
 import sys
 import threading
 import time
 from inspect import ismodule
-try:
-    from math import inf
-except ImportError:
-    # python 2 doesn't have math.inf, but can use float('inf')
-    inf = float('inf')
 
 from kombu import (
     Consumer,
@@ -25,7 +21,6 @@ from kombu.mixins import ConsumerProducerMixin
 from kombu.pools import (
     producers,
 )
-from six.moves import reload_module
 
 import galaxy.queues
 from galaxy import util
@@ -42,10 +37,10 @@ def send_local_control_task(app, task, get_response=False, kwargs=None):
     """
     if kwargs is None:
         kwargs = {}
-    log.info("Queuing {} task {} for {}.".format("sync" if get_response else "async", task, app.config.server_name))
+    log.info(f"Queuing {'sync' if get_response else 'async'} task {task} for {app.config.server_name}.")
     payload = {'task': task,
                'kwargs': kwargs}
-    routing_key = 'control.{}@{}'.format(app.config.server_name, socket.gethostname())
+    routing_key = f'control.{app.config.server_name}@{socket.gethostname()}'
     control_task = ControlTask(app.queue_worker)
     return control_task.send_task(payload, routing_key, local=True, get_response=get_response)
 
@@ -61,7 +56,7 @@ def send_control_task(app, task, noop_self=False, get_response=False, routing_ke
     """
     if kwargs is None:
         kwargs = {}
-    log.info("Sending %s control task." % task)
+    log.info(f"Sending {task} control task.")
     payload = {'task': task,
                'kwargs': kwargs}
     if noop_self:
@@ -156,7 +151,7 @@ def create_panel_section(app, **kwargs):
 def reload_tool(app, **kwargs):
     params = util.Params(kwargs)
     tool_id = params.get('tool_id', None)
-    log.debug("Executing reload tool task for %s" % tool_id)
+    log.debug(f"Executing reload tool task for {tool_id}")
     if tool_id:
         app.toolbox.reload_tool_by_id(tool_id)
     else:
@@ -182,8 +177,6 @@ def _get_new_toolbox(app, save_integrated_tool_panel=True):
     """
     from galaxy import tools
     from galaxy.tools.special_tools import load_lib_tools
-    if hasattr(app, 'tool_shed_repository_cache'):
-        app.tool_shed_repository_cache.rebuild()
     tool_configs = app.config.tool_configs
 
     new_toolbox = tools.ToolBox(tool_configs, app.config.tool_path, app, save_integrated_tool_panel=save_integrated_tool_panel)
@@ -200,8 +193,6 @@ def reload_data_managers(app, **kwargs):
     reload_timer = util.ExecutionTimer()
     from galaxy.tools.data_manager.manager import DataManagers
     log.debug("Executing data managers reload on '%s'", app.config.server_name)
-    if hasattr(app, 'tool_shed_repository_cache'):
-        app.tool_shed_repository_cache.rebuild()
     app._configure_tool_data_tables(from_shed_config=False)
     reload_tool_data_tables(app)
     reload_count = app.data_managers._reload_count
@@ -216,7 +207,7 @@ def reload_data_managers(app, **kwargs):
 
 def reload_display_application(app, **kwargs):
     display_application_ids = kwargs.get('display_application_ids', None)
-    log.debug("Executing display application reload task for %s" % display_application_ids)
+    log.debug(f"Executing display application reload task for {display_application_ids}")
     app.datatypes_registry.reload_display_applications(display_application_ids)
 
 
@@ -233,7 +224,7 @@ def recalculate_user_disk_usage(app, **kwargs):
         if user:
             user.calculate_and_set_disk_usage()
         else:
-            log.error("Recalculate user disk usage task failed, user %s not found" % user_id)
+            log.error(f"Recalculate user disk usage task failed, user {user_id} not found")
     else:
         log.error("Recalculate user disk usage task received without user_id.")
 
@@ -260,10 +251,10 @@ def reload_job_rules(app, **kwargs):
     for module in job_rule_modules(app):
         rules_module_name = module.__name__
         for name, module in sys.modules.items():
-            if ((name == rules_module_name or name.startswith(rules_module_name + '.'))
+            if ((name == rules_module_name or name.startswith(f"{rules_module_name}."))
                     and ismodule(module)):
                 log.debug("Reloading job rules module: %s", name)
-                reload_module(module)
+                importlib.reload(module)
     log.debug("Job rules reloaded %s", reload_timer)
 
 
@@ -383,7 +374,7 @@ class GalaxyQueueWorker(ConsumerProducerMixin, threading.Thread):
             if body.get('noop', None) != self.app.config.server_name:
                 try:
                     f = self.task_mapping[body['task']]
-                    if message.headers.get('epoch', inf) > self.epoch:
+                    if message.headers.get('epoch', math.inf) > self.epoch:
                         # Message was created after QueueWorker was started, execute
                         log.info("Instance '%s' received '%s' task, executing now.", self.app.config.server_name, body['task'])
                         result = f(self.app, **body['kwargs'])
@@ -397,7 +388,7 @@ class GalaxyQueueWorker(ConsumerProducerMixin, threading.Thread):
             else:
                 result = 'NO_OP'
         else:
-            log.warning("Received a malformed task message:\n%s" % body)
+            log.warning(f"Received a malformed task message:\n{body}")
         if message.properties.get('reply_to'):
             self.producer.publish(
                 {'result': result},
@@ -411,3 +402,4 @@ class GalaxyQueueWorker(ConsumerProducerMixin, threading.Thread):
 
     def shutdown(self):
         self.should_stop = True
+        self.join()
