@@ -4,6 +4,7 @@ All message queues used by Galaxy
 
 """
 import socket
+from typing import Optional
 
 from kombu import (
     Connection,
@@ -22,7 +23,7 @@ def all_control_queues_for_declare(application_stack):
     """
     # Get all active processes and construct queues for each process
     process_names = ("{p.server_name}@{p.hostname}".format(p=p) for p in application_stack.app.database_heartbeat.get_active_processes())
-    return [Queue("control.%s" % server_name, galaxy_exchange, routing_key='control.*') for server_name in process_names]
+    return [Queue(f"control.{server_name}", galaxy_exchange, routing_key='control.*') for server_name in process_names]
 
 
 def control_queues_from_config(config):
@@ -31,13 +32,13 @@ def control_queues_from_config(config):
     galaxy process's config
     """
     hostname = socket.gethostname()
-    process_name = "{server_name}@{hostname}".format(server_name=config.server_name, hostname=hostname)
-    exchange_queue = Queue("control.%s" % process_name, galaxy_exchange, routing_key='control.*')
-    non_exchange_queue = Queue("control.%s" % process_name, routing_key='control.%s' % process_name)
+    process_name = f"{config.server_name}@{hostname}"
+    exchange_queue = Queue(f"control.{process_name}", galaxy_exchange, routing_key='control.*')
+    non_exchange_queue = Queue(f"control.{process_name}", routing_key=f'control.{process_name}')
     return exchange_queue, non_exchange_queue
 
 
-def connection_from_config(config):
+def connection_from_config(config) -> Optional[Connection]:
     if config.amqp_internal_connection:
         return Connection(config.amqp_internal_connection)
     else:
