@@ -5,10 +5,15 @@ into Galaxy from the Tool Shed.
 import json
 import logging
 import os
-
-from six.moves.urllib.error import HTTPError
-from six.moves.urllib.parse import urlencode, urlparse
-from six.moves.urllib.request import Request, urlopen
+from urllib.error import HTTPError
+from urllib.parse import (
+    urlencode,
+    urlparse,
+)
+from urllib.request import (
+    Request,
+    urlopen,
+)
 
 from galaxy.tool_shed.galaxy_install.tools import tool_panel_manager
 from galaxy.tool_shed.util import repository_util
@@ -17,6 +22,7 @@ from galaxy.tool_shed.util.shed_util_common import get_ctx_rev
 from galaxy.util import (
     asbool,
     build_url,
+    DEFAULT_SOCKET_TIMEOUT,
     smart_str,
     unicodify,
     url_get,
@@ -43,7 +49,7 @@ class RepositoryDependencyInstallManager:
         install_model = self.app.install_model
         log.debug("Building repository dependency relationships...")
         for repo_info_dict in repo_info_dicts:
-            for name, repo_info_tuple in repo_info_dict.items():
+            for repo_info_tuple in repo_info_dict.values():
                 description, \
                     repository_clone_url, \
                     changeset_revision, \
@@ -209,7 +215,7 @@ class RepositoryDependencyInstallManager:
                             elif repository_db_record.status in [install_model.ToolShedRepository.installation_status.DEACTIVATED]:
                                 # The current tool shed repository is deactivated, so updating its database record
                                 # is not necessary - just activate it.
-                                log.info("Reactivating deactivated tool_shed_repository '%s'." % str(repository_db_record.name))
+                                log.info(f"Reactivating deactivated tool_shed_repository '{str(repository_db_record.name)}'.")
                                 self.app.installed_repository_manager.activate_repository(repository_db_record)
                                 # No additional updates to the database record are necessary.
                                 can_update_db_record = False
@@ -227,10 +233,10 @@ class RepositoryDependencyInstallManager:
                         # The database record for the tool shed repository currently being processed can be updated.
                         # Get the repository metadata to see where it was previously located in the tool panel.
                         tpm = tool_panel_manager.ToolPanelManager(self.app)
-                        if repository_db_record and repository_db_record.metadata:
+                        if repository_db_record and repository_db_record.metadata_:
                             _, tool_panel_section_key = \
                                 tpm.handle_tool_panel_selection(toolbox=self.app.toolbox,
-                                                                metadata=repository_db_record.metadata,
+                                                                metadata=repository_db_record.metadata_,
                                                                 no_changes_checked=no_changes_checked,
                                                                 tool_panel_section_id=tool_panel_section_id,
                                                                 new_tool_panel_section_label=new_tool_panel_section_label)
@@ -477,13 +483,13 @@ class RepositoryDependencyInstallManager:
         will be set to the default NEW state.  This will enable the repository to be
         freshly installed.
         """
-        debug_msg = "Resetting tool_shed_repository '%s' for installation.\n" % repository.name
+        debug_msg = f"Resetting tool_shed_repository '{repository.name}' for installation.\n"
         debug_msg += "The current state of the tool_shed_repository is:\n"
-        debug_msg += "deleted: %s\n" % repository.deleted
-        debug_msg += "tool_shed_status: %s\n" % repository.tool_shed_status
-        debug_msg += "uninstalled: %s\n" % repository.uninstalled
-        debug_msg += "status: %s\n" % repository.status
-        debug_msg += "error_message: %s\n" % repository.error_message
+        debug_msg += f"deleted: {repository.deleted}\n"
+        debug_msg += f"tool_shed_status: {repository.tool_shed_status}\n"
+        debug_msg += f"uninstalled: {repository.uninstalled}\n"
+        debug_msg += f"status: {repository.status}\n"
+        debug_msg += f"error_message: {repository.error_message}\n"
         log.debug(debug_msg)
         repository.deleted = False
         repository.tool_shed_status = None
@@ -496,7 +502,7 @@ class RepositoryDependencyInstallManager:
 
 def _urlopen(url, data=None):
     scheme = urlparse(url).scheme
-    assert scheme in ('http', 'https', 'ftp'), 'Invalid URL scheme: %s' % scheme
+    assert scheme in ('http', 'https', 'ftp'), f'Invalid URL scheme: {scheme}'
     if data is not None:
         data = smart_str(data)
-    return urlopen(Request(url, data))
+    return urlopen(Request(url, data), timeout=DEFAULT_SOCKET_TIMEOUT)
