@@ -56,7 +56,22 @@ class ToolPanelManager:
                 config_elems.append(elem)
             # Add the new elements to the in-memory list of config_elems.
             for elem_entry in elem_list:
-                config_elems.append(elem_entry)
+                if elem_entry.tag == 'section':
+                    # Loop through section entries in the in-memory tool panel.
+                    for existing_elem in config_elems:
+                        # Compare the section ID for each one to the section ID for the tool being installed.
+                        if existing_elem.tag == 'section' and existing_elem.attrib.get('id', None) == elem_entry.attrib.get('id', None):
+                            # We've found a section, add the incoming tools to it.
+                            for child in elem_entry:
+                                existing_elem.append(child)
+                            # Break out of the config_elems loop back to the elem_list loop.
+                            break
+                    # If we reach this point, no section was found. Create one with contents.
+                    else:
+                        config_elems.append(elem_entry)
+                # This is not a section, but a tool or label. No need to search for matching sections, just add it.
+                else:
+                    config_elems.append(elem_entry)
             # Persist the altered shed_tool_config file.
             self.config_elems_to_xml_file(config_elems, shed_tool_conf, tool_path, tool_cache_data_dir)
             self.app.wait_for_toolbox_reload(old_toolbox)
@@ -64,8 +79,9 @@ class ToolPanelManager:
             log.error(error_message)
 
     def add_to_tool_panel(self, repository_name, repository_clone_url, changeset_revision, repository_tools_tups, owner,
-                          shed_tool_conf, tool_panel_dict, new_install=True, tool_panel_section_mapping={}):
+                          shed_tool_conf, tool_panel_dict, new_install=True, tool_panel_section_mapping=None):
         """A tool shed repository is being installed or updated so handle tool panel alterations accordingly."""
+        tool_panel_section_mapping = tool_panel_section_mapping or {}
         # We need to change the in-memory version and the file system version of the shed_tool_conf file.
         shed_tool_conf_dict = self.get_shed_tool_conf_dict(shed_tool_conf)
         tool_panel_dict = self.update_tool_panel_dict(tool_panel_dict, tool_panel_section_mapping, repository_tools_tups)
@@ -101,8 +117,8 @@ class ToolPanelManager:
         value of config_filename.
         """
         try:
-            tool_cache_data_dir = ' tool_cache_data_dir="{}"'.format(tool_cache_data_dir) if tool_cache_data_dir else ''
-            root = parse_xml_string('<?xml version="1.0"?>\n<toolbox tool_path="{}"{}></toolbox>'.format(tool_path, tool_cache_data_dir))
+            tool_cache_data_dir = f' tool_cache_data_dir="{tool_cache_data_dir}"' if tool_cache_data_dir else ''
+            root = parse_xml_string(f'<?xml version="1.0"?>\n<toolbox tool_path="{tool_path}"{tool_cache_data_dir}></toolbox>')
             for elem in config_elems:
                 root.append(elem)
             with RenamedTemporaryFile(config_filename, mode='w') as fh:
@@ -167,11 +183,15 @@ class ToolPanelManager:
         The intent is to call this method for every tool config in a repository and
         append each of these as entries to a tool panel dictionary for the repository.
         This enables each tool to be loaded into a different section in the tool panel.
-        {<Tool guid> :
-           [{ tool_config : <tool_config_file>,
-              id: <ToolSection id>,
-              version : <ToolSection version>,
-              name : <TooSection name>}]}
+
+        .. code-block::
+
+            {<Tool guid> :
+                [{ tool_config : <tool_config_file>,
+                    id: <ToolSection id>,
+                    version : <ToolSection version>,
+                    name : <TooSection name>}]}
+
         """
         tool_panel_dict = {}
         file_name = strip_path(tool_config)
@@ -192,7 +212,7 @@ class ToolPanelManager:
         tool_panel_dict = {}
         shed_tool_conf, tool_path, relative_install_dir = \
             get_tool_panel_config_tool_path_install_dir(self.app, repository)
-        metadata = repository.metadata
+        metadata = repository.metadata_
         # Create a dictionary of tool guid and tool config file name for each tool in the repository.
         guids_and_configs = {}
         if 'tools' in metadata:
@@ -248,7 +268,7 @@ class ToolPanelManager:
             for tool_section_dict in tool_section_dicts:
                 tool_section = None
                 inside_section = False
-                section_in_elem_list = False
+                section_in_elem_list = None
                 if tool_section_dict['id']:
                     inside_section = True
                     # Create a new section element only if we haven't already created it.
@@ -256,7 +276,7 @@ class ToolPanelManager:
                         if elem.tag == 'section':
                             section_id = elem.get('id', None)
                             if section_id == tool_section_dict['id']:
-                                section_in_elem_list = True
+                                section_in_elem_list = index
                                 tool_section = elem
                                 break
                     if tool_section is None:
@@ -274,8 +294,8 @@ class ToolPanelManager:
                                                     tool,
                                                     tool_section if inside_section else None)
                 if inside_section:
-                    if section_in_elem_list:
-                        elem_list[index] = tool_section
+                    if section_in_elem_list is not None:
+                        elem_list[section_in_elem_list] = tool_section
                     else:
                         elem_list.append(tool_section)
                 else:
@@ -334,7 +354,7 @@ class ToolPanelManager:
                 file_name = strip_path(shed_tool_conf_dict['config_filename'])
                 if shed_tool_conf == file_name:
                     return shed_tool_conf_dict
-        raise RequestParameterInvalidException("Requested shed_tool_conf '%s' is not an active shed_tool_config_file" % shed_tool_conf)
+        raise RequestParameterInvalidException(f"Requested shed_tool_conf '{shed_tool_conf}' is not an active shed_tool_config_file")
 
     def handle_tool_panel_section(self, toolbox, tool_panel_section_id=None, new_tool_panel_section_label=None):
         """Return a ToolSection object retrieved from the current in-memory tool_panel."""
@@ -447,7 +467,7 @@ class ToolPanelManager:
         # information so the tools can be displayed in the same way when the repository is
         # activated or reinstalled.
         tool_panel_dict = self.generate_tool_panel_dict_from_shed_tool_conf_entries(repository)
-        repository.metadata['tool_panel_section'] = tool_panel_dict
+        repository.metadata_['tool_panel_section'] = tool_panel_dict
         self.app.install_model.context.add(repository)
         self.app.install_model.context.flush()
         # Create a list of guids for all tools that will be removed from the in-memory tool panel
@@ -461,15 +481,15 @@ class ToolPanelManager:
         shed_tool_conf_dict = self.get_shed_tool_conf_dict(shed_tool_conf)
         if uninstall:
             # Remove from the shed_tool_conf file on disk.
-            self.remove_from_shed_tool_config(shed_tool_conf_dict, repository.metadata)
+            self.remove_from_shed_tool_config(shed_tool_conf_dict, repository.metadata_)
 
     def update_tool_panel_dict(self, tool_panel_dict, tool_panel_section_mapping, repository_tools_tups):
         for tool_guid in tool_panel_dict:
             if tool_guid not in tool_panel_section_mapping:
                 continue
-            for idx, tool in enumerate(tool_panel_dict[tool_guid]):
+            for tool in tool_panel_dict[tool_guid]:
                 section_name = tool_panel_section_mapping[tool_guid]['tool_panel_section']
                 section_id = str(tool_panel_section_mapping[tool_guid]['tool_panel_section'].lower().replace(' ', '_'))
-                tool_panel_dict[tool_guid][idx]['name'] = section_name
-                tool_panel_dict[tool_guid][idx]['id'] = section_id
+                tool['name'] = section_name
+                tool['id'] = section_id
         return tool_panel_dict
