@@ -26,9 +26,9 @@ def clone_repository(repository_clone_url, repository_file_dir, ctx_rev=None):
         subprocess.check_output(cmd, stderr=subprocess.STDOUT, stdin=subprocess.DEVNULL)
         return True, None
     except Exception as e:
-        error_message = 'Error cloning repository: %s' % unicodify(e)
+        error_message = f'Error cloning repository: {unicodify(e)}'
         if isinstance(e, subprocess.CalledProcessError):
-            error_message += "\nOutput was:\n%s" % unicodify(e.output)
+            error_message += f"\nOutput was:\n{unicodify(e.output)}"
         log.error(error_message)
         return False, error_message
 
@@ -60,7 +60,7 @@ def get_changectx_for_changeset(repo, changeset_revision, **kwd):
 
 
 def get_config_from_disk(config_file, relative_install_dir):
-    for root, dirs, files in os.walk(relative_install_dir):
+    for root, _dirs, files in os.walk(relative_install_dir):
         if root.find('.hg') < 0:
             for name in files:
                 if name == config_file:
@@ -112,9 +112,9 @@ def pull_repository(repo_path, repository_clone_url, ctx_rev):
     try:
         subprocess.check_output(['hg', 'pull', '-r', ctx_rev, repository_clone_url], stderr=subprocess.STDOUT, cwd=repo_path)
     except Exception as e:
-        error_message = "Error pulling revision '{}': {}".format(ctx_rev, unicodify(e))
+        error_message = f"Error pulling revision '{ctx_rev}': {unicodify(e)}"
         if isinstance(e, subprocess.CalledProcessError):
-            error_message += "\nOutput was:\n%s" % unicodify(e.output)
+            error_message += f"\nOutput was:\n{unicodify(e.output)}"
         raise Exception(error_message)
 
 
@@ -174,9 +174,9 @@ def update_repository(repo_path, ctx_rev=None):
     try:
         subprocess.check_output(cmd, stderr=subprocess.STDOUT, cwd=repo_path)
     except Exception as e:
-        error_message = 'Error updating repository: %s' % unicodify(e)
+        error_message = f'Error updating repository: {unicodify(e)}'
         if isinstance(e, subprocess.CalledProcessError):
-            error_message += "\nOutput was:\n%s" % unicodify(e.output)
+            error_message += f"\nOutput was:\n{unicodify(e.output)}"
         raise Exception(error_message)
 
 
