@@ -3,22 +3,32 @@ import logging
 import os
 import socket
 import tempfile
-from collections import OrderedDict
+from dataclasses import dataclass
+from io import StringIO
 from json import dump, dumps
+from typing import Dict, List, Optional
 from urllib.parse import urlparse
 
-from six import StringIO
-from sqlalchemy.orm import eagerload_all
+from sqlalchemy.orm import joinedload
 from webob.compat import cgi_FieldStorage
 
-from galaxy import datatypes, util
+from galaxy import util
+from galaxy.datatypes.sniff import stream_to_file
 from galaxy.exceptions import (
     ConfigDoesNotAllowException,
-    ObjectInvalid,
     RequestParameterInvalidException,
 )
-from galaxy.model import tags
-from galaxy.util import unicodify
+from galaxy.model import (
+    FormDefinition,
+    LibraryDataset,
+    LibraryFolder,
+    Role,
+    tags,
+)
+from galaxy.util import (
+    is_url,
+    unicodify
+)
 from galaxy.util.path import external_chown
 
 log = logging.getLogger(__name__)
@@ -26,7 +36,7 @@ log = logging.getLogger(__name__)
 
 def validate_datatype_extension(datatypes_registry, ext):
     if ext and ext not in ('auto', 'data') and not datatypes_registry.get_datatype_by_extension(ext):
-        raise RequestParameterInvalidException("Requested extension '%s' unknown, cannot upload dataset." % ext)
+        raise RequestParameterInvalidException(f"Requested extension '{ext}' unknown, cannot upload dataset.")
 
 
 def validate_url(url, ip_allowlist):
@@ -131,7 +141,7 @@ def persist_uploads(params, trans):
             elif type(f) == dict and 'local_filename' not in f:
                 raise Exception('Uploaded file was encoded in a way not understood by Galaxy.')
             if 'url_paste' in upload_dataset and upload_dataset['url_paste'] and upload_dataset['url_paste'].strip() != '':
-                upload_dataset['url_paste'] = datatypes.sniff.stream_to_file(
+                upload_dataset['url_paste'] = stream_to_file(
                     StringIO(validate_url(upload_dataset['url_paste'], trans.app.config.fetch_url_allowlist_ips)),
                     prefix="strio_url_paste_"
                 )
@@ -142,34 +152,51 @@ def persist_uploads(params, trans):
     return params
 
 
-def handle_library_params(trans, params, folder_id, replace_dataset=None):
+@dataclass
+class LibraryParams:
+    roles: List[Role]
+    tags: Optional[List[str]]
+    template: Optional[FormDefinition]
+    template_field_contents: Dict[str, str]
+    folder: LibraryFolder
+    message: str
+    replace_dataset: Optional[LibraryDataset]
+
+
+def handle_library_params(trans, params, folder_id: str, replace_dataset: Optional[LibraryDataset] = None) -> LibraryParams:
     # FIXME: the received params has already been parsed by util.Params() by the time it reaches here,
     # so no complex objects remain.  This is not good because it does not allow for those objects to be
     # manipulated here.  The received params should be the original kwd from the initial request.
-    library_bunch = util.bunch.Bunch()
-    library_bunch.replace_dataset = replace_dataset
-    library_bunch.message = params.get('ldda_message', '')
+    message = params.get('ldda_message', '')
     # See if we have any template field contents
-    library_bunch.template_field_contents = {}
+    template_field_contents = {}
     template_id = params.get('template_id', None)
-    library_bunch.folder = trans.sa_session.query(trans.app.model.LibraryFolder).get(trans.security.decode_id(folder_id))
+    folder = trans.sa_session.query(LibraryFolder).get(trans.security.decode_id(folder_id))
     # We are inheriting the folder's info_association, so we may have received inherited contents or we may have redirected
     # here after the user entered template contents ( due to errors ).
+    template: Optional[FormDefinition] = None
     if template_id not in [None, 'None']:
-        library_bunch.template = trans.sa_session.query(trans.app.model.FormDefinition).get(template_id)
-        for field in library_bunch.template.fields:
+        template = trans.sa_session.query(FormDefinition).get(template_id)
+        assert template
+        for field in template.fields:
             field_name = field['name']
             if params.get(field_name, False):
                 field_value = util.restore_text(params.get(field_name, ''))
-                library_bunch.template_field_contents[field_name] = field_value
-    else:
-        library_bunch.template = None
-    library_bunch.roles = []
+                template_field_contents[field_name] = field_value
+    roles: List[Role] = []
     for role_id in util.listify(params.get('roles', [])):
-        role = trans.sa_session.query(trans.app.model.Role).get(role_id)
-        library_bunch.roles.append(role)
-    library_bunch.tags = params.get('tags', None)
-    return library_bunch
+        role = trans.sa_session.query(Role).get(role_id)
+        roles.append(role)
+    tags = params.get('tags', None)
+    return LibraryParams(
+        folder=folder,
+        message=message,
+        roles=roles,
+        tags=tags,
+        template=template,
+        template_field_contents=template_field_contents,
+        replace_dataset=replace_dataset,
+    )
 
 
 def __new_history_upload(trans, uploaded_dataset, history=None, state=None):
@@ -187,7 +214,7 @@ def __new_history_upload(trans, uploaded_dataset, history=None, state=None):
     else:
         hda.state = hda.states.QUEUED
     trans.sa_session.flush()
-    history.add_dataset(hda, genome_build=uploaded_dataset.dbkey)
+    history.add_dataset(hda, genome_build=uploaded_dataset.dbkey, quota=False)
     permissions = trans.app.security_agent.history_get_default_permissions(history)
     trans.app.security_agent.set_all_dataset_permissions(hda.dataset, permissions)
     trans.sa_session.flush()
@@ -208,7 +235,7 @@ def __new_library_upload(trans, cntrller, uploaded_dataset, library_bunch, tag_h
             if matches:
                 folder = matches[0]
             else:
-                new_folder = trans.app.model.LibraryFolder(name=name, description='Automatically created by upload tool')
+                new_folder = LibraryFolder(name=name, description='Automatically created by upload tool')
                 new_folder.genome_build = trans.app.genome_builds.default_value
                 folder.add_folder(new_folder)
                 trans.sa_session.add(new_folder)
@@ -231,12 +258,12 @@ def __new_library_upload(trans, cntrller, uploaded_dataset, library_bunch, tag_h
                                                             sa_session=trans.sa_session)
     if uploaded_dataset.get('tag_using_filenames', False):
         tag_from_filename = os.path.splitext(os.path.basename(uploaded_dataset.name))[0]
-        tag_handler.apply_item_tag(item=ldda, user=trans.user, name='name', value=tag_from_filename)
+        tag_handler.apply_item_tag(item=ldda, user=trans.user, name='name', value=tag_from_filename, flush=False)
 
     tags_list = uploaded_dataset.get('tags', False)
     if tags_list:
         for tag in tags_list:
-            tag_handler.apply_item_tag(item=ldda, user=trans.user, name='name', value=tag)
+            tag_handler.apply_item_tag(item=ldda, user=trans.user, name='name', value=tag, flush=False)
 
     trans.sa_session.add(ldda)
     if state:
@@ -293,12 +320,12 @@ def new_upload(trans, cntrller, uploaded_dataset, library_bunch=None, history=No
         if library_bunch.tags and not uploaded_dataset.tags:
             new_tags = tag_handler.parse_tags_list(library_bunch.tags)
             for tag in new_tags:
-                tag_handler.apply_item_tag(user=trans.user, item=upload_target_dataset_instance, name=tag[0], value=tag[1])
+                tag_handler.apply_item_tag(user=trans.user, item=upload_target_dataset_instance, name=tag[0], value=tag[1], flush=False)
     else:
         upload_target_dataset_instance = __new_history_upload(trans, uploaded_dataset, history=history, state=state)
 
     if tag_list:
-        tag_handler.add_tags_from_list(trans.user, upload_target_dataset_instance, tag_list)
+        tag_handler.add_tags_from_list(trans.user, upload_target_dataset_instance, tag_list, flush=False)
 
     return upload_target_dataset_instance
 
@@ -378,7 +405,7 @@ def create_paramfile(trans, uploaded_datasets):
             # TODO: This will have to change when we start bundling inputs.
             # Also, in_place above causes the file to be left behind since the
             # user cannot remove it unless the parent directory is writable.
-            if link_data_only == 'copy_files' and trans.user:
+            if link_data_only == 'copy_files' and trans.user and trans.app.config.external_chown_script and not is_url(uploaded_dataset.path):
                 external_chown(uploaded_dataset.path,
                                trans.user.system_user_pwent(trans.app.config.real_system_username),
                                trans.app.config.external_chown_script, description="uploaded file")
@@ -394,6 +421,7 @@ def create_job(trans, params, tool, json_file_path, outputs, folder=None, histor
     Create the upload job.
     """
     job = trans.app.model.Job()
+    trans.sa_session.add(job)
     job.galaxy_version = trans.app.config.version_major
     galaxy_session = trans.get_galaxy_session()
     if type(galaxy_session) == trans.model.GalaxySession:
@@ -409,16 +437,10 @@ def create_job(trans, params, tool, json_file_path, outputs, folder=None, histor
     job.tool_id = tool.id
     job.tool_version = tool.version
     job.dynamic_tool = tool.dynamic_tool
-    job.set_state(job.states.UPLOAD)
-    trans.sa_session.add(job)
-    trans.sa_session.flush()
-    log.info('tool %s created job id %d' % (tool.id, job.id))
-    trans.log_event('created job id %d' % job.id, tool_id=tool.id)
 
     for name, value in tool.params_to_strings(params, trans.app).items():
         job.add_parameter(name, value)
     job.add_parameter('paramfile', dumps(json_file_path))
-    object_store_id = None
     for i, output_object in enumerate(outputs):
         output_name = "output%i" % i
         if hasattr(output_object, "collection"):
@@ -430,28 +452,13 @@ def create_job(trans, params, tool, json_file_path, outputs, folder=None, histor
                 job.add_output_library_dataset(output_name, dataset)
             else:
                 job.add_output_dataset(output_name, dataset)
-            # Create an empty file immediately
-            if not dataset.dataset.external_filename and trans.app.config.legacy_eager_objectstore_initialization:
-                dataset.dataset.object_store_id = object_store_id
-                try:
-                    trans.app.object_store.create(dataset.dataset)
-                except ObjectInvalid:
-                    raise Exception('Unable to create output dataset: object store is full')
-                object_store_id = dataset.dataset.object_store_id
 
-        trans.sa_session.add(output_object)
-
-    job.object_store_id = object_store_id
     job.set_state(job.states.NEW)
     if job_params:
         for name, value in job_params.items():
             job.add_parameter(name, value)
-    trans.sa_session.add(job)
 
-    # Queue the job for execution
-    trans.app.job_manager.enqueue(job, tool=tool)
-    trans.log_event("Added job to the job queue, id: %s" % str(job.id), tool_id=job.tool_id)
-    output = OrderedDict()
+    output = {}
     for i, v in enumerate(outputs):
         if not hasattr(output_object, "collection_type"):
             output['output%i' % i] = v
@@ -462,8 +469,8 @@ def active_folders(trans, folder):
     # Stolen from galaxy.web.controllers.library_common (importing from which causes a circular issues).
     # Much faster way of retrieving all active sub-folders within a given folder than the
     # performance of the mapper.  This query also eagerloads the permissions on each folder.
-    return trans.sa_session.query(trans.app.model.LibraryFolder) \
+    return trans.sa_session.query(LibraryFolder) \
                            .filter_by(parent=folder, deleted=False) \
-                           .options(eagerload_all("actions")) \
-                           .order_by(trans.app.model.LibraryFolder.table.c.name) \
+                           .options(joinedload("actions")) \
+                           .order_by(LibraryFolder.table.c.name) \
                            .all()
