@@ -19,14 +19,14 @@ def __main__():
         max_size = sys.maxsize
     maf_writer = bx.align.maf.Writer(open(output_filename1, 'w'))
     try:
-        maf_reader = bx.align.maf.Reader(open(input_maf_filename, 'r'))
+        maf_reader = bx.align.maf.Reader(open(input_maf_filename))
     except Exception:
         print("Your MAF file appears to be malformed.", file=sys.stderr)
         sys.exit()
 
     blocks_kept = 0
     i = 0
-    for i, m in enumerate(maf_reader):
+    for i, m in enumerate(maf_reader):  # noqa: B007
         if min_size <= m.text_size <= max_size:
             maf_writer.write(m)
             blocks_kept += 1
