@@ -7,16 +7,18 @@ import tempfile
 import zlib
 from collections import defaultdict
 from threading import Lock
+from typing import Dict, List, Tuple
 
-from sqlalchemy.orm import (
-    defer,
-    joinedload,
-)
+from sqlalchemy.orm import defer
 from sqlitedict import SqliteDict
 
+from galaxy.model.scoped_session import install_model_scoped_session
+from galaxy.model.tool_shed_install import ToolShedRepository
+from galaxy.tool_util.toolbox.base import ToolConfRepository
 from galaxy.util import unicodify
 from galaxy.util.hash_util import md5_hash_file
 
+
 log = logging.getLogger(__name__)
 
 CURRENT_TOOL_CACHE_VERSION = 0
@@ -38,24 +40,42 @@ class ToolDocumentCache:
             os.makedirs(self.cache_dir)
         self.cache_file = os.path.join(self.cache_dir, 'cache.sqlite')
         self.writeable_cache_file = None
-        # Create database if necessary using 'c' flag
-        self._cache = SqliteDict(self.cache_file, flag='c', encode=encoder, decode=decoder, autocommit=False)
-        # Switch SqliteDict back to readonly
-        self._cache.flag = 'r'
+        self._cache = None
+        self.disabled = False
+        self._get_cache(create_if_necessary=True)
 
     def close(self):
-        self._cache.close()
+        self._cache and self._cache.close()
+
+    def _get_cache(self, flag='r', create_if_necessary=False):
+        try:
+            if create_if_necessary and not os.path.exists(self.cache_file):
+                # Create database if necessary using 'c' flag
+                self._cache = SqliteDict(self.cache_file, flag='c', encode=encoder, decode=decoder, autocommit=False)
+                if flag == 'r':
+                    self._cache.flag = flag
+            else:
+                cache_file = self.writeable_cache_file.name if self.writeable_cache_file else self.cache_file
+                self._cache = SqliteDict(cache_file, flag=flag, encode=encoder, decode=decoder, autocommit=False)
+        except sqlite3.OperationalError:
+            log.warning('Tool document cache unavailable')
+            self._cache = None
+            self.disabled = True
 
     @property
     def cache_file_is_writeable(self):
         return os.access(self.cache_file, os.W_OK)
 
     def reopen_ro(self):
-        self._cache = SqliteDict(self.cache_file, flag='r', encode=encoder, decode=decoder, autocommit=False)
+        self._get_cache(flag='r')
         self.writeable_cache_file = None
 
     def get(self, config_file):
-        tool_document = self._cache.get(config_file)
+        try:
+            tool_document = self._cache.get(config_file)
+        except sqlite3.OperationalError:
+            log.debug("Tool document cache unavailable")
+            return None
         if not tool_document:
             return None
         if tool_document.get('tool_cache_version') != CURRENT_TOOL_CACHE_VERSION:
@@ -71,7 +91,7 @@ class ToolDocumentCache:
             self.writeable_cache_file = tempfile.NamedTemporaryFile(dir=self.cache_dir, suffix='cache.sqlite.tmp', delete=False)
             if os.path.exists(self.cache_file):
                 shutil.copy(self.cache_file, self.writeable_cache_file.name)
-            self._cache = SqliteDict(self.writeable_cache_file.name, flag='c', encode=encoder, decode=decoder, autocommit=False)
+            self._get_cache(flag='c')
 
     def persist(self):
         if self.writeable_cache_file:
@@ -80,22 +100,28 @@ class ToolDocumentCache:
             self.reopen_ro()
 
     def set(self, config_file, tool_source):
-        if self.cache_file_is_writeable:
-            self._make_writable()
-            to_persist = {
-                'document': tool_source.to_string(),
-                'macro_paths': tool_source.macro_paths,
-                'paths_and_modtimes': tool_source.paths_and_modtimes(),
-                'tool_cache_version': CURRENT_TOOL_CACHE_VERSION,
-            }
-            self._cache[config_file] = to_persist
+        try:
+            if self.cache_file_is_writeable:
+                self._make_writable()
+                to_persist = {
+                    'document': tool_source.to_string(),
+                    'macro_paths': tool_source.macro_paths,
+                    'paths_and_modtimes': tool_source.paths_and_modtimes(),
+                    'tool_cache_version': CURRENT_TOOL_CACHE_VERSION,
+                }
+                try:
+                    self._cache[config_file] = to_persist
+                except RuntimeError:
+                    log.debug("Tool document cache not writeable")
+        except sqlite3.OperationalError:
+            log.debug("Tool document cache unavailable")
 
     def delete(self, config_file):
         if self.cache_file_is_writeable:
             self._make_writable()
             try:
                 del self._cache[config_file]
-            except KeyError:
+            except (KeyError, RuntimeError):
                 pass
 
     def __del__(self):
@@ -159,29 +185,30 @@ class ToolCache:
                     if tool_id in self._new_tool_ids:
                         self._new_tool_ids.remove(tool_id)
                 if persist_tool_document_cache:
-                    tool.toolbox.persist_tool_document_cache()
+                    tool.app.toolbox.persist_cache()
         except Exception as e:
             log.debug("Exception while checking tools to remove from cache: %s", unicodify(e))
             # If by chance the file is being removed while calculating the hash or modtime
             # we don't want the thread to die.
         if removed_tool_ids:
-            log.debug("Removed the following tools from cache: %s" % removed_tool_ids)
+            log.debug(f"Removed the following tools from cache: {removed_tool_ids}")
         return removed_tool_ids
 
     def _should_cleanup(self, config_filename):
         """Return True if `config_filename` does not exist or if modtime and hash have changes, else return False."""
-        if not os.path.exists(config_filename):
+        try:
+            new_mtime = os.path.getmtime(config_filename)
+            tool_hash = self._hash_by_tool_paths.get(config_filename)
+            if tool_hash.modtime < new_mtime:
+                if md5_hash_file(config_filename) != tool_hash.hash:
+                    return True
+            tool = self._tools_by_path[config_filename]
+            for macro_path in tool._macro_paths:
+                new_mtime = os.path.getmtime(macro_path)
+                if self._hash_by_tool_paths.get(macro_path).modtime < new_mtime:
+                    return True
+        except FileNotFoundError:
             return True
-        new_mtime = os.path.getmtime(config_filename)
-        tool_hash = self._hash_by_tool_paths.get(config_filename)
-        if tool_hash.modtime < new_mtime:
-            if md5_hash_file(config_filename) != tool_hash.hash:
-                return True
-        tool = self._tools_by_path[config_filename]
-        for macro_path in tool._macro_paths:
-            new_mtime = os.path.getmtime(macro_path)
-            if self._hash_by_tool_paths.get(macro_path).modtime < new_mtime:
-                return True
         return False
 
     def get_tool(self, config_filename):
@@ -252,35 +279,32 @@ class ToolShedRepositoryCache:
     """
     Cache installed ToolShedRepository objects.
     """
+    local_repositories: List[ToolConfRepository]
+    repositories: List[ToolShedRepository]
+    repos_by_tuple: Dict[Tuple[str, str, str], List[ToolConfRepository]]
 
-    def __init__(self, app):
-        self.app = app
+    def __init__(self, session: install_model_scoped_session):
+        self.session = session()
         # Contains ToolConfRepository objects created from shed_tool_conf.xml entries
         self.local_repositories = []
         # Repositories loaded from database
         self.repositories = []
         self.repos_by_tuple = defaultdict(list)
-        self.rebuild()
+        self._build()
+        self.session.close()
 
     def add_local_repository(self, repository):
         self.local_repositories.append(repository)
         self.repos_by_tuple[(repository.tool_shed, repository.owner, repository.name)].append(repository)
 
-    def rebuild(self):
-        try:
-            session = self.app.install_model.context.current.session_factory()
-            self.repositories = session.query(self.app.install_model.ToolShedRepository).options(
-                defer(self.app.install_model.ToolShedRepository.metadata),
-                joinedload('tool_dependencies').subqueryload('tool_shed_repository').options(
-                    defer(self.app.install_model.ToolShedRepository.metadata)
-                ),
-            ).all()
-            repos_by_tuple = defaultdict(list)
-            for repository in self.repositories + self.local_repositories:
-                repos_by_tuple[(repository.tool_shed, repository.owner, repository.name)].append(repository)
-            self.repos_by_tuple = repos_by_tuple
-        finally:
-            session.close()
+    def _build(self):
+        self.repositories = self.session.query(ToolShedRepository).options(
+            defer(ToolShedRepository.metadata_)
+        ).all()
+        repos_by_tuple = defaultdict(list)
+        for repository in self.repositories + self.local_repositories:
+            repos_by_tuple[(repository.tool_shed, repository.owner, repository.name)].append(repository)
+        self.repos_by_tuple = repos_by_tuple
 
     def get_installed_repository(self, tool_shed=None, name=None, owner=None, installed_changeset_revision=None, changeset_revision=None, repository_id=None):
         if repository_id:
