@@ -8,22 +8,25 @@ to modify the tool configurations.
 
 import errno
 import hashlib
+import json
 import logging
 import os
 import os.path
 import re
 import string
 import time
-from collections import OrderedDict
 from glob import glob
 from tempfile import NamedTemporaryFile
+from typing import List
 
 import refgenconf
 import requests
 
 from galaxy import util
+from galaxy.exceptions import MessageException
 from galaxy.util import RW_R__R__
 from galaxy.util.dictifiable import Dictifiable
+from galaxy.util.filelock import FileLock
 from galaxy.util.renamed_temporary_file import RenamedTemporaryFile
 from galaxy.util.template import fill_template
 
@@ -66,7 +69,7 @@ class ToolDataPathFiles:
             return os.path.exists(path)
 
 
-class ToolDataTableManager:
+class ToolDataTableManager(Dictifiable):
     """Manages a collection of tool data tables"""
 
     def __init__(self, tool_data_path, config_filename=None, tool_data_table_config_path_set=None, other_config_dict=None):
@@ -103,6 +106,19 @@ class ToolDataTableManager:
     def get_tables(self):
         return self.data_tables
 
+    def to_dict(self):
+        return {name: data_table.to_dict(view='export') for name, data_table in self.data_tables.items()}
+
+    def to_json(self, path):
+        with open(path, 'w') as out:
+            out.write(json.dumps(self.to_dict()))
+
+    @classmethod
+    def from_dict(cls, d):
+        tdtm = cls.__new__(cls)
+        tdtm.data_tables = {name: ToolDataTable.from_dict(data) for name, data in d.items()}
+        return tdtm
+
     def load_from_config_file(self, config_filename, tool_data_path, from_shed_config=False):
         """
         This method is called under 3 conditions:
@@ -159,7 +175,7 @@ class ToolDataTableManager:
                                                      tool_data_path=tool_data_path,
                                                      from_shed_config=True)
         except Exception as e:
-            error_message = 'Error attempting to parse file {}: {}'.format(str(os.path.split(config_filename)[1]), util.unicodify(e))
+            error_message = f'Error attempting to parse file {str(os.path.split(config_filename)[1])}: {util.unicodify(e)}'
             log.debug(error_message, exc_info=True)
             table_elems = []
         if persist:
@@ -240,13 +256,24 @@ class ToolDataTableManager:
 
 
 class ToolDataTable:
+    type_key: str
 
     @classmethod
     def from_elem(cls, table_elem, tool_data_path, from_shed_config, filename, tool_data_path_files, other_config_dict=None):
         table_type = table_elem.get('type', 'tabular')
-        assert table_type in tool_data_table_types, "Unknown data table type '%s'" % table_type
+        assert table_type in tool_data_table_types, f"Unknown data table type '{table_type}'"
         return tool_data_table_types[table_type](table_elem, tool_data_path, from_shed_config=from_shed_config, filename=filename, tool_data_path_files=tool_data_path_files, other_config_dict=other_config_dict)
 
+    @classmethod
+    def from_dict(cls, d):
+        data_table_class = globals()[d['model_class']]
+        data_table = data_table_class.__new__(data_table_class)
+        for attr, val in d.items():
+            if not attr == 'model_class':
+                setattr(data_table, attr, val)
+        data_table._loaded_content_version = 1
+        return data_table
+
     def __init__(self, config_element, tool_data_path, from_shed_config=False, filename=None, tool_data_path_files=None, other_config_dict=None):
         self.name = config_element.get('name')
         self.comment_char = config_element.get('comment_char')
@@ -254,7 +281,7 @@ class ToolDataTable:
         self.empty_field_values = {}
         self.allow_duplicate_entries = util.asbool(config_element.get('allow_duplicate_entries', True))
         self.here = filename and os.path.dirname(filename)
-        self.filenames = OrderedDict()
+        self.filenames = {}
         self.tool_data_path = tool_data_path
         self.tool_data_path_files = tool_data_path_files
         self.other_config_dict = other_config_dict or {}
@@ -262,7 +289,7 @@ class ToolDataTable:
         # increment this variable any time a new entry is added, or when the table is totally reloaded
         # This value has no external meaning, and does not represent an abstract version of the underlying data
         self._loaded_content_version = 1
-        self._load_info = ([config_element, tool_data_path], {'from_shed_config': from_shed_config, 'tool_data_path_files': self.tool_data_path_files, 'other_config_dict': other_config_dict})
+        self._load_info = ([config_element, tool_data_path], {'from_shed_config': from_shed_config, 'tool_data_path_files': self.tool_data_path_files, 'other_config_dict': other_config_dict, 'filename': filename})
         self._merged_load_info = []
 
     def _update_version(self, version=None):
@@ -275,16 +302,19 @@ class ToolDataTable:
     def get_empty_field_by_name(self, name):
         return self.empty_field_values.get(name, self.empty_field_value)
 
-    def _add_entry(self, entry, allow_duplicates=True, persist=False, persist_on_error=False, entry_source=None, **kwd):
+    def _add_entry(self, entry, allow_duplicates=True, persist=False, entry_source=None, **kwd):
         raise NotImplementedError("Abstract method")
 
-    def add_entry(self, entry, allow_duplicates=True, persist=False, persist_on_error=False, entry_source=None, **kwd):
-        self._add_entry(entry, allow_duplicates=allow_duplicates, persist=persist, persist_on_error=persist_on_error, entry_source=entry_source, **kwd)
+    def add_entry(self, entry, allow_duplicates=True, persist=False, entry_source=None, **kwd):
+        self._add_entry(entry, allow_duplicates=allow_duplicates, persist=persist, entry_source=entry_source, **kwd)
         return self._update_version()
 
-    def add_entries(self, entries, allow_duplicates=True, persist=False, persist_on_error=False, entry_source=None, **kwd):
+    def add_entries(self, entries, allow_duplicates=True, persist=False, entry_source=None, **kwd):
         for entry in entries:
-            self.add_entry(entry, allow_duplicates=allow_duplicates, persist=persist, persist_on_error=persist_on_error, entry_source=entry_source, **kwd)
+            try:
+                self.add_entry(entry, allow_duplicates=allow_duplicates, persist=persist, entry_source=entry_source, **kwd)
+            except Exception as e:
+                log.error(str(e))
         return self._loaded_content_version
 
     def _remove_entry(self, values):
@@ -297,7 +327,7 @@ class ToolDataTable:
     def is_current_version(self, other_version):
         return self._loaded_content_version == other_version
 
-    def merge_tool_data_table(self, other_table, allow_duplicates=True, persist=False, persist_on_error=False, entry_source=None, **kwd):
+    def merge_tool_data_table(self, other_table, allow_duplicates=True, persist=False, entry_source=None, **kwd):
         raise NotImplementedError("Abstract method")
 
     def reload_from_files(self):
@@ -313,7 +343,9 @@ class ToolDataTable:
 class TabularToolDataTable(ToolDataTable, Dictifiable):
     """
     Data stored in a tabular / separated value format on disk, allows multiple
-    files to be merged but all must have the same column definitions::
+    files to be merged but all must have the same column definitions:
+
+    .. code-block:: xml
 
         <table type="tabular" name="test">
             <column name='...' index = '...' />
@@ -323,6 +355,8 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
 
     """
     dict_collection_visible_keys = ['name']
+    dict_element_visible_keys = ['name', 'fields']
+    dict_export_visible_keys = ['name', 'data', 'largest_index', 'columns', 'missing_index_file']
 
     type_key = 'tabular'
 
@@ -356,7 +390,7 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
                 # Handle URLs as files
                 filename = file_element.get('url', None)
                 if filename:
-                    tmp_file = NamedTemporaryFile(prefix='TTDT_URL_%s-' % self.name, mode='w')
+                    tmp_file = NamedTemporaryFile(prefix=f'TTDT_URL_{self.name}-', mode='w')
                     try:
                         tmp_file.write(requests.get(filename, timeout=url_timeout).text)
                     except Exception as e:
@@ -391,7 +425,7 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
                 filename = os.path.join(tool_data_path, filename)
             if self.tool_data_path_files.exists(filename):
                 found = True
-            else:
+            elif not os.path.isabs(filename):
                 # Since the path attribute can include a hard-coded path to a specific directory
                 # (e.g., <file path="tool-data/cg_crr_files.loc" />) which may not be the same value
                 # as self.tool_data_path, we'll parse the path to get the filename and see if it is
@@ -402,9 +436,9 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
                     if self.tool_data_path_files.exists(corrected_filename):
                         filename = corrected_filename
                         found = True
-                    elif not from_shed_config and self.tool_data_path_files.exists("%s.sample" % corrected_filename):
-                        log.info("Could not find tool data %s, reading sample" % corrected_filename)
-                        filename = "%s.sample" % corrected_filename
+                    elif not from_shed_config and self.tool_data_path_files.exists(f"{corrected_filename}.sample"):
+                        log.info(f"Could not find tool data {corrected_filename}, reading sample")
+                        filename = f"{corrected_filename}.sample"
                         found = True
 
             errors = []
@@ -419,7 +453,7 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
                 # warnings about missing location files that would otherwise be
                 # empty and we don't care about unless the admin chooses to
                 # populate them.
-                log.warning("Cannot find index file '{}' for tool data table '{}'".format(filename, self.name))
+                log.warning(f"Cannot find index file '{filename}' for tool data table '{self.name}'")
 
             if filename not in self.filenames or not self.filenames[filename]['found']:
                 self.filenames[filename] = dict(found=found, filename=filename, from_shed_config=from_shed_config, tool_data_path=tool_data_path,
@@ -430,8 +464,8 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
             if tmp_file is not None:
                 tmp_file.close()
 
-    def merge_tool_data_table(self, other_table, allow_duplicates=True, persist=False, persist_on_error=False, entry_source=None, **kwd):
-        assert self.columns == other_table.columns, "Merging tabular data tables with non matching columns is not allowed: {}:{} != {}:{}".format(self.name, self.columns, other_table.name, other_table.columns)
+    def merge_tool_data_table(self, other_table, allow_duplicates=True, persist=False, entry_source=None, **kwd):
+        assert self.columns == other_table.columns, f"Merging tabular data tables with non matching columns is not allowed: {self.name}:{self.columns} != {other_table.name}:{other_table.columns}"
         # merge filename info
         for filename, info in other_table.filenames.items():
             if filename not in self.filenames:
@@ -444,7 +478,7 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
             self.allow_duplicate_entries = False
             self._deduplicate_data()
         # add data entries and return current data table version
-        return self.add_entries(other_table.data, allow_duplicates=allow_duplicates, persist=persist, persist_on_error=persist_on_error, entry_source=entry_source, **kwd)
+        return self.add_entries(other_table.data, allow_duplicates=allow_duplicates, persist=persist, entry_source=entry_source, **kwd)
 
     def handle_found_index_file(self, filename):
         self.missing_index_file = None
@@ -612,7 +646,7 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
                 break
         return filename
 
-    def _add_entry(self, entry, allow_duplicates=True, persist=False, persist_on_error=False, entry_source=None, **kwd):
+    def _add_entry(self, entry, allow_duplicates=True, persist=False, entry_source=None, **kwd):
         # accepts dict or list of columns
         if isinstance(entry, dict):
             fields = []
@@ -625,42 +659,40 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
                 fields.append(field_value)
         else:
             fields = entry
-        is_error = False
         if self.largest_index < len(fields):
             fields = self._replace_field_separators(fields)
             if (allow_duplicates and self.allow_duplicate_entries) or fields not in self.get_fields():
                 self.data.append(fields)
             else:
-                log.debug("Attempted to add fields (%s) to data table '%s', but this entry already exists and allow_duplicates is False.", fields, self.name)
-                is_error = True
+                raise MessageException(f"Attempted to add fields ({fields}) to data table '{self.name}', but this entry already exists and allow_duplicates is False.")
         else:
-            log.error("Attempted to add fields (%s) to data table '%s', but there were not enough fields specified ( %i < %i ).", fields, self.name, len(fields), self.largest_index + 1)
-            is_error = True
+            raise MessageException(f"Attempted to add fields ({fields}) to data table '{self.name}', but there were not enough fields specified ( {len(fields)} < {self.largest_index + 1} ).")
         filename = None
 
-        if persist and (not is_error or persist_on_error):
+        if persist:
             filename = self.get_filename_for_source(entry_source)
             if filename is None:
-                # should we default to using any filename here instead?
-                log.error("Unable to determine filename for persisting data table '%s' values: '%s'.", self.name, fields)
-                is_error = True
+                # If we reach this point, there is no data table with a corresponding .loc file.
+                raise MessageException(f"Unable to determine filename for persisting data table '{self.name}' values: '{self.fields}'.")
             else:
-                # FIXME: Need to lock these files for editing
                 log.debug("Persisting changes to file: %s", filename)
-                try:
-                    data_table_fh = open(filename, 'r+b')
-                except OSError as e:
-                    log.warning('Error opening data table file (%s) with r+b, assuming file does not exist and will open as wb: %s', filename, e)
-                    data_table_fh = open(filename, 'wb')
-                if os.stat(filename).st_size != 0:
-                    # ensure last existing line ends with new line
-                    data_table_fh.seek(-1, 2)  # last char in file
-                    last_char = data_table_fh.read(1)
-                    if last_char not in [b'\n', b'\r']:
-                        data_table_fh.write(b'\n')
-                fields = "%s\n" % self.separator.join(fields)
+                with FileLock(filename):
+                    try:
+                        if os.path.exists(filename):
+                            data_table_fh = open(filename, 'r+b')
+                            if os.stat(filename).st_size > 0:
+                                # ensure last existing line ends with new line
+                                data_table_fh.seek(-1, 2)  # last char in file
+                                last_char = data_table_fh.read(1)
+                                if last_char not in [b'\n', b'\r']:
+                                    data_table_fh.write(b'\n')
+                        else:
+                            data_table_fh = open(filename, 'wb')
+                    except OSError as e:
+                        log.exception('Error opening data table file (%s): %s', filename, e)
+                        raise
+                fields = f"{self.separator.join(fields)}\n"
                 data_table_fh.write(fields.encode('utf-8'))
-        return not is_error
 
     def _remove_entry(self, values):
 
@@ -671,7 +703,7 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
                 values = self._replace_field_separators(values)
                 self.filter_file_fields(filename, values)
             else:
-                log.warning("Cannot find index file '{}' for tool data table '{}'".format(filename, self.name))
+                log.warning(f"Cannot find index file '{filename}' for tool data table '{self.name}'")
 
         self.reload_from_files()
 
@@ -734,7 +766,7 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
         return util.xml_to_string(self.config_element)
 
     def to_dict(self, view='collection'):
-        rval = super().to_dict()
+        rval = super().to_dict(view=view)
         if view == 'element':
             rval['columns'] = sorted(self.columns.keys(), key=lambda x: self.columns[x])
             rval['fields'] = self.get_fields()
@@ -743,7 +775,7 @@ class TabularToolDataTable(ToolDataTable, Dictifiable):
 
 class TabularToolDataField(Dictifiable):
 
-    dict_collection_visible_keys = []
+    dict_collection_visible_keys: List[str] = []
 
     def __init__(self, data):
         self.data = data
@@ -761,10 +793,10 @@ class TabularToolDataField(Dictifiable):
         return path
 
     def clean_base_dir(self, path):
-        return re.sub("^" + self.get_base_dir() + r"/*", "", path)
+        return re.sub(f"^{self.get_base_dir()}/*", "", path)
 
     def get_files(self):
-        return glob(self.get_base_path() + "*")
+        return glob(f"{self.get_base_path()}*")
 
     def get_filesize_map(self, rm_base_dir=False):
         out = {}
@@ -796,15 +828,20 @@ class TabularToolDataField(Dictifiable):
 class RefgenieToolDataTable(TabularToolDataTable):
     """
     Data stored in refgenie
-    <table name="all_fasta" type="refgenie" asset="fasta" >
-        <file path="refgenie.yml" />
-        <field name="value" template="true">${__REFGENIE_UUID__}</field>
-        <field name="dbkey" template="true">${__REFGENIE_GENOME__}</field>
-        <field name="name" template="true">${__REFGENIE_DISPLAY_NAME__}</field>
-        <field name="path" template="true">${__REFGENIE_ASSET__}</field>
-    </table>
+
+    .. code-block:: xml
+
+        <table name="all_fasta" type="refgenie" asset="fasta" >
+            <file path="refgenie.yml" />
+            <field name="value" template="true">${__REFGENIE_UUID__}</field>
+            <field name="dbkey" template="true">${__REFGENIE_GENOME__}</field>
+            <field name="name" template="true">${__REFGENIE_DISPLAY_NAME__}</field>
+            <field name="path" template="true">${__REFGENIE_ASSET__}</field>
+        </table>
     """
     dict_collection_visible_keys = ['name']
+    dict_element_visible_keys = ['name', 'fields']
+    dict_export_visible_keys = ['name', 'data', 'rg_asset', 'largest_index', 'columns', 'missing_index_file']
 
     type_key = 'refgenie'
 
@@ -845,19 +882,28 @@ class RefgenieToolDataTable(TabularToolDataTable):
             self.columns['name'] = self.columns['value']
 
     def parse_file_fields(self, filename, errors=None, here="__HERE__"):
-        rgc = refgenconf.RefGenConf(filename)
+        try:
+            rgc = refgenconf.RefGenConf(filename, writable=False, skip_read_lock=True)
+        except refgenconf.exceptions.RefgenconfError as e:
+            log.error('Unable to load refgenie config file "%s": %s', filename, e)
+            if errors is not None:
+                errors.append(e)
+            return []
         rval = []
         for genome in rgc.list_genomes_by_asset(self.rg_asset):
             genome_attributes = rgc.get_genome_attributes(genome)
-            description = genome_attributes.get('description', None)
+            genome_description = genome_attributes.get('genome_description', None)
             asset_list = rgc.list(genome, include_tags=True)[genome]
             for tagged_asset in asset_list:
                 asset, tag = tagged_asset.rsplit(':', 1)
                 if asset != self.rg_asset:
                     continue
                 digest = rgc.id(genome, asset, tag=tag)
-                uuid = 'refgenie:{}/{}:{}@{}'.format(genome, self.rg_asset, tag, digest)
-                display_name = description or '{}/{}'.format(genome, tagged_asset)
+                uuid = f'refgenie:{genome}/{self.rg_asset}:{tag}@{digest}'
+                if genome_description:
+                    display_name = f'{genome_description} (refgenie: {genome}@{digest})'
+                else:
+                    display_name = f'{genome}/{tagged_asset}@{digest}'
 
                 def _seek_key(key):
                     return rgc.seek(genome, asset, tag_name=tag, seek_key=key)
@@ -887,7 +933,11 @@ class RefgenieToolDataTable(TabularToolDataTable):
         return rval
 
     def _remove_entry(self, values):
-        raise NotImplementedError("Not supported")
+
+        log.warning("Deletion from refgenie-backed '%s' data table is not supported, will only try to delete from .loc files", self.name)
+
+        # Update every non-refgenie files
+        super()._remove_entry(values)
 
 
 def expand_here_template(content, here=None):
