@@ -5,9 +5,10 @@ import os
 import shutil
 import sys
 import tempfile
+from io import StringIO
+from typing import Any, Dict, List, Tuple
 
 import bdbag.bdbag_api
-from six.moves import StringIO
 
 from galaxy.datatypes import sniff
 from galaxy.datatypes.registry import Registry
@@ -36,9 +37,12 @@ def main(argv=None):
     with open(request_path) as f:
         request = json.load(f)
 
-    upload_config = UploadConfig(request, registry)
+    working_directory = args.working_directory or os.getcwd()
+    allow_failed_collections = request.get("allow_failed_collections", False)
+    upload_config = UploadConfig(request, registry, working_directory, allow_failed_collections)
     galaxy_json = _request_to_galaxy_json(upload_config, request)
-    with open("galaxy.json", "w") as f:
+    galaxy_json_path = os.path.join(working_directory, "galaxy.json")
+    with open(galaxy_json_path, "w") as f:
         json.dump(galaxy_json, f)
 
 
@@ -74,18 +78,30 @@ def _fetch_target(upload_config, target):
                 _, elements_from_path = _has_src_to_path(upload_config, target_or_item, is_dataset=False)
                 items = _directory_to_items(elements_from_path)
             else:
-                raise Exception("Unknown elements from type encountered [%s]" % elements_from)
+                raise Exception(f"Unknown elements from type encountered [{elements_from}]")
 
         if items:
             del target_or_item["elements_from"]
             target_or_item["elements"] = items
 
-    _for_each_src(expand_elements_from, target)
-    items = target.get("elements", None)
-    assert items is not None, "No element definition found for destination [%s]" % destination
+    expansion_error = None
+    try:
+        _for_each_src(expand_elements_from, target)
+    except Exception as e:
+        expansion_error = f"Error expanding elements/items for upload destination. {str(e)}"
+
+    if expansion_error is None:
+        items = target.get("elements", None)
+        assert items is not None, f"No element definition found for destination [{destination}]"
+    else:
+        items = []
 
     fetched_target = {}
     fetched_target["destination"] = destination
+    destination_type = destination["type"]
+    is_collection = destination_type == "hdca"
+    failed_elements = []
+
     if "collection_type" in target:
         fetched_target["collection_type"] = target["collection_type"]
     if "name" in target:
@@ -105,6 +121,8 @@ def _fetch_target(upload_config, target):
             target_metadata["tags"] = tags
         if created_from_basename:
             target_metadata["created_from_basename"] = created_from_basename
+        if "error_message" in src_item:
+            target_metadata["error_message"] = src_item["error_message"]
         return target_metadata
 
     def _resolve_item(item):
@@ -115,19 +133,33 @@ def _fetch_target(upload_config, target):
         composite = item.pop("composite", None)
         if datatype and datatype.composite_type:
             composite_type = datatype.composite_type
-            writable_files = datatype.writable_files
             assert composite_type == "auto_primary_file", "basic composite uploads not yet implemented"
 
             # get_composite_dataset_name finds dataset name from basename of contents
             # and such but we're not implementing that here yet. yagni?
             # also need name...
-            dataset_bunch = Bunch()
-            name = item.get("name") or 'Composite Dataset'
-            dataset_bunch.name = name
-            primary_file = sniff.stream_to_file(StringIO(datatype.generate_primary_file(dataset_bunch)), prefix='upload_auto_primary_file', dir=".")
-            extra_files_path = primary_file + "_extra"
+            metadata = {
+                composite_file.substitute_name_with_metadata: datatype.metadata_spec[
+                    composite_file.substitute_name_with_metadata
+                ].default
+                for composite_file in datatype.composite_files.values()
+                if composite_file.substitute_name_with_metadata
+            }
+            name = item.get("name") or "Composite Dataset"
+            metadata["base_name"] = name
+            dataset = Bunch(
+                name=name,
+                metadata=metadata,
+            )
+            writable_files = datatype.get_writable_files_for_dataset(dataset)
+            primary_file = sniff.stream_to_file(
+                StringIO(datatype.generate_primary_file(dataset)),
+                prefix="upload_auto_primary_file",
+                dir=".",
+            )
+            extra_files_path = f"{primary_file}_extra"
             os.mkdir(extra_files_path)
-            rval = {
+            rval: Dict[str, Any] = {
                 "name": name,
                 "filename": primary_file,
                 "ext": requested_ext,
@@ -138,7 +170,7 @@ def _fetch_target(upload_config, target):
             }
             _copy_and_validate_simple_attributes(item, rval)
             composite_items = composite.get("elements", [])
-            keys = [value.name for value in writable_files.values()]
+            keys = list(writable_files.keys())
             composite_item_idx = 0
             for composite_item in composite_items:
                 if composite_item_idx >= len(keys):
@@ -155,7 +187,7 @@ def _fetch_target(upload_config, target):
                     key,
                     writable_file.is_binary,
                     ".",
-                    os.path.basename(extra_files_path) + "_",
+                    f"{os.path.basename(extra_files_path)}_",
                     composite_item,
                 )
                 composite_item_idx += 1
@@ -171,112 +203,155 @@ def _fetch_target(upload_config, target):
             return rval
         else:
             if composite:
-                raise Exception("Non-composite datatype [%s] attempting to be created with composite data." % datatype)
+                raise Exception(f"Non-composite datatype [{datatype}] attempting to be created with composite data.")
             return _resolve_item_with_primary(item)
 
     def _resolve_item_with_primary(item):
+        error_message = None
         converted_path = None
 
         name, path = _has_src_to_path(upload_config, item, is_dataset=True)
         sources = []
 
         url = item.get("url")
+        source_dict = {"source_uri": url}
         if url:
-            sources.append({"source_uri": url})
+            sources.append(source_dict)
         hashes = item.get("hashes", [])
         for hash_dict in hashes:
             hash_function = hash_dict.get("hash_function")
             hash_value = hash_dict.get("hash_value")
-            _handle_hash_validation(upload_config, hash_function, hash_value, path)
+            try:
+                _handle_hash_validation(upload_config, hash_function, hash_value, path)
+            except Exception as e:
+                error_message = str(e)
+                item["error_message"] = error_message
 
         dbkey = item.get("dbkey", "?")
-        requested_ext = item.get("ext", "auto")
         link_data_only = upload_config.link_data_only
         if "link_data_only" in item:
             # Allow overriding this on a per file basis.
             link_data_only = _link_data_only(item)
-        to_posix_lines = upload_config.get_option(item, "to_posix_lines")
-        space_to_tab = upload_config.get_option(item, "space_to_tab")
-        auto_decompress = upload_config.get_option(item, "auto_decompress")
-        in_place = item.get("in_place", False)
-        purge_source = item.get("purge_source", True)
 
-        registry = upload_config.registry
-        check_content = upload_config.check_content
-
-        stdout, ext, datatype, is_binary, converted_path = handle_upload(
-            registry=registry,
-            path=path,
-            requested_ext=requested_ext,
-            name=name,
-            tmp_prefix='data_fetch_upload_',
-            tmp_dir=".",
-            check_content=check_content,
-            link_data_only=link_data_only,
-            in_place=in_place,
-            auto_decompress=auto_decompress,
-            convert_to_posix_lines=to_posix_lines,
-            convert_spaces_to_tabs=space_to_tab,
-        )
-
-        if link_data_only:
-            # Never alter a file that will not be copied to Galaxy's local file store.
-            if datatype.dataset_content_needs_grooming(path):
-                err_msg = 'The uploaded files need grooming, so change your <b>Copy data into Galaxy?</b> selection to be ' + \
-                    '<b>Copy files into Galaxy</b> instead of <b>Link to files without copying into Galaxy</b> so grooming can be performed.'
-                raise UploadProblemException(err_msg)
-
-        # If this file is not in the workdir make sure it gets there.
-        if not link_data_only and converted_path:
-            path = upload_config.ensure_in_working_directory(converted_path, purge_source, in_place)
-        elif not link_data_only:
-            path = upload_config.ensure_in_working_directory(path, purge_source, in_place)
-
-        extra_files = item.get("extra_files")
+        ext = "data"
         staged_extra_files = None
-        if extra_files:
-            # TODO: optimize to just copy the whole directory to extra files instead.
-            assert not upload_config.link_data_only, "linking composite dataset files not yet implemented"
-            extra_files_path = path + "_extra"
-            staged_extra_files = extra_files_path
-            os.mkdir(extra_files_path)
+        if not error_message:
+            requested_ext = item.get("ext", "auto")
+            to_posix_lines = upload_config.get_option(item, "to_posix_lines")
+            space_to_tab = upload_config.get_option(item, "space_to_tab")
+            auto_decompress = upload_config.get_option(item, "auto_decompress")
+            in_place = item.get("in_place", False)
+            purge_source = item.get("purge_source", True)
 
-            def walk_extra_files(items, prefix=""):
-                for item in items:
-                    if "elements" in item:
-                        name = item.get("name")
-                        if not prefix:
-                            item_prefix = name
+            registry = upload_config.registry
+            check_content = upload_config.check_content
+
+            stdout, ext, datatype, is_binary, converted_path, converted_newlines, converted_spaces = handle_upload(
+                registry=registry,
+                path=path,
+                requested_ext=requested_ext,
+                name=name,
+                tmp_prefix='data_fetch_upload_',
+                tmp_dir=".",
+                check_content=check_content,
+                link_data_only=link_data_only,
+                in_place=in_place,
+                auto_decompress=auto_decompress,
+                convert_to_posix_lines=to_posix_lines,
+                convert_spaces_to_tabs=space_to_tab,
+            )
+            transform = []
+            if converted_newlines:
+                transform.append({"action": "to_posix_lines"})
+            if converted_spaces:
+                transform.append({"action": "spaces_to_tabs"})
+            if link_data_only:
+                # Never alter a file that will not be copied to Galaxy's local file store.
+                if datatype.dataset_content_needs_grooming(path):
+                    err_msg = 'The uploaded files need grooming, so change your <b>Copy data into Galaxy?</b> selection to be ' + \
+                        '<b>Copy files into Galaxy</b> instead of <b>Link to files without copying into Galaxy</b> so grooming can be performed.'
+                    raise UploadProblemException(err_msg)
+
+            # If this file is not in the workdir make sure it gets there.
+            if not link_data_only and converted_path:
+                path = upload_config.ensure_in_working_directory(converted_path, purge_source, in_place)
+            elif not link_data_only:
+                path = upload_config.ensure_in_working_directory(path, purge_source, in_place)
+
+            extra_files = item.get("extra_files")
+            if extra_files:
+                # TODO: optimize to just copy the whole directory to extra files instead.
+                assert not upload_config.link_data_only, "linking composite dataset files not yet implemented"
+                extra_files_path = f"{path}_extra"
+                staged_extra_files = extra_files_path
+                os.mkdir(extra_files_path)
+
+                def walk_extra_files(items, prefix=""):
+                    for item in items:
+                        if "elements" in item:
+                            name = item.get("name")
+                            if not prefix:
+                                item_prefix = name
+                            else:
+                                item_prefix = os.path.join(prefix, name)
+                            walk_extra_files(item.get("elements"), prefix=item_prefix)
                         else:
-                            item_prefix = os.path.join(prefix, name)
-                        walk_extra_files(item.get("elements"), prefix=item_prefix)
-                    else:
-                        name, src_path = _has_src_to_path(upload_config, item)
-                        if prefix:
-                            rel_path = os.path.join(prefix, name)
-                        else:
-                            rel_path = name
+                            src_name, src_path = _has_src_to_path(upload_config, item)
+                            if prefix:
+                                rel_path = os.path.join(prefix, src_name)
+                            else:
+                                rel_path = src_name
 
-                        file_output_path = os.path.join(staged_extra_files, rel_path)
-                        parent_dir = os.path.dirname(file_output_path)
-                        if not os.path.exists(parent_dir):
-                            safe_makedirs(parent_dir)
-                        shutil.move(src_path, file_output_path)
-            walk_extra_files(extra_files.get("elements", []))
+                            file_output_path = os.path.join(extra_files_path, rel_path)
+                            parent_dir = os.path.dirname(file_output_path)
+                            if not os.path.exists(parent_dir):
+                                safe_makedirs(parent_dir)
+                            shutil.move(src_path, file_output_path)
+                walk_extra_files(extra_files.get("elements", []))
 
-        # TODO:
-        # in galaxy json add 'extra_files' and point at target derived from extra_files:
-        if not link_data_only and datatype and datatype.dataset_content_needs_grooming(path):
-            # Groom the dataset content if necessary
-            datatype.groom_dataset_content(path)
+            # TODO:
+            # in galaxy json add 'extra_files' and point at target derived from extra_files:
 
-        rval = {"name": name, "filename": path, "dbkey": dbkey, "ext": ext, "link_data_only": link_data_only, "sources": sources, "hashes": hashes}
+            needs_grooming = not link_data_only and datatype and datatype.dataset_content_needs_grooming(path)
+            if needs_grooming:
+                # Groom the dataset content if necessary
+                transform.append({
+                    "action": "datatype_groom",
+                    "datatype_ext": ext,
+                    "datatype_class": datatype.__class__.__name__
+                })
+                datatype.groom_dataset_content(path)
+
+            if len(transform) > 0:
+                source_dict["transform"] = transform
+
+        rval = {"name": name, "filename": path, "dbkey": dbkey, "ext": ext, "link_data_only": link_data_only, "sources": sources, "hashes": hashes, "info": f"uploaded {ext} file"}
         if staged_extra_files:
             rval["extra_files"] = os.path.abspath(staged_extra_files)
         return _copy_and_validate_simple_attributes(item, rval)
 
-    elements = elements_tree_map(_resolve_item, items)
-    fetched_target["elements"] = elements
+    def _resolve_item_capture_error(item):
+        try:
+            return _resolve_item(item)
+        except Exception as e:
+            rval = {"error_message": str(e)}
+            rval = _copy_and_validate_simple_attributes(item, rval)
+            failed_elements.append(rval)
+            return rval
+
+    if expansion_error is None:
+        elements = elements_tree_map(_resolve_item_capture_error, items)
+        if is_collection and not upload_config.allow_failed_collections and len(failed_elements) > 0:
+            element_error = "Failed to fetch collection element(s):\n"
+            for failed_element in failed_elements:
+                element_error += f"\n- {failed_element['error_message']}"
+            fetched_target["error_message"] = element_error
+            fetched_target["elements"] = None
+        else:
+            fetched_target["elements"] = elements
+    else:
+        fetched_target["elements"] = []
+        fetched_target["error_message"] = expansion_error
     return fetched_target
 
 
@@ -313,8 +388,8 @@ def elements_tree_map(f, items):
 
 
 def _directory_to_items(directory):
-    items = []
-    dir_elements = {}
+    items: List[Dict[str, Any]] = []
+    dir_elements: Dict[str, Any] = {}
     for root, dirs, files in os.walk(directory):
         if root in dir_elements:
             target = dir_elements[root]
@@ -330,13 +405,17 @@ def _directory_to_items(directory):
     return items
 
 
-def _has_src_to_path(upload_config, item, is_dataset=False):
+def _has_src_to_path(upload_config, item, is_dataset=False) -> Tuple[str, str]:
     assert "src" in item, item
     src = item.get("src")
     name = item.get("name")
     if src == "url":
         url = item.get("url")
-        path = sniff.stream_url_to_file(url, file_sources=get_file_sources())
+        try:
+            path = sniff.stream_url_to_file(url, file_sources=get_file_sources(upload_config.working_directory))
+        except Exception as e:
+            raise Exception(f"Failed to fetch url {url}. {str(e)}")
+
         if not is_dataset:
             # Actual target dataset will validate and put results in dict
             # that gets passed back to Galaxy.
@@ -362,7 +441,7 @@ def _handle_hash_validation(upload_config, hash_function, hash_value, path):
     if upload_config.validate_hashes:
         calculated_hash_value = memory_bound_hexdigest(hash_func_name=hash_function, path=path)
         if calculated_hash_value != hash_value:
-            raise Exception("Failed to validate upload with [{}] - expected [{}] got [{}]".format(hash_function, hash_value, calculated_hash_value))
+            raise Exception(f"Failed to validate upload with [{hash_function}] - expected [{hash_value}] got [{calculated_hash_value}]")
 
 
 def _arg_parser():
@@ -371,34 +450,38 @@ def _arg_parser():
     parser.add_argument("--datatypes-registry")
     parser.add_argument("--request-version")
     parser.add_argument("--request")
+    parser.add_argument("--working-directory")
     return parser
 
 
 _file_sources = None
 
 
-def get_file_sources():
+def get_file_sources(working_directory):
     global _file_sources
     if _file_sources is None:
         from galaxy.files import ConfiguredFileSources
         file_sources = None
-        if os.path.exists("file_sources.json"):
+        file_sources_path = os.path.join(working_directory, "file_sources.json")
+        if os.path.exists(file_sources_path):
             file_sources_as_dict = None
-            with open("file_sources.json", "r") as f:
+            with open(file_sources_path) as f:
                 file_sources_as_dict = json.load(f)
             if file_sources_as_dict is not None:
                 file_sources = ConfiguredFileSources.from_dict(file_sources_as_dict)
         if file_sources is None:
-            ConfiguredFileSources.from_dict([])
+            ConfiguredFileSources.from_dict(None)
         _file_sources = file_sources
     return _file_sources
 
 
 class UploadConfig:
 
-    def __init__(self, request, registry):
+    def __init__(self, request, registry, working_directory, allow_failed_collections):
         self.registry = registry
-        self.check_content = request.get("check_content" , True)
+        self.working_directory = working_directory
+        self.allow_failed_collections = allow_failed_collections
+        self.check_content = request.get("check_content", True)
         self.to_posix_lines = request.get("to_posix_lines", False)
         self.space_to_tab = request.get("space_to_tab", False)
         self.auto_decompress = request.get("auto_decompress", False)
@@ -432,6 +515,11 @@ class UploadConfig:
         if purge_source:
             try:
                 shutil.move(path, new_path)
+                # Drop .info file if it exists
+                try:
+                    os.remove(f"{path}.info")
+                except FileNotFoundError:
+                    pass
             except OSError as e:
                 # We may not have permission to remove converted_path
                 if e.errno != errno.EACCES:
@@ -457,7 +545,7 @@ def _for_each_src(f, obj):
     if isinstance(obj, dict):
         if "src" in obj:
             f(obj)
-        for key, value in obj.items():
+        for value in obj.values():
             _for_each_src(f, value)
 
 
