@@ -2,10 +2,10 @@ import errno
 import json
 import logging
 import os
-from collections import OrderedDict
-
+from typing import Dict
 
 from galaxy import util
+from galaxy.structured_app import MinimalManagerApp
 from galaxy.tools.data import TabularToolDataTable
 from galaxy.util.template import fill_template
 
@@ -17,10 +17,13 @@ DEFAULT_VALUE_TRANSLATION_TYPE = 'template'
 
 
 class DataManagers:
-    def __init__(self, app, xml_filename=None):
+    data_managers: Dict[str, 'DataManager']
+    managed_data_tables: Dict[str, 'DataManager']
+
+    def __init__(self, app: MinimalManagerApp, xml_filename=None):
         self.app = app
-        self.data_managers = OrderedDict()
-        self.managed_data_tables = OrderedDict()
+        self.data_managers = {}
+        self.managed_data_tables = {}
         self.tool_path = None
         self._reload_count = 0
         self.filename = xml_filename or self.app.config.data_manager_config_file
@@ -43,11 +46,11 @@ class DataManagers:
                 raise
             return  # default config option and it doesn't exist, which is fine
         except Exception as e:
-            log.error('There was an error parsing your Data Manager config file "{}": {}'.format(xml_filename, e))
+            log.error(f'There was an error parsing your Data Manager config file "{xml_filename}": {e}')
             return  # we are not able to load any data managers
         root = tree.getroot()
         if root.tag != 'data_managers':
-            log.error('A data managers configuration must have a "data_managers" tag as the root. "%s" is present' % (root.tag))
+            log.error(f'A data managers configuration must have a "data_managers" tag as the root. "{root.tag}" is present')
             return
         if store_tool_path:
             tool_path = root.get('tool_path', None)
@@ -75,12 +78,12 @@ class DataManagers:
             return None
         if add_manager:
             self.add_manager(data_manager)
-        log.debug('Loaded Data Manager: %s' % (data_manager.id))
+        log.debug(f'Loaded Data Manager: {data_manager.id}')
         return data_manager
 
     def add_manager(self, data_manager):
         if data_manager.id in self.data_managers:
-            log.warning("A data manager has been defined twice: %s " % (data_manager.id))
+            log.warning(f"A data manager has been defined twice: {data_manager.id} ")
         self.data_managers[data_manager.id] = data_manager
         for data_table_name in data_manager.data_tables.keys():
             if data_table_name not in self.managed_data_tables:
@@ -123,7 +126,7 @@ class DataManager:
         self.version = self.DEFAULT_VERSION
         self.guid = None
         self.tool = None
-        self.data_tables = OrderedDict()
+        self.data_tables = {}
         self.output_ref_by_data_table = {}
         self.move_by_data_table_column = {}
         self.value_translation_by_data_table_column = {}
@@ -133,17 +136,16 @@ class DataManager:
             self.load_from_element(elem, tool_path or self.data_managers.tool_path)
 
     def load_from_element(self, elem, tool_path):
-        assert elem.tag == 'data_manager', 'A data manager configuration must have a "data_manager" tag as the root. "%s" is present' % (elem.tag)
+        assert elem.tag == 'data_manager', f'A data manager configuration must have a "data_manager" tag as the root. "{elem.tag}" is present'
         self.declared_id = elem.get('id')
         self.guid = elem.get('guid')
         path = elem.get('tool_file')
-        self.version = elem.get('version', self.version)
         tool_shed_repository = None
         tool_guid = None
 
         if path is None:
             tool_elem = elem.find('tool')
-            assert tool_elem is not None, "Error loading tool for data manager. Make sure that a tool_file attribute or a tool tag set has been defined:\n%s" % (util.xml_to_string(elem))
+            assert tool_elem is not None, f"Error loading tool for data manager. Make sure that a tool_file attribute or a tool tag set has been defined:\n{util.xml_to_string(elem)}"
             path = tool_elem.get("file")
             tool_guid = tool_elem.get("guid")
             # need to determine repository info so that dependencies will work correctly
@@ -158,7 +160,7 @@ class DataManager:
                 shed_conf = self.data_managers.app.toolbox.get_shed_config_dict_by_filename(shed_conf_file)
                 if shed_conf:
                     tool_path = shed_conf.get("tool_path", tool_path)
-        assert path is not None, "A tool file path could not be determined:\n%s" % (util.xml_to_string(elem))
+        assert path is not None, f"A tool file path could not be determined:\n{util.xml_to_string(elem)}"
         self.load_tool(os.path.join(tool_path, path),
                        guid=tool_guid,
                        data_manager_id=self.id,
@@ -166,12 +168,13 @@ class DataManager:
         self.name = elem.get('name', self.tool.name)
         self.description = elem.get('description', self.tool.description)
         self.undeclared_tables = util.asbool(elem.get('undeclared_tables', self.undeclared_tables))
+        self.version = elem.get('version', self.tool.version)
 
         for data_table_elem in elem.findall('data_table'):
             data_table_name = data_table_elem.get("name")
             assert data_table_name is not None, "A name is required for a data table entry"
             if data_table_name not in self.data_tables:
-                self.data_tables[data_table_name] = OrderedDict()
+                self.data_tables[data_table_name] = {}
             output_elem = data_table_elem.find('output')
             if output_elem is not None:
                 for column_elem in output_elem.findall('column'):
@@ -198,9 +201,9 @@ class DataManager:
                                     if value_translation in VALUE_TRANSLATION_FUNCTIONS:
                                         value_translation = VALUE_TRANSLATION_FUNCTIONS[value_translation]
                                     else:
-                                        raise ValueError("Unsupported value translation function: '%s'" % (value_translation))
+                                        raise ValueError(f"Unsupported value translation function: '{value_translation}'")
                                 else:
-                                    assert value_translation_type == DEFAULT_VALUE_TRANSLATION_TYPE, ValueError("Unsupported value translation type: '%s'" % (value_translation_type))
+                                    assert value_translation_type == DEFAULT_VALUE_TRANSLATION_TYPE, ValueError(f"Unsupported value translation type: '{value_translation_type}'")
                                 self.value_translation_by_data_table_column[data_table_name][data_table_coumn_name].append(value_translation)
 
                     for move_elem in column_elem.findall('move'):
@@ -254,7 +257,7 @@ class DataManager:
             try:
                 output_dict = json.loads(open(output_dataset.file_name).read())
             except Exception as e:
-                log.warning('Error reading DataManagerTool json for "{}": {}'.format(output_name, e))
+                log.warning(f'Error reading DataManagerTool json for "{output_name}": {e}')
                 continue
             data_manager_dicts[output_name] = output_dict
             for key, value in output_dict.items():
@@ -267,7 +270,7 @@ class DataManager:
         for data_table_name in self.data_tables.keys():
             data_table_values = data_tables_dict.pop(data_table_name, None)
             if not data_table_values:
-                log.warning('No values for data table "{}" were returned by the data manager "{}".'.format(data_table_name, self.id))
+                log.warning(f'No values for data table "{data_table_name}" were returned by the data manager "{self.id}".')
                 continue  # next data table
             data_table_remove_values = None
             if isinstance(data_table_values, dict):
@@ -279,10 +282,10 @@ class DataManager:
 
             data_table = self.data_managers.app.tool_data_tables.get(data_table_name, None)
             if data_table is None:
-                log.error('The data manager "{}" returned an unknown data table "{}" with new entries "{}". These entries will not be created. Please confirm that an entry for "{}" exists in your "{}" file.'.format(self.id, data_table_name, data_table_values, data_table_name, 'tool_data_table_conf.xml'))
+                log.error(f"The data manager \"{self.id}\" returned an unknown data table \"{data_table_name}\" with new entries \"{data_table_values}\". These entries will not be created. Please confirm that an entry for \"{data_table_name}\" exists in your \"tool_data_table_conf.xml\" file.")
                 continue  # next table name
             if not isinstance(data_table, SUPPORTED_DATA_TABLE_TYPES):
-                log.error('The data manager "{}" returned an unsupported data table "{}" with type "{}" with new entries "{}". These entries will not be created. Please confirm that the data table is of a supported type ({}).'.format(self.id, data_table_name, type(data_table), data_table_values, SUPPORTED_DATA_TABLE_TYPES))
+                log.error(f'The data manager "{self.id}" returned an unsupported data table "{data_table_name}" with type "{type(data_table)}" with new entries "{data_table_values}". These entries will not be created. Please confirm that the data table is of a supported type ({SUPPORTED_DATA_TABLE_TYPES}).')
                 continue  # next table name
             output_ref_values = {}
             if data_table_name in self.output_ref_by_data_table:
@@ -297,7 +300,7 @@ class DataManager:
                 data_table_remove_values = [data_table_remove_values] if data_table_remove_values else []
             for data_table_row in data_table_values:
                 data_table_value = dict(**data_table_row)  # keep original values here
-                for name, value in data_table_row.items():  # FIXME: need to loop through here based upon order listed in data_manager config
+                for name in data_table_row.keys():  # FIXME: need to loop through here based upon order listed in data_manager config
                     if name in output_ref_values:
                         self.process_move(data_table_name, name, output_ref_values[name].extra_files_path, **data_table_value)
                         data_table_value[name] = self.process_value_translation(data_table_name, name, **data_table_value)
@@ -339,7 +342,7 @@ class DataManager:
             for data_table_name, data_table_values in data_tables_dict.items():
                 # tool returned extra data table entries, but data table was not declared in data manager
                 # do not add these values, but do provide messages
-                log.warning('The data manager "{}" returned an undeclared data table "{}" with new entries "{}". These entries will not be created. Please confirm that an entry for "{}" exists in your "{}" file.'.format(self.id, data_table_name, data_table_values, data_table_name, self.data_managers.filename))
+                log.warning(f'The data manager "{self.id}" returned an undeclared data table "{data_table_name}" with new entries "{data_table_values}". These entries will not be created. Please confirm that an entry for "{data_table_name}" exists in your "{self.data_managers.filename}" file.')
 
     def process_move(self, data_table_name, column_name, source_base_path, relative_symlinks=False, **kwd):
         if data_table_name in self.move_by_data_table_column and column_name in self.move_by_data_table_column[data_table_name]:
