@@ -27,7 +27,7 @@ def evaluate(config, input):
                           close_fds=True,
                           stdin=subprocess.PIPE,
                           stdout=subprocess.PIPE)
-    input_str = json.dumps(new_input) + "\n\n"
+    input_str = f"{json.dumps(new_input)}\n\n"
     input_bytes = input_str.encode("utf-8")
     (stdoutdata, stderrdata) = sp.communicate(input_bytes)
     if sp.returncode != 0:
@@ -35,4 +35,5 @@ def evaluate(config, input):
         message = "Expression engine returned non-zero exit code on evaluation of\n%s%s%s" % args
         raise Exception(message)
 
-    return json.loads(stdoutdata.decode("utf-8"))
+    rval_raw = stdoutdata.decode("utf-8")
+    return json.loads(rval_raw)
