@@ -6,36 +6,27 @@ usage: %prog archive_source dest_dir
     --[url|file] source type, either a URL or a file.
 """
 
+import json
 import math
 import optparse
 import os
 import tarfile
-import tempfile
 from base64 import b64decode
 
-import requests
+from galaxy.datatypes import sniff
 
 # Set max size of archive/file that will be handled to be 100 GB. This is
 # arbitrary and should be adjusted as needed.
 MAX_SIZE = 100 * math.pow(2, 30)
 
 
-def url_to_file(url, dest_file):
-    """
-    Transfer a file from a remote URL to a temporary file.
-    """
-    url_reader = requests.get(url, stream=True)
-    assert url_reader.ok, "History import failed, server returned '%s'" % url_reader.reason
-    CHUNK = 10 * 1024  # 10k
-    total = 0
-    with open(dest_file, 'wb') as fp:
-        for chunk in url_reader.iter_content(chunk_size=CHUNK):
-            if chunk:
-                fp.write(chunk)
-                total += CHUNK
-                if total > MAX_SIZE:
-                    break
-    return dest_file
+def get_file_sources(file_sources_path):
+    assert os.path.exists(file_sources_path), f"file sources path [{file_sources_path}] does not exist"
+    from galaxy.files import ConfiguredFileSources
+    with open(file_sources_path) as f:
+        file_sources_as_dict = json.load(f)
+    file_sources = ConfiguredFileSources.from_dict(file_sources_as_dict)
+    return file_sources
 
 
 def check_archive(archive_file, dest_dir):
@@ -50,7 +41,7 @@ def check_archive(archive_file, dest_dir):
                     dest_dir,
                     arc_path
                 )).startswith(dest_dir.rstrip(os.sep) + os.sep), \
-                "Archive member would extract outside target directory: %s" % arc_path
+                f"Archive member would extract outside target directory: {arc_path}"
     return True
 
 
@@ -74,7 +65,7 @@ def main(options, args):
 
     # Get archive from URL.
     if is_url:
-        archive_file = url_to_file(archive_source, tempfile.NamedTemporaryFile(dir=dest_dir).name)
+        archive_file = sniff.stream_url_to_file(archive_source, file_sources=get_file_sources(options.file_sources))
     elif is_file:
         archive_file = archive_source
 
@@ -89,5 +80,6 @@ if __name__ == "__main__":
     parser.add_option('-U', '--url', dest='is_url', action="store_true", help='Source is a URL.')
     parser.add_option('-F', '--file', dest='is_file', action="store_true", help='Source is a file.')
     parser.add_option('-e', '--encoded', dest='is_b64encoded', action="store_true", default=False, help='Source and destination dir values are base64 encoded.')
+    parser.add_option('--file-sources', type=str, help='file sources json')
     (options, args) = parser.parse_args()
     main(options, args)
