@@ -1,7 +1,7 @@
 import copy
 import itertools
 import logging
-from collections import namedtuple, OrderedDict
+from collections import namedtuple
 
 from galaxy import (
     exceptions,
@@ -17,14 +17,14 @@ log = logging.getLogger(__name__)
 WorkflowParameterExpansion = namedtuple('WorkflowParameterExpansion', ['param_combinations', 'param_keys', 'input_combinations'])
 
 
-class ParamKey(object):
+class ParamKey:
 
     def __init__(self, step_id, key):
         self.step_id = step_id
         self.key = key
 
 
-class InputKey(object):
+class InputKey:
 
     def __init__(self, input_id):
         self.input_id = input_id
@@ -178,7 +178,7 @@ def expand_meta_parameters(trans, tool, incoming):
         if not incoming_key.startswith('__'):
             process_key(incoming_key, incoming_value=incoming_value, d=nested_dict)
 
-    reordered_incoming = OrderedDict()
+    reordered_incoming = {}
 
     def visitor(input, value, prefix, prefixed_name, prefixed_label, error, **kwargs):
         if prefixed_name in incoming_copy:
@@ -218,7 +218,7 @@ def expand_meta_parameters(trans, tool, incoming):
 
     expanded_incomings = permutations.expand_multi_inputs(incoming_template, classifier)
     if collections_to_match.has_collections():
-        collection_info = trans.app.dataset_collections_service.match_collections(collections_to_match)
+        collection_info = trans.app.dataset_collection_manager.match_collections(collections_to_match)
     else:
         collection_info = None
     return expanded_incomings, collection_info
@@ -233,7 +233,7 @@ def __expand_collection_parameter(trans, input_key, incoming_val, collections_to
         try:
             src = incoming_val["src"]
             if src != "hdca":
-                raise exceptions.ToolMetaParameterException("Invalid dataset collection source type %s" % src)
+                raise exceptions.ToolMetaParameterException(f"Invalid dataset collection source type {src}")
             encoded_hdc_id = incoming_val["id"]
             subcollection_type = incoming_val.get('map_over_type', None)
         except TypeError:
