@@ -11,6 +11,7 @@ import yaml
 
 from galaxy.tools.parameters import populate_state
 from galaxy.tools.parameters.basic import workflow_building_modes
+from galaxy.util import DEFAULT_SOCKET_TIMEOUT
 from galaxy.workflow.modules import module_factory
 
 log = logging.getLogger(__name__)
@@ -79,7 +80,7 @@ class ToolRecommendations():
                         # iterate through all the attributes of the model to find weights of neural network layers
                         for item in trained_model.keys():
                             if "weight_" in item:
-                                weight = trained_model["weight_" + str(counter_layer_weights)][()]
+                                weight = trained_model[f"weight_{str(counter_layer_weights)}"][()]
                                 model_weights.append(weight)
                                 counter_layer_weights += 1
                         self.loaded_model = tf.keras.models.model_from_json(model_config)
@@ -133,7 +134,7 @@ class ToolRecommendations():
         """
         local_dir = os.path.join(os.getcwd(), download_local, 'tool_recommendation_model.hdf5')
         # read model from remote
-        model_binary = requests.get(model_url)
+        model_binary = requests.get(model_url, timeout=DEFAULT_SOCKET_TIMEOUT)
         # save model to a local directory
         with open(local_dir, 'wb') as model_file:
             model_file.write(model_binary.content)
@@ -271,7 +272,7 @@ class ToolRecommendations():
                 try:
                     sample[idx] = int(self.model_data_dictionary[tool_name])
                 except Exception:
-                    log.exception("Failed to find tool %s in model" % (tool_name))
+                    log.exception(f"Failed to find tool {tool_name} in model")
                     return prediction_data
             sample = np.reshape(sample, (1, self.max_seq_len))
             # boost the predicted scores using tools' usage
