@@ -12,14 +12,18 @@ from galaxy.util.dbkeys import GenomeBuilds
 class ValidationContext:
     """Minimal App object for tool validation."""
 
-    def __init__(self, app_name,
-                 security,
-                 model,
-                 tool_data_path,
-                 shed_tool_data_path,
-                 tool_data_tables=None,
-                 registry=None,
-                 hgweb_config_manager=None):
+    def __init__(
+        self,
+        app_name,
+        security,
+        model,
+        tool_data_path,
+        shed_tool_data_path,
+        tool_data_tables=None,
+        registry=None,
+        hgweb_config_manager=None,
+        biotools_metadata_source=None,
+    ):
         self.name = app_name
         self.security = security
         self.model = model
@@ -34,12 +38,14 @@ class ValidationContext:
         self.tool_shed_registry = Bunch(tool_sheds={})
         self.datatypes_registry = registry
         self.hgweb_config_manager = hgweb_config_manager
+        self.biotools_metadata_source = biotools_metadata_source
         self.config.len_file_path = os.path.join(self.temporary_path, 'chromlen.txt')
         # If the builds file path is set to None, tools/__init__.py will load the default.
         # Otherwise it will attempt to load a nonexistent file and log an error. This does
         # not appear to be an issue with the len_file_path config option.
         self.config.builds_file_path = None
         self.genome_builds = GenomeBuilds(self)
+        self.job_search = None
 
     def __enter__(self):
         return self
@@ -55,15 +61,19 @@ class ValidationContext:
             work_dir = tempfile.mkdtemp()
             cleanup = True
         tool_data_tables = ToolDataTableManager(work_dir)
-        with ValidationContext(app_name=app.name,
-                               security=app.security,
-                               model=app.model,
-                               tool_data_path=work_dir,
-                               shed_tool_data_path=work_dir,
-                               tool_data_tables=tool_data_tables,
-                               registry=app.datatypes_registry,
-                               hgweb_config_manager=getattr(app, 'hgweb_config_manager', None)
-                               ) as app:
-            yield app
-        if cleanup:
-            shutil.rmtree(work_dir, ignore_errors=True)
+        try:
+            with ValidationContext(
+                app_name=app.name,
+                security=app.security,
+                model=app.model,
+                tool_data_path=work_dir,
+                shed_tool_data_path=work_dir,
+                tool_data_tables=tool_data_tables,
+                registry=app.datatypes_registry,
+                hgweb_config_manager=getattr(app, 'hgweb_config_manager', None),
+                biotools_metadata_source=getattr(app, 'biotools_metadata_source', None),
+            ) as app:
+                yield app
+        finally:
+            if cleanup:
+                shutil.rmtree(work_dir, ignore_errors=True)
