@@ -7,6 +7,7 @@ through the library docs at https://whoosh.readthedocs.io.
 import logging
 import os
 import re
+from typing import Dict, List, Tuple, Union
 
 from whoosh import (
     analysis,
@@ -34,8 +35,11 @@ from galaxy.web.framework.helpers import to_unicode
 
 log = logging.getLogger(__name__)
 
+CanConvertToFloat = Union[str, int, float]
+CanConvertToInt = Union[str, int, float]
 
-def get_or_create_index(index_dir, schema):
+
+def get_or_create_index(index_dir: str, schema: Schema) -> index.Index:
     if not os.path.exists(index_dir):
         os.makedirs(index_dir)
     if index.exists_in(index_dir):
@@ -49,12 +53,44 @@ def get_or_create_index(index_dir, schema):
 
 
 class ToolBoxSearch:
+    """Support searching across all fixed panel views in a toolbox.
+
+    Search is delegated off to ToolPanelViewSearch for each panel object.
+    """
+
+    def __init__(self, toolbox, index_dir: str, index_help: bool = True):
+        panel_searches = {}
+        for panel_view in toolbox.panel_views():
+            panel_view_id = panel_view.id
+            panel_index_dir = os.path.join(index_dir, panel_view_id)
+            panel_searches[panel_view_id] = ToolPanelViewSearch(panel_view_id, panel_index_dir, index_help=index_help)
+        self.panel_searches = panel_searches
+        # We keep track of how many times the tool index has been rebuilt.
+        # We start at -1, so that after the first index the count is at 0,
+        # which is the same as the toolbox reload count. This way we can skip
+        # reindexing if the index count is equal to the toolbox reload count.
+        self.index_count = -1
+
+    def build_index(self, tool_cache, toolbox, index_help: bool = True) -> None:
+        self.index_count += 1
+        for panel_search in self.panel_searches.values():
+            panel_search.build_index(tool_cache, toolbox, index_help=index_help)
+
+    def search(self, *args, **kwd) -> List[str]:
+        panel_view = kwd.pop("panel_view")
+        if panel_view not in self.panel_searches:
+            raise KeyError(f"Unknown panel_view specified {panel_view}")
+        panel_search = self.panel_searches[panel_view]
+        return panel_search.search(*args, **kwd)
+
+
+class ToolPanelViewSearch:
     """
     Support searching tools in a toolbox. This implementation uses
     the Whoosh search library.
     """
 
-    def __init__(self, toolbox, index_dir=None, index_help=True):
+    def __init__(self, panel_view_id: str, index_dir: str, index_help: bool = True):
         self.schema = Schema(id=ID(stored=True, unique=True),
                              old_id=ID,
                              stub=KEYWORD,
@@ -65,24 +101,18 @@ class ToolBoxSearch:
                              labels=KEYWORD)
         self.rex = analysis.RegexTokenizer()
         self.index_dir = index_dir
-        self.toolbox = toolbox
+        self.panel_view_id = panel_view_id
         self.index = self._index_setup()
-        # We keep track of how many times the tool index has been rebuilt.
-        # We start at -1, so that after the first index the count is at 0,
-        # which is the same as the toolbox reload count. This way we can skip
-        # reindexing if the index count is equal to the toolbox reload count.
-        self.index_count = -1
 
-    def _index_setup(self):
+    def _index_setup(self) -> index.Index:
         return get_or_create_index(index_dir=self.index_dir, schema=self.schema)
 
-    def build_index(self, tool_cache, index_help=True):
+    def build_index(self, tool_cache, toolbox, index_help: bool = True) -> None:
         """
         Prepare search index for tools loaded in toolbox.
         Use `tool_cache` to determine which tools need indexing and which tools should be expired.
         """
-        log.debug('Starting to build toolbox index.')
-        self.index_count += 1
+        log.debug(f"Starting to build toolbox index of panel {self.panel_view_id}.")
         execution_timer = ExecutionTimer()
         with self.index.reader() as reader:
             # Index ocasionally contains empty stored fields
@@ -90,17 +120,19 @@ class ToolBoxSearch:
         tool_ids_to_remove = (indexed_tool_ids - set(tool_cache._tool_paths_by_id.keys())).union(tool_cache._removed_tool_ids)
         for indexed_tool_id in indexed_tool_ids:
             indexed_tool = tool_cache.get_tool_by_id(indexed_tool_id)
-            if not indexed_tool:
-                tool_ids_to_remove.add(indexed_tool_id)
-                continue
-            if not indexed_tool.is_latest_version and not indexed_tool.latest_version.hidden:
-                tool_ids_to_remove.add(indexed_tool_id)
+            if indexed_tool:
+                if indexed_tool.is_latest_version:
+                    continue
+                latest_version = indexed_tool.latest_version
+                if latest_version and latest_version.hidden:
+                    continue
+            tool_ids_to_remove.add(indexed_tool_id)
         with AsyncWriter(self.index) as writer:
             for tool_id in tool_ids_to_remove:
                 writer.delete_by_term('id', tool_id)
             for tool_id in tool_cache._new_tool_ids - indexed_tool_ids:
-                tool = tool_cache.get_tool_by_id(tool_id)
-                if tool and tool.is_latest_version:
+                tool = toolbox.get_tool(tool_id)
+                if tool and tool.is_latest_version and toolbox.panel_has_tool(tool, self.panel_view_id):
                     if tool.hidden:
                         # we check if there is an older tool we can return
                         if tool.lineage:
@@ -115,9 +147,9 @@ class ToolBoxSearch:
                             continue
                     add_doc_kwds = self._create_doc(tool_id=tool_id, tool=tool, index_help=index_help)
                     writer.update_document(**add_doc_kwds)
-        log.debug("Toolbox index finished %s", execution_timer)
+        log.debug(f"Toolbox index of panel {self.panel_view_id} finished {execution_timer}")
 
-    def _create_doc(self, tool_id, tool, index_help=True):
+    def _create_doc(self, tool_id: str, tool, index_help: bool = True) -> Dict[str, str]:
         #  Do not add data managers to the public index
         if tool.tool_type == 'manage_data':
             return {}
@@ -151,10 +183,20 @@ class ToolBoxSearch:
                     pass
         return add_doc_kwds
 
-    def search(self, q, tool_name_boost, tool_id_boost, tool_section_boost,
-            tool_description_boost, tool_label_boost, tool_stub_boost,
-            tool_help_boost, tool_search_limit, tool_enable_ngram_search,
-            tool_ngram_minsize, tool_ngram_maxsize):
+    def search(
+        self, q: str,
+        tool_name_boost: CanConvertToFloat,
+        tool_id_boost: CanConvertToFloat,
+        tool_section_boost: CanConvertToFloat,
+        tool_description_boost: CanConvertToFloat,
+        tool_label_boost: CanConvertToFloat,
+        tool_stub_boost: CanConvertToFloat,
+        tool_help_boost: CanConvertToFloat,
+        tool_search_limit: CanConvertToFloat,
+        tool_enable_ngram_search: bool,
+        tool_ngram_minsize: CanConvertToInt,
+        tool_ngram_maxsize: CanConvertToInt
+    ) -> List[str]:
         """
         Perform search on the in-memory index. Weight in the given boosts.
         """
@@ -188,22 +230,22 @@ class ToolBoxSearch:
         else:
             cleaned_query = ' '.join(token.text for token in self.rex(cleaned_query))
             # Use asterisk Whoosh wildcard so e.g. 'bow' easily matches 'bowtie'
-            parsed_query = self.parser.parse('*' + cleaned_query + '*')
+            parsed_query = self.parser.parse(f"*{cleaned_query}*")
             hits = self.searcher.search(parsed_query, limit=float(tool_search_limit), sortedby='')
             return [hit['id'] for hit in hits]
 
-    def _search_ngrams(self, cleaned_query, tool_ngram_minsize, tool_ngram_maxsize, tool_search_limit):
+    def _search_ngrams(self, cleaned_query: str, tool_ngram_minsize: CanConvertToInt, tool_ngram_maxsize: CanConvertToInt, tool_search_limit: CanConvertToFloat) -> List[str]:
         """
         Break tokens into ngrams and search on those instead.
         This should make searching more resistant to typos and unfinished words.
         See docs at https://whoosh.readthedocs.io/en/latest/ngrams.html
         """
-        hits_with_score = {}
+        hits_with_score: Dict[str, float] = {}
         token_analyzer = StandardAnalyzer() | analysis.NgramFilter(minsize=int(tool_ngram_minsize), maxsize=int(tool_ngram_maxsize))
         ngrams = [token.text for token in token_analyzer(cleaned_query)]
         for query in ngrams:
             # Get the tool list with respective scores for each qgram
-            curr_hits = self.searcher.search(self.parser.parse('*' + query + '*'), limit=float(tool_search_limit))
+            curr_hits = self.searcher.search(self.parser.parse(f"*{query}*"), limit=float(tool_search_limit))
             for i, curr_hit in enumerate(curr_hits):
                 is_present = False
                 for prev_hit in hits_with_score:
@@ -217,6 +259,6 @@ class ToolBoxSearch:
                 if not is_present:
                     hits_with_score[curr_hit['id']] = curr_hits.score(i)
         # Sort the results based on aggregated BM25 score in decreasing order of scores
-        hits_with_score = sorted(hits_with_score.items(), key=lambda x: x[1], reverse=True)
+        hits_with_score_list: List[Tuple[str, float]] = sorted(hits_with_score.items(), key=lambda x: x[1], reverse=True)
         # Return the tool ids
-        return [item[0] for item in hits_with_score[0:int(tool_search_limit)]]
+        return [item[0] for item in hits_with_score_list[0:int(tool_search_limit)]]
