@@ -1,13 +1,14 @@
 import logging
 import os
 import os.path
-
+from typing import List, Tuple
 
 import galaxy.tools.parameters.basic
 import galaxy.tools.parameters.grouping
 from galaxy.tool_util.verify.interactor import ToolTestDescription
 from galaxy.util import (
     string_as_bool,
+    string_as_bool_or_none,
     unicodify,
 )
 
@@ -35,14 +36,16 @@ def parse_tests(tool, tests_source):
 
 
 def description_from_tool_object(tool, test_index, raw_test_dict):
-    required_files = []
+    required_files: List[Tuple[str, dict]] = []
+    required_data_tables: List[str] = []
+    required_loc_files: List[str] = []
 
     num_outputs = raw_test_dict.get('expect_num_outputs', None)
     if num_outputs:
         num_outputs = int(num_outputs)
 
     try:
-        processed_inputs = _process_raw_inputs(tool, tool.inputs, raw_test_dict["inputs"], required_files)
+        processed_inputs = _process_raw_inputs(tool, tool.inputs, raw_test_dict["inputs"], required_files, required_data_tables, required_loc_files)
         processed_test_dict = {
             "inputs": processed_inputs,
             "outputs": raw_test_dict["outputs"],
@@ -54,8 +57,12 @@ def description_from_tool_object(tool, test_index, raw_test_dict):
             "stderr": raw_test_dict.get("stderr", None),
             "expect_exit_code": raw_test_dict.get("expect_exit_code", None),
             "expect_failure": raw_test_dict.get("expect_failure", False),
+            "expect_test_failure": raw_test_dict.get("expect_test_failure", False),
             "required_files": required_files,
+            "required_data_tables": required_data_tables,
+            "required_loc_files": required_loc_files,
             "tool_id": tool.id,
+            "tool_version": tool.version,
             "test_index": test_index,
             "error": False,
         }
@@ -63,6 +70,7 @@ def description_from_tool_object(tool, test_index, raw_test_dict):
         log.exception("Failed to load tool test number [%d] for %s" % (test_index, tool.id))
         processed_test_dict = {
             "tool_id": tool.id,
+            "tool_version": tool.version,
             "test_index": test_index,
             "inputs": {},
             "error": True,
@@ -72,7 +80,7 @@ def description_from_tool_object(tool, test_index, raw_test_dict):
     return ToolTestDescription(processed_test_dict)
 
 
-def _process_raw_inputs(tool, tool_inputs, raw_inputs, required_files, parent_context=None):
+def _process_raw_inputs(tool, tool_inputs, raw_inputs, required_files, required_data_tables, required_loc_files, parent_context=None):
     """
     Recursively expand flat list of inputs into "tree" form of flat list
     (| using to nest to new levels) structure and expand dataset
@@ -80,16 +88,17 @@ def _process_raw_inputs(tool, tool_inputs, raw_inputs, required_files, parent_co
     """
     parent_context = parent_context or RootParamContext()
     expanded_inputs = {}
-    for key, value in tool_inputs.items():
+    for value in tool_inputs.values():
         if isinstance(value, galaxy.tools.parameters.grouping.Conditional):
             cond_context = ParamContext(name=value.name, parent_context=parent_context)
+            assert value.test_param
             case_context = ParamContext(name=value.test_param.name, parent_context=cond_context)
             raw_input_dict = case_context.extract_value(raw_inputs)
             case_value = raw_input_dict["value"] if raw_input_dict else None
             case = _matching_case_for_value(tool, value, case_value)
             if case:
                 for input_name, input_value in case.inputs.items():
-                    case_inputs = _process_raw_inputs(tool, {input_name: input_value}, raw_inputs, required_files, parent_context=cond_context)
+                    case_inputs = _process_raw_inputs(tool, {input_name: input_value}, raw_inputs, required_files, required_data_tables, required_loc_files, parent_context=cond_context)
                     expanded_inputs.update(case_inputs)
                 if not value.type == "text":
                     expanded_case_value = _split_if_str(case.value)
@@ -101,12 +110,13 @@ def _process_raw_inputs(tool, tool_inputs, raw_inputs, required_files, parent_co
                     # as a new instance with value defined and hence enter
                     # an infinite loop - hence the "case_value is not None"
                     # check.
-                    processed_value = _process_simple_value(value.test_param, expanded_case_value)
+                    processed_value = _process_simple_value(value.test_param, expanded_case_value, required_data_tables, required_loc_files)
                     expanded_inputs[case_context.for_state()] = processed_value
         elif isinstance(value, galaxy.tools.parameters.grouping.Section):
             context = ParamContext(name=value.name, parent_context=parent_context)
-            for r_name, r_value in value.inputs.items():
-                expanded_input = _process_raw_inputs(tool, {context.for_state(): r_value}, raw_inputs, required_files, parent_context=context)
+            assert value.inputs
+            for r_value in value.inputs.values():
+                expanded_input = _process_raw_inputs(tool, {context.for_state(): r_value}, raw_inputs, required_files, required_data_tables, required_loc_files, parent_context=context)
                 if expanded_input:
                     expanded_inputs.update(expanded_input)
         elif isinstance(value, galaxy.tools.parameters.grouping.Repeat):
@@ -114,8 +124,9 @@ def _process_raw_inputs(tool, tool_inputs, raw_inputs, required_files, parent_co
             while True:
                 context = ParamContext(name=value.name, index=repeat_index, parent_context=parent_context)
                 updated = False
-                for r_name, r_value in value.inputs.items():
-                    expanded_input = _process_raw_inputs(tool, {context.for_state(): r_value}, raw_inputs, required_files, parent_context=context)
+                assert value.inputs
+                for r_value in value.inputs.values():
+                    expanded_input = _process_raw_inputs(tool, {context.for_state(): r_value}, raw_inputs, required_files, required_data_tables, required_loc_files, parent_context=context)
                     if expanded_input:
                         expanded_inputs.update(expanded_input)
                         updated = True
@@ -147,13 +158,13 @@ def _process_raw_inputs(tool, tool_inputs, raw_inputs, required_files, parent_co
                         require_file(name, value, attributes, required_files)
                     processed_value = collection_def
                 else:
-                    processed_value = _process_simple_value(value, param_value)
+                    processed_value = _process_simple_value(value, param_value, required_data_tables, required_loc_files)
                 expanded_inputs[context.for_state()] = processed_value
     return expanded_inputs
 
 
-def _process_simple_value(param, param_value):
-    if isinstance(param, galaxy.tools.parameters.basic.SelectToolParameter) and hasattr(param, 'static_options'):
+def _process_simple_value(param, param_value, required_data_tables, required_loc_files):
+    if isinstance(param, galaxy.tools.parameters.basic.SelectToolParameter):
         # Tests may specify values as either raw value or the value
         # as they appear in the list - the API doesn't and shouldn't
         # accept the text value - so we need to convert the text
@@ -161,12 +172,16 @@ def _process_simple_value(param, param_value):
         def process_param_value(param_value):
             found_value = False
             value_for_text = None
-            if param.static_options:
-                for (text, opt_value, selected) in param.static_options:
-                    if param_value == opt_value:
-                        found_value = True
-                    if value_for_text is None and param_value == text:
-                        value_for_text = opt_value
+            for (text, opt_value, _) in getattr(param, 'static_options', []):
+                if param_value == opt_value:
+                    found_value = True
+                if value_for_text is None and param_value == text:
+                    value_for_text = opt_value
+            if param.options:
+                if param.options.tool_data_table_name:
+                    required_data_tables.append(param.options.tool_data_table_name)
+                elif param.options.index_file:
+                    required_loc_files.append(param.options.index_file)
             if not found_value and value_for_text is not None:
                 processed_value = value_for_text
             else:
@@ -208,7 +223,7 @@ def _matching_case_for_value(tool, cond, declared_value):
             # No explicit value in test case, not much to do if options are dynamic but
             # if static options are available can find the one specified as default or
             # fallback on top most option (like GUI).
-            for (name, value, selected) in test_param.static_options:
+            for (name, _, selected) in test_param.static_options:
                 if selected:
                     default_option = name
             else:
@@ -222,12 +237,12 @@ def _matching_case_for_value(tool, cond, declared_value):
             # No explicit value for this param and cannot determine a
             # default - give up. Previously this would just result in a key
             # error exception.
-            msg = "Failed to find test parameter value specification required for conditional %s" % cond.name
+            msg = f"Failed to find test parameter value specification required for conditional {cond.name}"
             raise Exception(msg)
 
     # Check the tool's defined cases against predicate to determine
     # selected or default.
-    for i, case in enumerate(cond.cases):
+    for case in cond.cases:
         if matches_declared_value(case.value):
             return case
     else:
@@ -238,7 +253,7 @@ def _matching_case_for_value(tool, cond, declared_value):
 
 def _add_uploaded_dataset(name, value, extra, input_parameter, required_files):
     if value is None:
-        assert input_parameter.optional, '%s is not optional. You must provide a valid filename.' % name
+        assert input_parameter.optional, f'{name} is not optional. You must provide a valid filename.'
         return value
     return require_file(name, value, extra, required_files)
 
@@ -261,7 +276,10 @@ def _process_bool_param_value(param, param_value):
     elif param.falsevalue == param_value:
         processed_value = False
     else:
-        processed_value = string_as_bool(param_value)
+        if param.optional:
+            processed_value = string_as_bool_or_none(param_value)
+        else:
+            processed_value = string_as_bool(param_value)
     return [processed_value] if was_list else processed_value
 
 
@@ -292,19 +310,19 @@ class ParamContext:
         name = self.name if self.index is None else "%s_%d" % (self.name, self.index)
         parent_for_state = self.parent_context.for_state()
         if parent_for_state:
-            return "{}|{}".format(parent_for_state, name)
+            return f"{parent_for_state}|{name}"
         else:
             return name
 
     def __str__(self):
-        return "Context[for_state=%s]" % self.for_state()
+        return f"Context[for_state={self.for_state()}]"
 
     def param_names(self):
         for parent_context_param in self.parent_context.param_names():
             if self.index is not None:
                 yield "%s|%s_%d" % (parent_context_param, self.name, self.index)
             else:
-                yield "{}|{}".format(parent_context_param, self.name)
+                yield f"{parent_context_param}|{self.name}"
         if self.index is not None:
             yield "%s_%d" % (self.name, self.index)
         else:
