@@ -36,7 +36,7 @@ def parse_cols_arg(cols):
 
 def default_printer(stream, exc, obj):
     print("%d: %s" % (obj.linenum, obj.current_line), file=stream)
-    print("\tError: %s" % unicodify(exc), file=stream)
+    print(f"\tError: {unicodify(exc)}", file=stream)
 
 
 def skipped(reader, filedesc=""):
