@@ -1,14 +1,20 @@
 import sys
 from json import loads
+from typing import Iterator
 
 from galaxy.datatypes.tabular import Tabular
+from galaxy.model import DatasetInstance
 
 
 class BaseDataProvider:
     """
-    Base class for data providers. Data providers (a) read and package data from datasets;
-    and (b) write subsets of data to new datasets.
+    Base class for data providers. Data providers both:
+
+    - read and package data from datasets
+    - write subsets of data to new datasets
+
     """
+    original_dataset: DatasetInstance
 
     def __init__(self, converted_dataset=None, original_dataset=None, dependencies=None,
                  error_max_vals="Only the first %i values are returned."):
@@ -25,7 +31,7 @@ class BaseDataProvider:
         """
         raise Exception("Unimplemented Function")
 
-    def get_iterator(self, **kwargs):
+    def get_iterator(self, data_file, chrom, start, end, **kwargs) -> Iterator[str]:
         """
         Returns an iterator that provides data in the region chrom:start-end
         """
@@ -92,27 +98,27 @@ class ColumnDataProvider(BaseDataProvider):
 
         # skip comment lines (if any/avail)
         # pre: should have original_dataset and
-        if(skip_comments and
-                self.original_dataset.metadata.comment_lines and
-                start_val < self.original_dataset.metadata.comment_lines):
+        if(skip_comments
+                and self.original_dataset.metadata.comment_lines
+                and start_val < self.original_dataset.metadata.comment_lines):
             start_val = int(self.original_dataset.metadata.comment_lines)
 
         # columns is an array of ints for now (should handle column names later)
         columns = loads(columns)
         for column in columns:
-            assert((column < self.original_dataset.metadata.columns) and
-                   (column >= 0)), (
-                "column index (%d) must be positive and less" % (column) +
-                " than the number of columns: %d" % (self.original_dataset.metadata.columns))
+            assert((column < self.original_dataset.metadata.columns)
+                   and (column >= 0)), (
+                "column index (%d) must be positive and less" % (column)
+                + " than the number of columns: %d" % (self.original_dataset.metadata.columns))
 
         # set up the response, column lists
         response = {}
         response['data'] = data = [[] for column in columns]
         response['meta'] = meta = [{
-            'min'   : None,
-            'max'   : None,
-            'count' : 0,
-            'sum'   : 0
+            'min': None,
+            'max': None,
+            'count': 0,
+            'sum': 0
         } for column in columns]
 
         column_types = [self.original_dataset.metadata.column_types[column] for column in columns]
