@@ -1,4 +1,5 @@
 """ Data providers code for PhyloViz """
+from typing import Any, Dict
 
 from galaxy.visualization.data_providers.basic import BaseDataProvider
 from galaxy.visualization.data_providers.phyloviz.newickparser import Newick_Parser
@@ -23,7 +24,7 @@ class PhylovizDataProvider(BaseDataProvider):
         file_name = self.original_dataset.file_name
         parseMsg = None
         jsonDicts = []
-        rval = {'dataset_type': self.dataset_type}
+        rval: Dict[str, Any] = {'dataset_type': self.dataset_type}
 
         if file_ext in ["newick", "nhx"]:  # parses newick files
             newickParser = Newick_Parser()
