@@ -43,7 +43,7 @@ class Newick_Parser(Base_Parser):
         """elements separated by comma could be empty"""
 
         if string.find("(") != -1:
-            raise Exception("Tree is not well form, location: " + string)
+            raise Exception(f"Tree is not well form, location: {string}")
 
         childrenString = string.split(",")
         childrenNodes = []
@@ -52,17 +52,17 @@ class Newick_Parser(Base_Parser):
             if len(childString) == 0:
                 continue
             nodeInfo = childString.split(":")
-            name, length, bootstrap = "", None, -1
+            name, length, bootstrap = "", None, -1.0
             if len(nodeInfo) == 2:  # has length info
                 length = nodeInfo[1]
                 # checking for bootstap values
                 name = nodeInfo[0]
                 try:    # Nexus may bootstrap in names position
-                    name = float(name)
-                    if 0 <= name <= 1:
-                        bootstrap = name
-                    elif 1 <= name <= 100:
-                        bootstrap = name / 100
+                    name_as_float = float(name)
+                    if 0 <= name_as_float <= 1:
+                        bootstrap = name_as_float
+                    elif 1 <= name_as_float <= 100:
+                        bootstrap = name_as_float / 100
                     name = ""
                 except ValueError:
                     name = nodeInfo[0]
@@ -94,7 +94,7 @@ class Newick_Parser(Base_Parser):
                         if enclosingSymbol == ")" or enclosingSymbol == ":" or enclosingSymbol == ",":
                             termToReplace = newickString[end:j]
 
-                            newString += newickString[start : end] + nameMap[termToReplace]  # + "'"  "'" +
+                            newString += newickString[start:end] + nameMap[termToReplace]  # + "'"  "'" +
                             start = j
                             break
 
@@ -159,7 +159,7 @@ class Newick_Parser(Base_Parser):
                         lenOfPreceedingInternalNodeString = 0
 
                     # recussive call to make the internal claude
-                    childSubString = string[i + 1 : j]
+                    childSubString = string[i + 1:j]
                     InternalNode.addChildNode(self.parseNode(childSubString, depth + 1))
 
                     nodes.append(InternalNode)  # we append the internal node later to preserve order
