@@ -1,7 +1,8 @@
 import logging
-
+from typing import Any, Dict, List
 
 import galaxy.model
+from galaxy.util import asbool
 from galaxy.util.xml_macros import load
 
 log = logging.getLogger(__name__)
@@ -67,13 +68,10 @@ class VisualizationsConfigParser:
             log.info('Visualizations plugin disabled: %s. Skipping...', returned['name'])
             return None
 
-        # record the embeddable flag - defaults to false
-        #   this is a design by contract promise that the visualization can be rendered inside another page
-        #   often by rendering only a DOM fragment. Since this is an advanced feature that requires a bit more
-        #   work from the creator's side - it defaults to False
-        returned['embeddable'] = False
+        # record the embeddable flag - defaults to true
+        returned['embeddable'] = True
         if 'embeddable' in xml_tree.attrib:
-            returned['embeddable'] = xml_tree.attrib.get('embeddable', False) == 'true'
+            returned['embeddable'] = asbool(xml_tree.attrib.get('embeddable'))
 
         # a (for now) text description of what the visualization does
         description = xml_tree.find('description')
@@ -110,7 +108,7 @@ class VisualizationsConfigParser:
         # param modifiers provide extra information for other params (e.g. hda_ldda='hda' -> dataset_id is an hda id)
         # store these modifiers in a 2-level dictionary { target_param: { param_modifier_key: { param_mod_data }
         # ugh - wish we didn't need these
-        param_modifiers = {}
+        param_modifiers: Dict[str, Any] = {}
         param_modifier_elements = param_confs.findall('param_modifier') if param_confs is not None else []
         for param_modifier_conf in param_modifier_elements:
             param_modifier = self.param_modifier_parser.parse(param_modifier_conf)
@@ -139,8 +137,8 @@ class VisualizationsConfigParser:
         # render_target: where in the browser to open the rendered visualization
         # defaults to: galaxy_main
         render_target = xml_tree.find('render_target')
-        if((render_target is not None and render_target.text) and
-                (render_target.text in self.VALID_RENDER_TARGETS)):
+        if((render_target is not None and render_target.text)
+                and (render_target.text in self.VALID_RENDER_TARGETS)):
             returned['render_target'] = render_target.text
         else:
             returned['render_target'] = 'galaxy_main'
@@ -166,7 +164,7 @@ class VisualizationsConfigParser:
     def parse_entry_point(self, xml_tree):
         """
         Parse the config file for an appropriate entry point: a mako template, a script tag,
-        or an html file, returning as dictionary with: `type`, `file`, and `attr`ibutes of
+        or an html file, returning as dictionary with: ``type``, ``file``, and ``attr`` (-ibutes) of
         the element.
         """
         # (older) mako-only syntax: the template to use in rendering the visualization
@@ -174,9 +172,9 @@ class VisualizationsConfigParser:
         if template is not None and template.text:
             log.info('template syntax is deprecated: use entry_point instead')
             return {
-                'type' : 'mako',
-                'file' : template.text,
-                'attr' : {}
+                'type': 'mako',
+                'file': template.text,
+                'attr': {}
             }
 
         # need one of the two: (the deprecated) template or entry_point
@@ -188,11 +186,11 @@ class VisualizationsConfigParser:
         entry_point_attrib = dict(entry_point.attrib)
         entry_point_type = entry_point_attrib.pop('entry_point_type', 'mako')
         if entry_point_type not in self.ALLOWED_ENTRY_POINT_TYPES:
-            raise ParsingException('Unknown entry_point type: ' + entry_point_type)
+            raise ParsingException(f"Unknown entry_point type: {entry_point_type}")
         return {
-            'type' : entry_point_type,
-            'file' : entry_point.text,
-            'attr' : entry_point_attrib
+            'type': entry_point_type,
+            'file': entry_point.text,
+            'attr': entry_point_attrib
         }
 
 
@@ -257,7 +255,7 @@ class DataSourceParser:
 
         if xml_tree.text not in self.ALLOWED_MODEL_CLASSES:
             # log.debug( 'available data_source model_classes: %s' %( str( self.ALLOWED_MODEL_CLASSES ) ) )
-            raise ParsingException('Invalid data_source model_class: %s' % (xml_tree.text))
+            raise ParsingException(f'Invalid data_source model_class: {xml_tree.text}')
 
         # look up the model from the model module returning an empty data_source if not found
         model_class = getattr(galaxy.model, xml_tree.text, None)
@@ -289,7 +287,7 @@ class DataSourceParser:
         # tests should NOT include expensive operations: reading file data, running jobs, etc.
         # do as much here as possible to reduce the overhead of seeing if a visualization is applicable
         # currently tests are or'd only (could be and'd or made into compound boolean tests)
-        tests = []
+        tests: List[Dict[str, Any]] = []
         if not xml_tree_list:
             return tests
 
@@ -297,8 +295,8 @@ class DataSourceParser:
             test_type = test_elem.get('type', 'eq')
             test_result = test_elem.text.strip() if test_elem.text else None
             if not test_type or not test_result:
-                log.warning('Skipping test. Needs both type attribute and text node to be parsed: ' +
-                          '{}, {}'.format(test_type, test_elem.text))
+                log.warning('Skipping test. Needs both type attribute and text node to be parsed: '
+                          + f'{test_type}, {test_elem.text}')
                 continue
             test_result = test_result.strip()
 
@@ -324,8 +322,8 @@ class DataSourceParser:
             elif test_type == 'has_dataprovider':
                 # does the object itself have a datatype attr and does that datatype have the given dataprovider
                 def test_fn(o, result):
-                    return (hasattr(getter(o), 'has_dataprovider') and
-                            getter(o).has_dataprovider(result))
+                    return (hasattr(getter(o), 'has_dataprovider')
+                            and getter(o).has_dataprovider(result))
 
             elif test_type == 'has_attribute':
                 # does the object itself have attr in 'result' (no equivalence checking)
@@ -342,10 +340,10 @@ class DataSourceParser:
                     return str(getter(o)) == result
 
             tests.append({
-                'type'          : test_type,
-                'result'        : test_result,
-                'result_type'   : test_result_type,
-                'fn'            : test_fn
+                'type': test_type,
+                'result': test_result,
+                'result_type': test_result_type,
+                'fn': test_fn
             })
 
         return tests
@@ -356,7 +354,7 @@ class DataSourceParser:
         the registry to convert the data_source into one or more appropriate
         params for the visualization.
         """
-        to_param_dict = {}
+        to_param_dict: Dict[str, Any] = {}
         if not xml_tree_list:
             return to_param_dict
 
@@ -421,13 +419,15 @@ class DictParser(dict):
             self.update(dict(parent_element.items()))
         for element in parent_element:
             if len(element) > 0:
+                asJson: Any
                 if element.tag == element[0].tag:
-                    aDict = ListParser(element)
+                    asJson = ListParser(element)
                 else:
                     aDict = DictParser(element)
-                if element.items():
-                    aDict.update(dict(element.items()))
-                self.update({element.tag: aDict})
+                    if element.items():
+                        aDict.update(dict(element.items()))
+                    asJson = aDict
+                self.update({element.tag: asJson})
             elif element.items():
                 self.update({element.tag: dict(element.items())})
             else:
