@@ -5,6 +5,7 @@ from a query string and render a webpage based on those data.
 import copy
 import logging
 import os
+from typing import Any, Dict
 
 import mako.lookup
 
@@ -23,6 +24,7 @@ class ServesTemplatesPluginMixin:
     """
     An object that renders (mako) template files from the server.
     """
+    path: str
 
     #: default number of templates to search for plugin template lookup
     DEFAULT_TEMPLATE_COLLECTION_SIZE = 10
@@ -74,7 +76,7 @@ class VisualizationPlugin(ServesTemplatesPluginMixin):
         self.base_url = '/'.join((base_url, self.name)) if base_url else self.name
         self.static_path = self._get_static_path(self.path)
         if self.static_path and os.path.exists(os.path.join(self.static_path, 'logo.png')):
-            self.config['logo'] = self.static_path + '/logo.png'
+            self.config['logo'] = f"{self.static_path}/logo.png"
         template_cache_dir = context.get('template_cache_dir', None)
         additional_template_paths = context.get('additional_template_paths', [])
         self._set_up_template_plugin(template_cache_dir, additional_template_paths=additional_template_paths)
@@ -85,9 +87,8 @@ class VisualizationPlugin(ServesTemplatesPluginMixin):
         Render and return the text of the non-saved plugin webpage/fragment.
         """
         # not saved - no existing config
-        config = {}
         # set up render vars based on plugin.config and kwargs
-        render_vars = self._build_render_vars(config, trans=trans, **kwargs)
+        render_vars = self._build_render_vars({}, trans=trans, **kwargs)
         return self._render(render_vars, trans=trans, embedded=embedded)
 
     def render_saved(self, visualization, trans=None, embedded=None, **kwargs):
@@ -95,7 +96,7 @@ class VisualizationPlugin(ServesTemplatesPluginMixin):
         Render and return the text of the plugin webpage/fragment using the
         config/data of a saved visualization.
         """
-        config = self._get_saved_visualization_config(visualization, **kwargs)
+        config: Dict[str, Any] = self._get_saved_visualization_config(visualization, **kwargs)
         # pass the saved visualization config for parsing into render vars
         render_vars = self._build_render_vars(config, trans=trans, **kwargs)
         # update any values that were loaded from the saved Visualization
@@ -108,18 +109,18 @@ class VisualizationPlugin(ServesTemplatesPluginMixin):
 
     def to_dict(self):
         return {
-            'name'          : self.name,
-            'html'          : self.config.get('name'),
-            'description'   : self.config.get('description'),
-            'logo'          : self.config.get('logo'),
-            'title'         : self.config.get('title'),
-            'target'        : self.config.get('render_target', 'galaxy_main'),
-            'embeddable'    : self.config.get('embeddable', False),
-            'entry_point'   : self.config.get('entry_point'),
-            'settings'      : self.config.get('settings'),
-            'groups'        : self.config.get('groups'),
-            'specs'         : self.config.get('specs'),
-            'href'          : self._get_url()
+            'name': self.name,
+            'html': self.config.get('name'),
+            'description': self.config.get('description'),
+            'logo': self.config.get('logo'),
+            'title': self.config.get('title'),
+            'target': self.config.get('render_target', 'galaxy_main'),
+            'embeddable': self.config.get('embeddable'),
+            'entry_point': self.config.get('entry_point'),
+            'settings': self.config.get('settings'),
+            'groups': self.config.get('groups'),
+            'specs': self.config.get('specs'),
+            'href': self._get_url()
         }
 
     def _get_url(self):
@@ -132,9 +133,9 @@ class VisualizationPlugin(ServesTemplatesPluginMixin):
             match = path.split('/config/')[-1]
             return os.path.join('./static', match, 'static')
         else:
-            log.debug('Visualization has no static path: %s.' % path)
+            log.debug(f'Visualization has no static path: {path}.')
 
-    def _get_saved_visualization_config(self, visualization, revision=None, **kwargs):
+    def _get_saved_visualization_config(self, visualization, revision=None, **kwargs) -> Dict[str, Any]:
         """
         Return the config of a saved visualization and revision.
 
@@ -144,11 +145,11 @@ class VisualizationPlugin(ServesTemplatesPluginMixin):
         return copy.copy(visualization.latest_revision.config)
 
     # ---- non-public
-    def _build_render_vars(self, config, trans=None, **kwargs):
+    def _build_render_vars(self, config: Dict[str, Any], trans=None, **kwargs) -> Dict[str, Any]:
         """
         Build all the variables that will be passed into the renderer.
         """
-        render_vars = {}
+        render_vars: Dict[str, Any] = {}
         # Meta variables passed to the template/renderer to describe the visualization being rendered.
         render_vars.update(
             visualization_name=self.name,
@@ -161,14 +162,14 @@ class VisualizationPlugin(ServesTemplatesPluginMixin):
             query=kwargs,
         )
         # config based on existing or kwargs
-        config = self._build_config(config, trans=trans, **kwargs)
-        render_vars['config'] = config
+        render_config = self._build_config(config, trans=trans, **kwargs)
+        render_vars['config'] = render_config
         # further parse config to resources (models, etc.) used in template based on registry config
-        resources = self._config_to_resources(trans, config)
+        resources = self._config_to_resources(trans, render_config)
         render_vars.update(resources)
         return render_vars
 
-    def _build_config(self, config, trans=None, **kwargs):
+    def _build_config(self, config, trans=None, **kwargs) -> utils.OpenObject:
         """
         Build the configuration for this new/saved visualization by combining
         any existing config and the kwargs (gen. from the url query).
@@ -305,10 +306,10 @@ class ScriptVisualizationPlugin(VisualizationPlugin):
         template.
         """
         render_vars['embedded'] = self._parse_embedded(embedded)
-        render_vars['static_url'] = url_for('/%s/' % self.static_path)
+        render_vars['static_url'] = url_for(f'/{self.static_path}/')
         render_vars.update(vars={})
         render_vars.update({
-            "script_attributes" : self.config['entry_point']['attr']
+            "script_attributes": self.config['entry_point']['attr']
         })
         template_filename = os.path.join(self.MAKO_TEMPLATE)
         return trans.fill_template(template_filename, template_lookup=self.template_lookup, **render_vars)
