@@ -46,7 +46,7 @@ def _read_defined_parameter_definitions(config):
         params_file = getattr(config, "job_resource_params_file", None)
     if not params_file or not os.path.exists(params_file):
         params_file = None
-    log.debug("Loading workflow resource parameter definitions from %s" % params_file)
+    log.debug(f"Loading workflow resource parameter definitions from {params_file}")
     if params_file:
         return galaxy.util.parse_resource_parameters(params_file)
     else:
@@ -85,7 +85,7 @@ def _resource_parameters_by_group(trans, **kwds):
 # returns an array of parameters that a users set of permissions can access.
 def get_workflow_parameter_list(params, user_permissions):
     param_list = []
-    for param_name, param_elem in params.items():
+    for param_elem in params.values():
         attr = deepcopy(param_elem.attrib)
         if attr['name'] in user_permissions:
             # Allow 'select' type parameters to be used
@@ -123,8 +123,8 @@ def validate_by_group_workflow_parameters_mapper(by_group, workflow_resource_par
         if 'groups' not in by_group:
             raise Exception("'workflow_resource_params_mapper' YAML file is malformed, 'groups' attribute not found!")
         if default_group not in by_group['groups']:
-            raise Exception("'workflow_resource_params_mapper' YAML file is malformed, default group with title '" +
-                            default_group + "' not found in 'groups'!")
+            raise Exception("'workflow_resource_params_mapper' YAML file is malformed, default group with title '"
+                            + default_group + "' not found in 'groups'!")
         for group in by_group['groups']:
             for attrib in by_group['groups'][group]:
                 if type(attrib) is dict:
@@ -132,8 +132,8 @@ def validate_by_group_workflow_parameters_mapper(by_group, workflow_resource_par
                         raise Exception("'workflow_resource_params_mapper' YAML file is malformed, "
                                         "'name' attribute not found in attribute of group '" + group + "'!")
                     if attrib['name'] not in workflow_resource_params:
-                        raise Exception("'workflow_resource_params_mapper' YAML file is malformed, group with name '" +
-                                        attrib['name'] + "' not found in 'workflow_resource_params'!")
+                        raise Exception("'workflow_resource_params_mapper' YAML file is malformed, group with name '"
+                                        + attrib['name'] + "' not found in 'workflow_resource_params'!")
                     if 'options' not in attrib:
                         raise Exception("'workflow_resource_params_mapper' YAML file is malformed, "
                                         "'options' attribute not found in attribute of group '" + group + "'!")
@@ -143,8 +143,8 @@ def validate_by_group_workflow_parameters_mapper(by_group, workflow_resource_par
                         valid_options.append(param_option.attrib['value'])
                     for option in attrib['options']:
                         if option not in valid_options:
-                            raise Exception("'workflow_resource_params_mapper' YAML file is malformed, '" + option +
-                                            "' in 'options' of '" + attrib['name'] + "' not found in attribute of group '" + group + "'!")
+                            raise Exception("'workflow_resource_params_mapper' YAML file is malformed, '" + option
+                                            + "' in 'options' of '" + attrib['name'] + "' not found in attribute of group '" + group + "'!")
                 else:
                     if attrib not in workflow_resource_params:
                         raise Exception("'workflow_resource_params_mapper' YAML file is malformed, attribute with name "
@@ -162,13 +162,13 @@ def _import_resource_mapping_function(qualified_function_path):
     try:
         __import__(full_module_name)
     except ImportError:
-        raise Exception("Failed to find workflow resource mapper module %s" % full_module_name)
+        raise Exception(f"Failed to find workflow resource mapper module {full_module_name}")
 
     module = sys.modules[full_module_name]
     if hasattr(module, function_name):
         return getattr(module, function_name)
     else:
-        raise Exception("Failed to find workflow resource mapper function {}.{}".format(full_module_name, function_name))
+        raise Exception(f"Failed to find workflow resource mapper function {full_module_name}.{function_name}")
 
 
 def _null_mapper_function(*args, **kwds):
