@@ -1,6 +1,6 @@
 import logging
 import uuid
-from collections import OrderedDict
+from typing import List, Union
 
 from galaxy import model
 from galaxy.util import ExecutionTimer
@@ -14,59 +14,11 @@ from galaxy.workflow.run_request import (
 log = logging.getLogger(__name__)
 
 
-# Entry point for direct invoke via controllers. Deprecated to some degree.
-def invoke(trans, workflow, workflow_run_config, workflow_invocation=None, populate_state=False):
-    if force_queue(trans, workflow):
-        invocation = queue_invoke(trans, workflow, workflow_run_config, populate_state=populate_state)
-        return [], invocation
-    else:
-        return __invoke(trans, workflow, workflow_run_config, workflow_invocation, populate_state)
-
-
 # Entry point for core workflow scheduler.
 def schedule(trans, workflow, workflow_run_config, workflow_invocation):
     return __invoke(trans, workflow, workflow_run_config, workflow_invocation)
 
 
-BASIC_WORKFLOW_STEP_TYPES = [None, "tool", "data_input", "data_collection_input"]
-
-
-def force_queue(trans, workflow):
-    # Default behavior is still to just schedule workflows completley right
-    # away. This can be modified here in various ways.
-
-    # TODO: check for implicit connections - these should also force backgrounding
-    #       this would fix running Dan's data manager workflows via UI.
-    # TODO: ensure state if populated before calling force_queue from old API
-    #       workflow endpoint so the has_module check below is unneeded and these
-    #       interesting workflows will work with the older endpoint.
-    config = trans.app.config
-    force_for_collection = config.force_beta_workflow_scheduled_for_collections
-    force_min_steps = config.force_beta_workflow_scheduled_min_steps
-
-    step_count = len(workflow.steps)
-    if step_count > force_min_steps:
-        log.info("Workflow has many steps %d, backgrounding execution" % step_count)
-        return True
-    for step in workflow.steps:
-        # State and module haven't been populated if workflow submitted via
-        # the API. API requests for "interesting" workflows should use newer
-        # endpoint that skips this check entirely - POST /api/workflows/<id>/invocations
-        has_module = hasattr(step, "module")
-        if step.type not in BASIC_WORKFLOW_STEP_TYPES:
-            log.info("Found non-basic workflow step type - backgrounding execution")
-            # Force all new beta modules types to be use force queueing of
-            # workflow.
-            return True
-        if step.type == "data_collection_input" and force_for_collection:
-            log.info("Found collection input step - backgrounding execution")
-            return True
-        if step.type == "tool" and has_module and step.module.tool.produces_collections_with_unknown_structure:
-            log.info("Found dynamically structured output collection - backgrounding execution")
-            return True
-    return False
-
-
 def __invoke(trans, workflow, workflow_run_config, workflow_invocation=None, populate_state=False):
     """ Run the supplied workflow in the supplied target_history.
     """
@@ -106,7 +58,8 @@ def __invoke(trans, workflow, workflow_run_config, workflow_invocation=None, pop
     return outputs, invoker.workflow_invocation
 
 
-def queue_invoke(trans, workflow, workflow_run_config, request_params={}, populate_state=True, flush=True):
+def queue_invoke(trans, workflow, workflow_run_config, request_params=None, populate_state=True, flush=True):
+    request_params = request_params or {}
     if populate_state:
         modules.populate_module_and_state(trans, workflow, workflow_run_config.param_map, allow_tool_state_corrections=workflow_run_config.allow_tool_state_corrections)
     workflow_invocation = workflow_run_config_to_request(trans, workflow_run_config, workflow)
@@ -158,7 +111,7 @@ class WorkflowInvoker:
         config = self.trans.app.config
         maximum_duration = getattr(config, "maximum_workflow_invocation_duration", -1)
         if maximum_duration > 0 and workflow_invocation.seconds_since_created > maximum_duration:
-            log.debug("Workflow invocation [{}] exceeded maximum number of seconds allowed for scheduling [{}], failing.".format(workflow_invocation.id, maximum_duration))
+            log.debug(f"Workflow invocation [{workflow_invocation.id}] exceeded maximum number of seconds allowed for scheduling [{maximum_duration}], failing.")
             workflow_invocation.state = model.WorkflowInvocation.states.FAILED
             # All jobs ran successfully, so we can save now
             self.trans.sa_session.add(workflow_invocation)
@@ -173,7 +126,12 @@ class WorkflowInvoker:
 
         remaining_steps = self.progress.remaining_steps()
         delayed_steps = False
+        max_jobs_per_iteration_reached = False
         for (step, workflow_invocation_step) in remaining_steps:
+            max_jobs_to_schedule = self.progress.maximum_jobs_to_schedule_or_none
+            if max_jobs_to_schedule is not None and max_jobs_to_schedule <= 0:
+                max_jobs_per_iteration_reached = True
+                break
             step_delayed = False
             step_timer = ExecutionTimer()
             try:
@@ -206,9 +164,9 @@ class WorkflowInvoker:
                 raise
 
             if not step_delayed:
-                log.debug("Workflow step {} of invocation {} invoked {}".format(step.id, workflow_invocation.id, step_timer))
+                log.debug(f"Workflow step {step.id} of invocation {workflow_invocation.id} invoked {step_timer}")
 
-        if delayed_steps:
+        if delayed_steps or max_jobs_per_iteration_reached:
             state = model.WorkflowInvocation.states.READY
         else:
             state = model.WorkflowInvocation.states.SCHEDULED
@@ -236,28 +194,22 @@ class WorkflowInvoker:
 
         # No steps created yet - have to delay evaluation.
         if not step_invocation:
-            delayed_why = "depends on step [%s] but that step has not been invoked yet" % output_id
+            delayed_why = f"depends on step [{output_id}] but that step has not been invoked yet"
             raise modules.DelayedWorkflowEvaluation(why=delayed_why)
 
         if step_invocation.state != 'scheduled':
-            delayed_why = "depends on step [%s] job has not finished scheduling yet" % output_id
+            delayed_why = f"depends on step [{output_id}] job has not finished scheduling yet"
             raise modules.DelayedWorkflowEvaluation(delayed_why)
 
-        for job_assoc in step_invocation.jobs:
-            job = job_assoc.job
-            if job:
-                # At least one job in incomplete.
-                if not job.finished:
-                    delayed_why = "depends on step [%s] but one or more jobs created from that step have not finished yet" % output_id
-                    raise modules.DelayedWorkflowEvaluation(why=delayed_why)
+        # TODO: Handle implicit dependency on stuff like pause steps.
+        for job in step_invocation.jobs:
+            # At least one job in incomplete.
+            if not job.finished:
+                delayed_why = f"depends on step [{output_id}] but one or more jobs created from that step have not finished yet"
+                raise modules.DelayedWorkflowEvaluation(why=delayed_why)
 
-                if job.state != job.states.OK:
-                    raise modules.CancelWorkflowEvaluation()
-
-            else:
-                # TODO: Handle implicit dependency on stuff like
-                # pause steps.
-                pass
+            if job.state != job.states.OK:
+                raise modules.CancelWorkflowEvaluation()
 
     def _invoke_step(self, invocation_step):
         incomplete_or_none = invocation_step.workflow_step.module.execute(self.trans,
@@ -273,7 +225,7 @@ STEP_OUTPUT_DELAYED = object()
 class WorkflowProgress:
 
     def __init__(self, workflow_invocation, inputs_by_step_id, module_injector, param_map, jobs_per_scheduling_iteration=-1):
-        self.outputs = OrderedDict()
+        self.outputs = {}
         self.module_injector = module_injector
         self.workflow_invocation = workflow_invocation
         self.inputs_by_step_id = inputs_by_step_id
@@ -319,19 +271,27 @@ class WorkflowProgress:
         return remaining_steps
 
     def replacement_for_input(self, step, input_dict):
-        replacement = modules.NO_REPLACEMENT
+        replacement: Union[
+            modules.NoReplacement,
+            model.DatasetCollectionInstance,
+            List[model.DatasetCollectionInstance],
+        ] = modules.NO_REPLACEMENT
         prefixed_name = input_dict["name"]
         multiple = input_dict["multiple"]
         if prefixed_name in step.input_connections_by_name:
             connection = step.input_connections_by_name[prefixed_name]
             if input_dict["input_type"] == "dataset" and multiple:
-                replacement = [self.replacement_for_connection(c) for c in connection]
+                temp = [self.replacement_for_connection(c) for c in connection]
                 # If replacement is just one dataset collection, replace tool
                 # input_dict with dataset collection - tool framework will extract
                 # datasets properly.
-                if len(replacement) == 1:
-                    if isinstance(replacement[0], model.HistoryDatasetCollectionAssociation):
-                        replacement = replacement[0]
+                if len(temp) == 1:
+                    if isinstance(temp[0], model.HistoryDatasetCollectionAssociation):
+                        replacement = temp[0]
+                    else:
+                        replacement = temp
+                else:
+                    replacement = temp
             else:
                 is_data = input_dict["input_type"] in ["dataset", "dataset_collection"]
                 replacement = self.replacement_for_connection(connection[0], is_data=is_data)
@@ -341,27 +301,20 @@ class WorkflowProgress:
     def replacement_for_connection(self, connection, is_data=True):
         output_step_id = connection.output_step.id
         if output_step_id not in self.outputs:
-            template = "No outputs found for step id %s, outputs are %s"
-            message = template % (output_step_id, self.outputs)
+            message = f"No outputs found for step id {output_step_id}, outputs are {self.outputs}"
             raise Exception(message)
         step_outputs = self.outputs[output_step_id]
         if step_outputs is STEP_OUTPUT_DELAYED:
-            delayed_why = "dependent step [%s] delayed, so this step must be delayed" % output_step_id
+            delayed_why = f"dependent step [{output_step_id}] delayed, so this step must be delayed"
             raise modules.DelayedWorkflowEvaluation(why=delayed_why)
         output_name = connection.output_name
         try:
             replacement = step_outputs[output_name]
         except KeyError:
-            replacement = self.inputs_by_step_id.get(output_step_id)
-            if connection.output_step.type == 'parameter_input' and output_step_id is not None:
-                # FIXME: parameter_input step outputs should be properly recorded as step outputs, but for now we can
-                # short-circuit and just pick the input value
-                pass
-            else:
-                # Must resolve.
-                template = "Workflow evaluation problem - failed to find output_name %s in step_outputs %s"
-                message = template % (output_name, step_outputs)
-                raise Exception(message)
+            # Must resolve.
+            template = "Workflow evaluation problem - failed to find output_name %s in step_outputs %s"
+            message = template % (output_name, step_outputs)
+            raise Exception(message)
         if isinstance(replacement, model.HistoryDatasetCollectionAssociation):
             if not replacement.collection.populated:
                 if not replacement.waiting_for_elements:
@@ -371,11 +324,12 @@ class WorkflowProgress:
                     # TODO: consider distinguish between cancelled and failed?
                     raise modules.CancelWorkflowEvaluation()
 
-                delayed_why = "dependent collection [%s] not yet populated with datasets" % replacement.id
+                delayed_why = f"dependent collection [{replacement.id}] not yet populated with datasets"
                 raise modules.DelayedWorkflowEvaluation(why=delayed_why)
 
-        data_inputs = (model.HistoryDatasetAssociation, model.HistoryDatasetCollectionAssociation, model.DatasetCollection)
-        if not is_data and isinstance(replacement, data_inputs):
+        if isinstance(replacement, model.DatasetCollection):
+            raise NotImplementedError
+        if not is_data and isinstance(replacement, (model.HistoryDatasetAssociation, model.HistoryDatasetCollectionAssociation)):
             if isinstance(replacement, model.HistoryDatasetAssociation):
                 if replacement.is_pending:
                     raise modules.DelayedWorkflowEvaluation()
@@ -400,7 +354,7 @@ class WorkflowProgress:
         output_name = workflow_output.output_name
         step_outputs = self.outputs[step.id]
         if step_outputs is STEP_OUTPUT_DELAYED:
-            delayed_why = "depends on workflow output [%s] but that output has not been created yet" % output_name
+            delayed_why = f"depends on workflow output [{output_name}] but that output has not been created yet"
             raise modules.DelayedWorkflowEvaluation(why=delayed_why)
         else:
             return step_outputs[output_name]
@@ -428,21 +382,25 @@ class WorkflowProgress:
 
     def set_step_outputs(self, invocation_step, outputs, already_persisted=False):
         step = invocation_step.workflow_step
+        if invocation_step.output_value:
+            outputs[invocation_step.output_value.workflow_output.output_name] = invocation_step.output_value.value
         self.outputs[step.id] = outputs
         if not already_persisted:
+            workflow_outputs_by_name = {wo.output_name: wo for wo in step.workflow_outputs}
             for output_name, output_object in outputs.items():
                 if hasattr(output_object, "history_content_type"):
                     invocation_step.add_output(output_name, output_object)
                 else:
-                    # This is a problem, this non-data, non-collection output
-                    # won't be recovered on a subsequent workflow scheduling
-                    # iteration. This seems to have been a pre-existing problem
-                    # prior to #4584 though.
-                    pass
+                    # Add this non-data, non workflow-output output to the workflow outputs.
+                    # This is required for recovering the output in the next scheduling iteration,
+                    # and should be replaced with a WorkflowInvocationStepOutputValue ASAP.
+                    if not workflow_outputs_by_name.get(output_name) and not output_object == modules.NO_REPLACEMENT:
+                        workflow_output = model.WorkflowOutput(step, output_name=output_name)
+                        step.workflow_outputs.append(workflow_output)
             for workflow_output in step.workflow_outputs:
                 output_name = workflow_output.output_name
                 if output_name not in outputs:
-                    message = "Failed to find expected workflow output [{}] in step outputs [{}]".format(output_name, outputs)
+                    message = f"Failed to find expected workflow output [{output_name}] in step outputs [{outputs}]"
                     # raise KeyError(message)
                     # Pre-18.01 we would have never even detected this output wasn't configured
                     # and even in 18.01 we don't have a way to tell the user something bad is
@@ -463,7 +421,7 @@ class WorkflowProgress:
 
     def mark_step_outputs_delayed(self, step, why=None):
         if why:
-            message = "Marking step {} outputs of invocation {} delayed ({})".format(step.id, self.workflow_invocation.id, why)
+            message = f"Marking step {step.id} outputs of invocation {self.workflow_invocation.id} delayed ({why})"
             log.debug(message)
         self.outputs[step.id] = STEP_OUTPUT_DELAYED
 
@@ -471,7 +429,7 @@ class WorkflowProgress:
         workflow_invocation = self.workflow_invocation
         subworkflow_invocation = workflow_invocation.get_subworkflow_invocation_for_step(step)
         if subworkflow_invocation is None:
-            raise Exception("Failed to find persisted workflow invocation for step [%s]" % step.id)
+            raise Exception(f"Failed to find persisted workflow invocation for step [{step.id}]")
         return subworkflow_invocation
 
     def subworkflow_invoker(self, trans, step, use_cached_job=False):
@@ -520,4 +478,4 @@ class WorkflowProgress:
             self.mark_step_outputs_delayed(step_invocation.workflow_step, de.why)
 
 
-__all__ = ('invoke', 'WorkflowRunConfig')
+__all__ = ('queue_invoke', 'WorkflowRunConfig')
